/* 
This file is part of the BIAS library (Basic ImageAlgorithmS).

Copyright (C) 2003, 2004    (see file CONTACTS for details)
  Multimediale Systeme der Informationsverarbeitung
  Institut fuer Informatik
  Christian-Albrechts-Universitaet Kiel


BIAS is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or
(at your option) any later version.

BIAS is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with BIAS; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include <Base/Common/BaseFactory.hh>

#include <string>
#include <iostream>

using namespace BIAS;
using namespace std;

/* @author woelk 02/2006 */
class Base 
{
public:
  Base(const string& name)
    : Name_(name) {}
  inline std::string GetTag() const { return Name_; }
  void Who() { cout << "I am "<<Name_<<endl; }
  std::string Name_;
};

/* @author woelk 02/2006 */
class A : public Base
{
public:
  A() : Base("a") {};
  ALLOCATOR_IMPL(Base, A);
};

/* @author woelk 02/2006 */
class B : public Base
{
public:
  B() : Base("b") {};
  ALLOCATOR_IMPL(Base, B);
};

/* @author woelk 02/2006 */
class C : public Base
{
public:
  C() : Base("c") {};
  ALLOCATOR_IMPL(Base, C);
};


/* @author woelk 02/2006 */
class MyFactory
  : public BaseFactory<Base>
{
public:
  MyFactory()
    : BaseFactory<Base>()
  {
    AddType((&A::Allocator));
    AddType((&B::Allocator));
    AddType((&C::Allocator));
  }
};

/* @brief explains the usage of the Factory class
   @author woelk 02/2006 */
int main()
{
  // main knows nothing of A, B and C
  // all knowledge is in factory and henec D coudl be introduced without
  // changing main
  MyFactory fac;
  Base *p = fac.Generate("a");
  p->Who();
  p = fac.Generate("b");
  p->Who();
  p = fac.Generate("c");
  p->Who();
  p = fac.Generate("d");
  if (p!=NULL){
    p->Who();
  } else {
    cerr << "could not generate class with name \"d\""<<endl;
  }

  return 0;
}
