/*
This file is part of the BIAS library (Basic ImageAlgorithmS).

Copyright (C) 2003, 2004    (see file CONTACTS for details)
  Multimediale Systeme der Informationsverarbeitung
  Institut fuer Informatik
  Christian-Albrechts-Universitaet Kiel


BIAS is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or
(at your option) any later version.

BIAS is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with BIAS; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#ifndef __FileHandling_hh__
#define __FileHandling_hh__

/** File handling tools
@todo: correct handling of Win32 SAMBA paths including \\machine\....
@author Jan Woetzel 02/2005
*/

#include <bias_config.h>

#include <string>

#define DEFAULT_LEADING_ZEROS 5


namespace BIAS {

  /* @return true if the file exists can be pendedis readable on file system */

  bool BIASCommon_EXPORT 
    FileReadable( const std::string & s);


  /**  split full path into:
  - directory
  - basename
  - .suffix (inlcluding dot) 
  @todo: Extract machine name or IP for SMB network shares
  for WIN32, e.g. \\machine\share and fix backslashes on  Win32.
  @author Jan Woetzel 03/2005 */
  // TODO: \todo WIN: fix to make \, \\ and / work on directories (jw)
  void BIASCommon_EXPORT
    SplitName(const std::string & fullname
    ,std::string & dir
    ,std::string & base
    ,std::string & suffix );

  /** get Directory from fullname path JW */
  std::string BIASCommon_EXPORT
    Directory(const std::string & fullname);

  /**  get Basename of file, no suffix, no direcory. */
  std::string  BIASCommon_EXPORT
    Basename(const std::string & fullname);

  /**  get .suffix of file including the ".dot"
  No basename, no direcory. */
  std::string  BIASCommon_EXPORT
    Suffix(const std::string & fullname);

  /**  get Extension of file without the "dot"
  No basename, no direcory. */
  std::string  BIASCommon_EXPORT
    Extension(const std::string & fullname);


  /// type correct helper wrapper for tolower required for stl transformations JW
  char ToLower_c(char c);
  /// type correct helper wrapper for toupper required for stl transformations JW
  char ToUpper_c(char c);


  /// @return lower case string of input JW
  std::string BIASCommon_EXPORT LowerCase(const std::string & input);

  /// @return upper case string of input JW
  std::string BIASCommon_EXPORT UpperCase(const std::string & input);

  /** write compilation time/date and all command line arguments 
  to a log file/stream JW */
  void BIASCommon_EXPORT 
    LogCommandLine( int argc,char** argv, std::ostream & os);

  void BIASCommon_EXPORT 
    LogCommandLine( int argc,char** argv, const std::string & filename);

  /** Creates a string with leading zeroes from number JW */
  std::string BIASCommon_EXPORT
    LeadingZeroString( const int & n, 
    const unsigned int & digits=DEFAULT_LEADING_ZEROS ) ;

  /** @brief create a string containing date and time like 20060329_173054
      @author koeser */
  std::string  BIASCommon_EXPORT CurrentDateTimeString();

  /** @brief extract numeric value fromstring beginnign from right, 
  e.g. "blah042foo.txt" --> 42 and the pos. of the 0 and 2.
  Useful helper for FilenameSequence JW */
  int BIASCommon_EXPORT ExtractNumberFromRight(const std::string & inStr,
    int & number,
    int & posL,
    int & posR );


  /** @brief construct i,g .seq filename from a given filename 
  by manipulating the number within base part of RefFilename.
  of an image sequence given a ref. filename and an offset.
  offset is the number of frames forward or backward from ref. image.
  stepsize is teh number of increment/decrement per offset. 
  e.g ref=100, offset=2, stepsize=7 --> 114
  @return 0 for success, other for failure.
  @author Jan Woetzel */
  int BIASCommon_EXPORT FilenameSequence(const std::string & refFileName,
    std::string & resultFilename,
    const int & offsetFromRef,
    const unsigned int & stepsize=1 );

} // namespace BIAS
#endif // do not add code after this line 
