#ifndef _Exception_hh_
#define _Exception_hh_

#include <exception>
#include <sstream>
#include <iostream>

#ifdef WIN32
/** disable vc++ .net warning 4290 stating that exception types are not 
    checked */
/// \todo never disabled warnign in headers because of teh global side effect (jw)
#  pragma warning( disable : 4290 )
#endif

namespace BIAS {
    
  /** macro throwing a BaseException using stream as argument,
      i.e. BEXCEPTION("variable 'a' has content "<<a) */
#define BEXCEPTION(arg)\
  { \
    std::ostringstream os;\
    os << __FILE__ << ":" << __LINE__ <<" : "<< arg;\
    throw BIAS::BaseException(os.str().c_str());\
  }

  /** @class BaseException
      @brief generic exception  
      @author woelk 01/2006 */
  class BaseException
    : public std::exception
  {
  public:
    BaseException(const char *error_message) throw()
    { 
      Message_ = error_message; 
      std::cerr << "exception raised: \""<<Message_<<"\"\n";
    } 

    ~BaseException() throw() {};

    /** Returns a C-style character string describing the general cause
        of the current error.  */
    virtual const char* What() const throw()
    { return Message_.c_str(); }

  protected:
    std::string Message_;
  };

} // namespace

#endif // _Exception_hh_
