/* 
This file is part of the BIAS library (Basic ImageAlgorithmS).

Copyright (C) 2003, 2004    (see file CONTACTS for details)
  Multimediale Systeme der Informationsverarbeitung
  Institut fuer Informatik
  Christian-Albrechts-Universitaet Kiel


BIAS is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or
(at your option) any later version.

BIAS is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with BIAS; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifndef __PlueckerMatrix_hh__
#define __PlueckerMatrix_hh__
#include "bias_config.h" 

#include <Base/Geometry/HomgPoint3D.hh>
#include <Base/Math/Vector3.hh>
#include <Base/Math/Vector2.hh>
#include <Base/Math/Vector.hh>
#include <Base/Math/Matrix.hh>
#include <Base/Math/Matrix4x4.hh>

#define PLUECKERMATRIX_TYPE double

namespace BIAS{

/** @class PlueckerMatrix
    @brief represente a 3d line using a 4x4 Pluecker matrix
    @author woelk 03/2006 */

  class PlueckerMatrix
    : public Matrix4x4<PLUECKERMATRIX_TYPE>
  {
  public:
    inline PlueckerMatrix() :  Matrix4x4<PLUECKERMATRIX_TYPE>(MatrixZero) {}

    /** @brief cast constructor, checks special constraints */
    PlueckerMatrix(const BIAS::Matrix4x4<PLUECKERMATRIX_TYPE>& m);
      
    /** @brief construction of a line through 2 points */
    inline PlueckerMatrix(const HomgPoint3D& p1, const HomgPoint3D& p2)
    { Set(p1, p2); }

    virtual ~PlueckerMatrix() {};
    
    /** @brief construction of a line through 2 points */
    void Set(const HomgPoint3D& p1, const HomgPoint3D& p2);

    Vector<PLUECKERMATRIX_TYPE> GetPlueckerLineCoordinates() const;

    /** @brief direction vetor */
    HomgPoint3D GetDirection() const;

    /** @brief normal vector to the line */
    HomgPoint3D GetNormal() const;

  protected:
    /** @brief checks for the rank two constraint, i.e. f the direction 
        and the normal vector are perpendicular to each other */
    bool CheckConstraint_() const;
  };
  
  
} // namespace BIAS


#endif // __PlueckerMatrix_hh__
