/* 
This file is part of the BIAS library (Basic ImageAlgorithmS).

Copyright (C) 2003, 2004    (see file CONTACTS for details)
  Multimediale Systeme der Informationsverarbeitung
  Institut fuer Informatik
  Christian-Albrechts-Universitaet Kiel


BIAS is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or
(at your option) any later version.

BIAS is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with BIAS; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


#include "Quaternion.hh"


using namespace BIAS;
using namespace std;


template<class QUAT_TYPE> 
Quaternion<QUAT_TYPE>::~Quaternion() 
{}

template<class QUAT_TYPE> 
Quaternion<QUAT_TYPE>::Quaternion() 
: Vector4<QUAT_TYPE>() 
{}


template<class QUAT_TYPE> 
Quaternion<QUAT_TYPE>::Quaternion(QUAT_TYPE i, QUAT_TYPE j,
                                  QUAT_TYPE k, QUAT_TYPE r) 
                                  : Vector4<QUAT_TYPE>(i, j, k, r)
{}

template<class QUAT_TYPE> 
int Quaternion<QUAT_TYPE>::GetAxisAngle(Vector3<QUAT_TYPE>& axis, 
                                        QUAT_TYPE& angle) const
{
  // nicked from OpenSG
  int res=0;
  Vector3<QUAT_TYPE> q((*this)[0], (*this)[1], (*this)[2]);
  QUAT_TYPE len = QUAT_TYPE(q.NormL2());
  
  if(len > QUATERNION_EPSILON) {
    q /= len;
    axis[0]  = q[0];
    axis[1]  = q[1];
    axis[2]  = q[2];
    angle = QUAT_TYPE(2.0 * acos((*this)[3]));
  } else {
    axis[0] = 0.0;
    axis[1] = 0.0;
    axis[2] = 1.0;
    angle = 0.0;
  }
  return res;
}

template<class QUAT_TYPE> 
int Quaternion<QUAT_TYPE>::GetRotationMatrix(BIAS::RMatrixBase& /*R*/) const
{
  BIASERR("Quaternion<QUAT_TYPE>::GetRotationMatrix() is deprecated, because"
          <<" it returned a TRANSPOSED matrix. To prevent code duplication "
          <<" this function will be removed in the future. Check your code."
          <<"Use RMatrixBase::SetFromQuaternion instead.");
  BIASABORT;
  return 1;
}

template<class QUAT_TYPE>
int Quaternion<QUAT_TYPE>::
SetXYZ(QUAT_TYPE radX, QUAT_TYPE radY, QUAT_TYPE radZ)
{
  Quaternion<QUAT_TYPE> q_x;
  Quaternion<QUAT_TYPE> q_y;
  Quaternion<QUAT_TYPE> q_z;

  q_x.SetValueAsAxisRad( 1.,0.,0.,  radX );
  q_y.SetValueAsAxisRad( 0.,1.,0.,  radY );
  q_z.SetValueAsAxisRad( 0.,0.,1.,  radZ );

  q_z.Mult(q_y);
  q_z.Mult(q_x);

  (*this) = q_z;

  return 0;
}

template<class QUAT_TYPE>
int Quaternion<QUAT_TYPE>::
SetZYX(QUAT_TYPE radX, QUAT_TYPE radY, QUAT_TYPE radZ)
{
  Quaternion<QUAT_TYPE> q_x;
  Quaternion<QUAT_TYPE> q_y;
  Quaternion<QUAT_TYPE> q_z;

  q_x.SetValueAsAxisRad( 1.,0.,0.,  radX );
  q_y.SetValueAsAxisRad( 0.,1.,0.,  radY );
  q_z.SetValueAsAxisRad( 0.,0.,1.,  radZ );

  q_x.Mult(q_y);
  q_x.Mult(q_z);

  (*this) = q_x;

  return 0;
}


template<class QUAT_TYPE>
Quaternion<QUAT_TYPE>& Quaternion<QUAT_TYPE>::operator=(const Quaternion<QUAT_TYPE>& vec) 
{
  memcpy((void *)this->data_, (void *)vec.GetData(), 
    VECTOR4SIZE*sizeof(QUAT_TYPE)); 
  return *this; 
}


// instances:
// for Quaternions only these make sense
template class BIASGeometryBase_EXPORT BIAS::Quaternion<float>;
template class BIASGeometryBase_EXPORT BIAS::Quaternion<double>;
