#include "ImageConvertThreaded.hh"

#include <Base/Debug/Exception.hh>

//TODO pthread DLL is linked into this DLL!
//#ifdef BIASImageBase_EXPORTS
//#  define PTW32_BUILD
//#  define _DLL
//#endif // BIASImageBase_EXPORTS
#include <pthread.h>



using namespace BIAS;
using namespace std;

BIAS::ImageConvertThreaded::
ImageConvertThreaded()
{}


BIAS::ImageConvertThreaded::
~ImageConvertThreaded()
{}


struct ThreadArg__ {
  ImageBase *src, *dst;
  ImageBase::EColorModel colormodel;
  bool planar;
};

int BIAS::ImageConvertThreaded::
ConvertThreaded(BIAS::ImageBase &source, BIAS::ImageBase &dest, 
                enum BIAS::ImageBase::EColorModel targetColorModel,
                bool bPlanar, const int num_threads)
{
  vector<ImageBase> src(num_threads), dst(num_threads);
  vector<ThreadArg__> arg(num_threads);
  vector<pthread_t> thread(num_threads);

  SplitImage_(source, num_threads, src);

  for (int i=0; i<num_threads; i++){
    arg[i].src = &(src[i]);
    arg[i].dst = &(dst[i]);
    arg[i].colormodel = targetColorModel;
    arg[i].planar = bPlanar;
    if (pthread_create(&(thread[i]), NULL, ConvertThread_, &(arg[i]))!=0){
      BEXCEPTION("error creating thread "<<i);
    }
  }

  int overall_result = 0;
  vector<int> res(num_threads);
  void *result=NULL;
  for (int i=0; i<num_threads; i++){
    if (pthread_join(thread[i], &result)!=0){
      BEXCEPTION("error joining thread "<<i);
    }
    res[i] = *(int *)result;
    if (res[i]!=0) { overall_result--; }
    delete (int *)result; 
    result = NULL;
  }

  int tlx, tly, brx, bry;
  source.GetROI()->GetCorners(tlx, tly, brx, bry);
  if (!source.SamePixelAndChannelCount(dest)){
    if (!dest.IsEmpty()){ dest.Release(); }
  }
  if (dest.IsEmpty()){ 
    dest.Init(source.GetWidth(), source.GetHeight(), dst[0].GetChannelCount(),
              dst[0].GetStorageType(), dst[0].IsInterleaved());
  }
  dest.SetColorModel(dst[0].GetColorModel());
  dest.GetROI()->SetCorners(tlx, tly, brx, bry);
  MergeImage_(dst, dest);

  return overall_result;
}

void *BIAS::ImageConvertThreaded::
ConvertThread_(void *arg)
{
  ThreadArg__ *targ = static_cast<ThreadArg__ *>(arg);
  int *res = new int;
  *res = ImageConvert::Convert(*(targ->src), *(targ->dst), 
                               targ->colormodel, targ->planar);
  return static_cast<void *>(res);
}

void BIAS::ImageConvertThreaded::
SplitImage_(const ImageBase &source, const int num,
            std::vector<ImageBase> &parts)
{
  if ((int)parts.size()!=num){
    parts.resize(num);
  }
  ImageBase src(source); // de-const
  switch (num){
  case 2:
    int tlx, tly, brx, bry;
    src.GetROI()->GetCorners(tlx, tly, brx, bry);
    src.GetROI()->SetCorners(tlx, tly, brx, bry/2);
    if (src.GetCopyOfROI(parts[0])!=0){
      BEXCEPTION("error copying ROI");
    }
    src.GetROI()->SetCorners(tlx, bry/2, brx, bry);
    if (src.GetCopyOfROI(parts[1])!=0){
      BEXCEPTION("error copying ROI");
    }
    break;
  default:
    BEXCEPTION("only 2 threads are supported");
    break;
  }
}

void BIAS::ImageConvertThreaded::
MergeImage_(const std::vector<ImageBase>& parts,
            ImageBase& dst)
{
  switch (parts.size()){
  case 2:
    int tlx, tly, brx, bry;
    dst.GetROI()->GetCorners(tlx, tly, brx, bry);
    dst.GetROI()->SetCorners(tlx, tly, brx, bry/2);
    dst.Paste2ROI(parts[0]);
    dst.GetROI()->SetCorners(tlx, bry/2, brx, bry);
    dst.Paste2ROI(parts[1]);
    dst.GetROI()->SetCorners(tlx, tly, brx, bry);
    break;
  default:
    BEXCEPTION("only 2 threads are supported");
    break;
  }
}

