/* 
This file is part of the BIAS library (Basic ImageAlgorithmS).

Copyright (C) 2003, 2004    (see file CONTACTS for details)
  Multimediale Systeme der Informationsverarbeitung
  Institut fuer Informatik
  Christian-Albrechts-Universitaet Kiel


BIAS is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or
(at your option) any later version.

BIAS is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with BIAS; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


#include "Random.hh"

#ifdef WIN32
// implement non-standard (double)rint(double x)
#  define rint(x) (floor(x+0.5))
#endif //_WIN32

#include <cmath>
#include <cstdlib>
#include <stdlib.h>
#include <float.h>

using namespace BIAS;
using namespace std;

//#define RANDOM_SEED_ZERO_FIXED 

// #ifdef RANDOM_SEED_ZERO_FIXED
// #include <fstream>
// namespace BIAS { 
//   int rand(void){
//     static std::ofstream os("random.txt");
//     if (!os.good()){ BIASERR("error writing to file "); BIASABORT; }
//     static int num = 0;
//     int ran = ::rand();
//     cout << setw(5) << num << setw(15) << ran << endl << flush;
//     os << setw(5) << num << setw(15) << ran << endl << flush;
//     num++;
//     return ran;
//   }
// }
//#endif

Random::~Random()
{}


Random::Random()
  : Seed_(0), Idum_(0), LongIdum_(0)
{
  // to avoid nondeterministics by only instantiating a new object
  // never call srand() here
  // evers 2005-06-30
  //  Reset();
#ifdef RANDOM_SEED_ZERO_FIXED 
  Reset(0);
#endif
}

Random::Random(long int seed)
  : Seed_(0), Idum_(0), LongIdum_(0)
{
  Reset(seed);
}

static bool first = true;

void Random::Reset(long int seed)
{
#ifdef RANDOM_SEED_ZERO_FIXED 
  if (!first){
    return;
  }
  first = false;
  Seed_ = 0;
#else
  first=first; // avoid warning
  Seed_ = seed;
#endif
  //cout << "srand("<<Seed_<<")\n";
  // initialize random source
  srand((long int)Seed_);
  // initialize seed for ran1_
  Idum_=rand();
  LongIdum_ = (long)Idum_;
  //cerr << "Idum_: " << Idum_ << "\tLongIdum_: "<<LongIdum_<<endl;
}

void Random::Reset()
{
#ifdef RANDOM_SEED_ZERO_FIXED 
  if (!first){
    return;
  }
  first = false;
  Seed_ = 0;
#else
  	//long int uc;
  time_t uc;

  // get the seed (seconds since 1.1.1970 0:00)
  time(&uc);
  Seed_=(unsigned int)(uc);
#endif 
  //cout << "srand("<<Seed_<<")\n";
  // initialize random source
  srand( Seed_ );
  // initialize seed for ran1_
  Idum_=rand();
  LongIdum_ = (long)Idum_;
  //cerr << "Idum_: " << Idum_ << "\tLongIdum_: "<<LongIdum_<<endl;
}

int Random::GetUniformDistributedInt(int min, int max)
{
  return int(min+(double(rand())/(RAND_MAX)*(max-min+1)));
  /*
  double bin=(double)(1+max-min);
  double res = GetUniformDistributed(0.0, 1.0) * bin;
  //cerr <<res/bin << " " <<res<<" "<< (int)floor(res)+min << endl;
  return (int)floor(res)+min;
*/
}

unsigned int 
Random::GetUniformDistributedInt(unsigned int min, unsigned int max)
{
  double bin=(double)(1+max-min);
  double res = GetUniformDistributed(0.0, 1.0) * bin;
  //cerr << setprecision(14) << res<<" "<<(unsigned int)floor(res)+min << endl;
  return (unsigned int)floor(res)+min;
}


void Random::GetUniformDistributed(double min, double max, 
                                   int num, double *res)
{
  double size=max-min;
  for (int i=0; i<num; i++)
    res[i]=min + ran2_(&LongIdum_) * size;
}

void Random::GetNormalDistributed(double mean, double sigma, 
                                  int num, double *res)
{
  for (int i=0; i<num; i++)
    res[i]=mean + gasdevf_(&LongIdum_) * sigma;
}



int Random::GetNormalDistributed(const Vector2<double>& mean, 
                                 const Matrix2x2<double>& cov,
                                 Vector2<double>& ran)
{
  int res=0;

  vector<Vector2<double> > vran;
  res = GetNormalDistributed(mean, cov, 1, vran);

  if (res==0) ran = vran[1];

  return res;
}

int Random::GetNormalDistributed(const Vector2<double>& mean, 
                                 const Matrix2x2<double>& cov,
                                 int num, vector<Vector2<double> >& ran)
{
  int res=0;
  Vector2<double> eigen_vec1, eigen_vec2, tmp_ran;
  double eigen_val1, eigen_val2;
  int eigen_res = 
    cov.EigenvalueDecomposition(eigen_val1,eigen_vec1, eigen_val2, eigen_vec2);
  
  //cout << "eignevalues of covariance "<<eigen_val1<<", "<<eigen_val2<<endl;
  //cout << "eignevectors of covariance "<<eigen_vec1<<", "<<eigen_vec2<<endl;
  
  Matrix2x2<double> R;
  R[0][0] = eigen_vec1[0];
  R[1][0] = eigen_vec1[1];
  R[0][1] = eigen_vec2[0];
  R[1][1] = eigen_vec2[1];
  //cout << "R: "<<R<<endl;

  double *ran1, *ran2;
  ran1 = new double[num];
  ran2 = new double[num];

  switch (eigen_res){
  case 2:
    //cout << "2D distribution" << endl;
    GetNormalDistributed(0.0, sqrt(eigen_val1), num, ran1);
    GetNormalDistributed(0.0, sqrt(eigen_val2), num, ran2);
    break;
  case 1:
    //cout << "pseudo 2D distribution = 1D dsitribution" << endl;
    GetNormalDistributed(0.0, sqrt(eigen_val1), num, ran1);
    memset((void *)ran2, 0, sizeof(double)*num);
    break;
  case 0:
    BIASERR("not a covariance matrix ("<<eigen_res<<") "<<cov<<endl);
    //BIASASSERT(false);
    BIASABORT;
    //res = -1;
    break;
  default:
    res = eigen_res;
    break;
  }

  if (res>=0){
    ran.resize(num);
    vector<Vector2<double> >::iterator it;
    for (int i=ran.size()-1; i>=0; i--){
      ran[i].Set(ran1[i], ran2[i]);
      //cout <<"---\nraw: "<< ran[i]<<endl;
      ran[i] = R * ran[i];
      //cout <<"rot: "<< ran[i]<<endl;
      ran[i] += mean;
      //cout <<"sht: "<< ran[i]<<endl;
      
    }
  }

  delete[] ran1;
  delete[] ran2;

  return res;
}



// from www.nr.com copied from pdf
#define IA 16807
#define IM 2147483647
#define AM (1.0/IM)
#define IQ 127773
#define IR 2836
#define NTAB 32
#define NDIV (1+(IM-1)/NTAB)
#define EPS DBL_EPSILON
#define RNMX (1.0-EPS)
double Random::ran1_(int *idum)
{
  int j;
  long k;
  static long iy=0;
  static long iv[NTAB];
  float temp;

  if (*idum <= 0 || !iy){
    if (-(*idum) < 1) *idum = 1;
    else *idum = -(*idum);
    for (j=NTAB+7; j>=0; j--){
      k=(*idum)/IQ;
      *idum=IA*(*idum-k*IQ)-IR*k;
      if (*idum <0) *idum += IM;
      if (j<NTAB) iv[j] = *idum;
    }
    iy=iv[0];
  }
  k=(*idum)/IQ;
  *idum=IA*(*idum-k*IQ)-IR*k;
  if (*idum < 0) *idum += IM;
  j=iy/NDIV;
  iy=iv[j];
  iv[j] = *idum;
  if ((temp=float(AM*iy)) > RNMX) return RNMX;
  else return temp;
}
#undef IA 
#undef IM 
#undef AM 
#undef IQ 
#undef IR 
#undef NTAB 
#undef NDIV 
#undef EPS 
#undef RNMX 

//from D_RAN1.c
#define M1 259200
#define IA1 7141
#define IC1 54773
#define RM1 (1.0/M1)
#define M2 134456
#define IA2 8121
#define IC2 28411
#define RM2 (1.0/M2)
#define M3 243000
#define IA3 4561
#define IC3 51349
double Random::ran1c_(int *idum)
{
  static long ix1,ix2,ix3;
  static double r[98];
  double temp;
  static int iff=0;
  int j;
  //void nrerror();

  if (*idum < 0 || iff == 0) {
    iff=1;
    ix1=(IC1-(*idum)) % M1;
    ix1=(IA1*ix1+IC1) % M1;
    ix2=ix1 % M2;
    ix1=(IA1*ix1+IC1) % M1;
    ix3=ix1 % M3;
    for (j=1;j<=97;j++) {
      ix1=(IA1*ix1+IC1) % M1;
      ix2=(IA2*ix2+IC2) % M2;
      r[j]=(ix1+ix2*RM2)*RM1;
    }
    *idum=1;
  }
  ix1=(IA1*ix1+IC1) % M1;
  ix2=(IA2*ix2+IC2) % M2;
  ix3=(IA3*ix3+IC3) % M3;
  j=(int)rint(1 + ((97.0*ix3)/M3));
  if (j > 97 || j < 1) BIASERR("RAN1: This cannot happen. "<<j);
  temp=r[j];
  r[j]=(ix1+ix2*RM2)*RM1;
  return temp;
}
#undef M1
#undef IA1
#undef IC1
#undef RM1
#undef M2
#undef IA2
#undef IC2
#undef RM2
#undef M3
#undef IA3
#undef IC3

// from D_RAN2.c
#define M 714025
#define IA 1366
#define IC 150889
double Random::ran2_(long *idum)
{
  //cout << "idum "<<*idum<<endl;
  static long iy=0, ir[98];
  static int iff=0;
  int j;

  BIASASSERT(idum!=NULL);
  if ((*idum) < 0 || iff == 0) 
  {
      iff=1;
      if (((*idum)=(IC-(*idum)) % M) < 0) *idum = -(*idum);
      for (j=1;j<=97;j++) {
          *idum=(IA*(*idum)+IC) % M;
          ir[j]=(*idum);
      }
      (*idum) = (IA*(*idum)+IC) % M;
      iy=(*idum);
  }
  j = (int)floor(1 + 97.0*iy / M);
  if (j > 97 || j < 1) BIASERR("RAN2: This cannot happen. "<<j);
  iy=ir[j];
  *idum=(IA*(*idum)+IC) % M;
  ir[j]=(*idum);
  return (double) iy/M;
}
#undef M
#undef IA
#undef IC

// from D_GASDEV.c
double Random::gasdev_(int *idum)
{
  static int iset=0;
  static double gset;
  double fac,r,v1,v2;

  if  (iset == 0) {
    do {
      v1=2.0*ran1_(idum)-1.0;
      v2=2.0*ran1_(idum)-1.0;
      r=v1*v1+v2*v2;
    } while (r >= 1.0);
    fac=sqrt(-2.0*log(r)/r);
    gset=v1*fac;
    iset=1;
    return v2*fac;
  } else {
    iset=0;
    return gset;
  }
}

double Random::gasdevf_(long *idum)
{
  static int iset=0;
  static double gset;
  double fac,r,v1,v2;

  if  (iset == 0) {
    do {
      v1=2.0*ran2_(idum)-1.0;
      v2=2.0*ran2_(idum)-1.0;
      r=v1*v1+v2*v2;
    } while (r >= 1.0);
    fac=sqrt(-2.0*log(r)/r);
    gset=v1*fac;
    iset=1;
    return v2*fac;
  } else {
    iset=0;
    return gset;
  }
}

// from makenois.c
double Random::gasdev3_(int *idum)
{
  static int iset=1;
  static double gset;
  double fac,r,v1,v2;

  if (iset) {
    do {
      v1=2.0*ran1_(idum)-1.0;
      v2=2.0*ran1_(idum)-1.0;
      r=v1*v1+v2*v2;
    } while (r >= 1.0);
    fac=sqrt(-2.0*log(r)/r);
    gset = v1*fac;
    iset = 0;
    return v2*fac;
  } else {
    iset = 1;
    return gset;
  }
}
