/* 
This file is part of the BIAS library (Basic ImageAlgorithmS).

Copyright (C) 2003, 2004    (see file CONTACTS for details)
  Multimediale Systeme der Informationsverarbeitung
  Institut fuer Informatik
  Christian-Albrechts-Universitaet Kiel


BIAS is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or
(at your option) any later version.

BIAS is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with BIAS; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "StructureTensor.hh"

#ifdef BIAS_DEBUG
#include <Base/Image/ImageIO.hh> 
#endif

#include <Base/Common/BIASpragma.hh>


  //#define TIME_MEASURE

#ifdef TIME_MEASURE
#include <Base/Debug/TimeMeasure.hh>
#endif

using namespace BIAS;
using namespace std;

//////////////////////////////////////////////////////////////////////////
//                 implementation
//////////////////////////////////////////////////////////////////////////

template <class InputStorageType, class OutputStorageType>
StructureTensor<InputStorageType, OutputStorageType>::
StructureTensor(): 
  FilterNTo3N<InputStorageType, OutputStorageType>(),
  _gsobel3x3(), 
  _ggauss(),
  _ggaussasymmetric()
{
  _grad=NULL;
  _HalfWinSize=new int;
  *_HalfWinSize=3;
  _GradientType=new int;
  *_GradientType = GT_Gauss;
  _InternalMemWidth = _InternalMemHeight = 0;
}

template <class InputStorageType, class OutputStorageType>
StructureTensor<InputStorageType, OutputStorageType>::
StructureTensor(Param& para, const std::string& postfix):
  FilterNTo3N<InputStorageType, OutputStorageType>(para, "-StructureTensor"+postfix),
  _gsobel3x3(para, "-StructureTensor"+postfix), 
  _ggauss(para, "-StructureTensor"+postfix),
  _ggaussasymmetric(para, "-StructureTensor"+postfix)
{
  _grad = NULL;
  _HalfWinSize = _GradientType = NULL;
  int grp = para.GetFreeGroupID();
  _AddParam(para, grp, "-StructureTensor"+postfix);
  if (para.IsUsedGroupID(grp)) para.SetGroupName(grp, "StructureTensor");
  _InternalMemWidth = _InternalMemHeight = 0;
}

template <class InputStorageType, class OutputStorageType>
StructureTensor<InputStorageType, OutputStorageType>::
StructureTensor(Param& para, const int grp, const std::string& postfix)
:   FilterNTo3N<InputStorageType, OutputStorageType>(para, grp, "-StructureTensor"+postfix),
    _gsobel3x3(para, grp, "-StructureTensor"+postfix), 
    _ggauss(para, grp, "-StructureTensor"+postfix),
    _ggaussasymmetric(para, grp, "-StructureTensor"+postfix)
{
  _grad=NULL;
  _HalfWinSize=_GradientType=NULL;
  _AddParam(para, grp, "-StructureTensor"+postfix);
  _InternalMemWidth=_InternalMemHeight=0;
}

template <class InputStorageType, class OutputStorageType>
StructureTensor<InputStorageType, OutputStorageType>::~StructureTensor()
{
  if (_ParamsAllocated){
    delete _HalfWinSize; _HalfWinSize = NULL;
    delete _GradientType; _GradientType = NULL;
  }
}

template <class InputStorageType, class OutputStorageType>
int StructureTensor<InputStorageType, OutputStorageType>::
Filter(const Image<InputStorageType>& src, Image<OutputStorageType>& dst)
{
  BIASCDOUT(D_FILTERBASE_CALLSTACK, "StructureTensor::Filter(src, dst)\n");
  BIASERR("unfinished");
  BIASABORT;
  return -10;
}

template <class InputStorageType, class OutputStorageType>
int StructureTensor<InputStorageType, OutputStorageType>::
FilterInt(const Image<InputStorageType>& src, Image<OutputStorageType>& dst)
{
  BIASCDOUT(D_FILTERBASE_CALLSTACK, "StructureTensor::FilterInt(src, dst)\n");
  BIASERR("unfinished");
  BIASABORT;
  return -10;
}

template <class InputStorageType, class OutputStorageType>
int StructureTensor<InputStorageType, OutputStorageType>::
FilterFloat(const Image<InputStorageType>& src, Image<OutputStorageType>& dst)
{
  BIASCDOUT(D_FILTERBASE_CALLSTACK, "StructureTensor::FilterFloat(src, dst)\n");
  BIASERR("unfinished");
  BIASABORT;
  return -10;
}

template <class InputStorageType, class OutputStorageType>
int StructureTensor<InputStorageType, OutputStorageType>::
Filter(const Image<InputStorageType>& src, 
       Image<OutputStorageType>& gxx, 
       Image<OutputStorageType>& gxy,
       Image<OutputStorageType>& gyy) 
{
  BIASCDOUT(D_FILTERBASE_CALLSTACK, "StructureTensor::Filter(src, d1, d2, d3)\n");  
  BIASERR("unfinished");
  BIASABORT;
  return -10;
}

template <class InputStorageType, class OutputStorageType>
int StructureTensor<InputStorageType, OutputStorageType>::
FilterInt(const Image<InputStorageType>& src, 
       Image<OutputStorageType>& gxx, 
       Image<OutputStorageType>& gxy,
       Image<OutputStorageType>& gyy) 
{
  BIASCDOUT(D_FILTERBASE_CALLSTACK, "StructureTensor::FilterInt(src, d1, d2, d3)\n");  
  BIASERR("unfinished");
  BIASABORT;
  return -10;
}

template <class InputStorageType, class OutputStorageType>
int StructureTensor<InputStorageType, OutputStorageType>::
FilterFloat(const Image<InputStorageType>& src, 
       Image<OutputStorageType>& gxx, 
       Image<OutputStorageType>& gxy,
       Image<OutputStorageType>& gyy) 
{
  BIASCDOUT(D_FILTERBASE_CALLSTACK, "StructureTensor::FilterFloat(src, d1, d2, d3)\n");  

  BIASERR("unfinished");
  BIASABORT;
  return -10;
}






















template <class InputStorageType, class OutputStorageType>
int StructureTensor<InputStorageType, OutputStorageType>::
CalcStructureTensor(const Image<InputStorageType>& image, 
                    Image<OutputStorageType>& sgxx, 
                    Image<OutputStorageType>& sgxy, 
                    Image<OutputStorageType>& sgyy) {
  _UpdateGrad();
  BIASASSERT(_grad);
  int gres=_grad->Filter(image, _gx, _gy);
#ifdef BIAS_DEBUG
  if (DebugLevelIsSet(D_ST_WRITE_DEBUG_IM)){
    if (ImageIO::Write("gx.mip", _gx)!=0){
      BIASERR("error writing gx");
    }
    if (ImageIO::Write("gy.mip", _gy)!=0){
      BIASERR("error writing gy");
    }
  }
#endif
  
  int sres = CalcStructureTensor(_gx, _gy, sgxx, sgxy, sgyy);

#ifdef BIAS_DEBUG
  if (DebugLevelIsSet(D_ST_WRITE_DEBUG_IM)){
    _WriteDebugImage(sgxx, "sgxx.mip");
    _WriteDebugImage(sgxy, "sgxy.mip");
    _WriteDebugImage(sgyy, "sgyy.mip");
  }
#endif
  return (sres==0 && gres==0)?0:-1;
}

template <class InputStorageType, class OutputStorageType>
int StructureTensor<InputStorageType, OutputStorageType>::
CalcStructureTensor(const Image<OutputStorageType>& gx, 
                    const Image<OutputStorageType>& gy,
                    Image<OutputStorageType>& sgxx, 
                    Image<OutputStorageType>& sgxy,
                    Image<OutputStorageType>& sgyy)
{
  int sres;
  // initialize the destination images
  if (!gx.SamePixelAndChannelCount(sgxx)){
    if (!sgxx.IsEmpty()) sgxx.Release();
    sgxx.Init(gx.GetWidth(), gx.GetHeight(), 1);
  }
  if (!gx.SamePixelAndChannelCount(sgxy)){
    if (!sgxy.IsEmpty()) sgxy.Release();
    sgxy.Init(gx.GetWidth(), gx.GetHeight(), 1);
  }
  if (!gx.SamePixelAndChannelCount(sgyy)){
    if (!sgyy.IsEmpty()) sgyy.Release();
    sgyy.Init(gx.GetWidth(), gx.GetHeight(), 1);
  }
  // calculate it
  //  if (*_FilterBorderHandling==FilterBase<InputStorageType>::TBH_valid){
    switch (*_HalfWinSize){
    case 1:
      sres=CalcStructureTensor3x3(gx, gy, sgxx, sgxy, sgyy);
      break;
    case 2:
      sres=CalcStructureTensor5x5(gx, gy, sgxx, sgxy, sgyy);
      break;
    case 3:
      sres=CalcStructureTensor7x7(gx, gy, sgxx, sgxy, sgyy);
      break;
    default:
      sres=CalcStructureTensorValid(gx, gy, sgxx, sgxy, sgyy);
      break;
    }
  
  
  return sres;
}

template <class InputStorageType, class OutputStorageType>
int StructureTensor<InputStorageType, OutputStorageType>::
CalcStructureTensorValid(const Image<OutputStorageType>& gx,
                         const Image<OutputStorageType>& gy,
                         Image<OutputStorageType>& sgxx, 
                         Image<OutputStorageType>& sgxy,
                         Image<OutputStorageType>& sgyy)
{
#ifdef BIAS_DEBUG
  BIASASSERT(gx.SamePixelAndChannelCount(gy));
  BIASASSERT(gx.SamePixelAndChannelCount(sgxx));
  BIASASSERT(gx.SamePixelAndChannelCount(sgxy));
  BIASASSERT(gx.SamePixelAndChannelCount(sgyy));
#endif  
  

  const int width=gx.GetWidth();
  const int height=gx.GetHeight();
  if (width>_InternalMemWidth || height>_InternalMemHeight){
    //_DeleteInternalMem();
    _AllocInternalMem(width, height);
  }

  _GradientProducts(gx, gy);

  // propagate ROI
  int tlx, tly, brx, bry;
  _gxx.GetROI()->GetCorners(tlx, tly, brx, bry);
  const int hws=*_HalfWinSize;
  const int minx=tlx+hws;
  const int maxx=brx-hws;
  const int miny=tly+hws;
  const int maxy=bry-hws;
  sgxx.GetROI()->SetCorners(minx, miny, maxx, maxy);
  sgxy.GetROI()->SetCorners(minx, miny, maxx, maxy);
  sgyy.GetROI()->SetCorners(minx, miny, maxx, maxy);

  register int x, y, ix, iy, mx, my;
  register OutputStorageType **isgxx=sgxx.GetImageDataArray();
  register OutputStorageType **isgxy=sgxy.GetImageDataArray();
  register OutputStorageType **isgyy=sgyy.GetImageDataArray();
  register OutputStorageType **igxx=_gxx.GetImageDataArray();
  register OutputStorageType **igxy=_gxy.GetImageDataArray();
  register OutputStorageType **igyy=_gyy.GetImageDataArray();

  for (y=miny; y<maxy; y++){
    for (x=minx; x<maxx; x++){
      isgxx[y][x]=isgxy[y][x]=isgyy[y][x]=(OutputStorageType)0.0;
      for (iy=-hws; iy<=hws; iy++){
        my=y+iy;
        for (ix=-hws; ix<=hws; ix++){
          mx=x+ix;
          isgxx[y][x]+=igxx[my][mx];
          isgxy[y][x]+=igxy[my][mx];
          isgyy[y][x]+=igyy[my][mx];
        }
      }
#ifdef BIAS_DEBUG
      if (DebugLevelIsSet(D_ST_FEATURES) && y<miny+2 && x<minx+2){
        cerr << "( "<<x<<", "<<y<<") sgxx "<<isgxx[y][x]<<"  sgxy "
             <<isgxy[y][x]<<"  sgyy "<<isgyy[y][x]<<endl;
      }
#endif      
    }
  }
  return 0;
}


template <class InputStorageType, class OutputStorageType>
int StructureTensor<InputStorageType, OutputStorageType>::
CalcStructureTensor3x3(const Image<OutputStorageType>& gx,
                       const Image<OutputStorageType>& gy, 
                       Image<OutputStorageType>& sgxx, 
                       Image<OutputStorageType>& sgxy,
                       Image<OutputStorageType>& sgyy)
{

#ifdef BIAS_DEBUG
  BIASASSERT(gx.SamePixelAndChannelCount(gy));
  BIASASSERT(gx.SamePixelAndChannelCount(sgxx));
  BIASASSERT(gx.SamePixelAndChannelCount(sgxy));
  BIASASSERT(gx.SamePixelAndChannelCount(sgyy));
#endif  

  const int width=gx.GetWidth();
  const int height=gx.GetHeight();
  if (width>_InternalMemWidth || height>_InternalMemHeight){
    //_DeleteInternalMem();
    _AllocInternalMem(width, height);
  }

  _GradientProducts(gx, gy);

  // propagate ROI
  int tlx, tly, brx, bry;
  _gxx.GetROI()->GetCorners(tlx, tly, brx, bry);
  const int minx=tlx+1;
  const int maxx=brx-1;
  const int miny=tly+1;
  const int maxy=bry-1;
  sgxx.GetROI()->SetCorners(minx, miny, maxx, maxy);
  sgxy.GetROI()->SetCorners(minx, miny, maxx, maxy);
  sgyy.GetROI()->SetCorners(minx, miny, maxx, maxy);

  register int x, y, xm, xp, ym, yp;
  register OutputStorageType **isgxx=sgxx.GetImageDataArray();
  register OutputStorageType **isgxy=sgxy.GetImageDataArray();
  register OutputStorageType **isgyy=sgyy.GetImageDataArray();
  register OutputStorageType **igxx=_gxx.GetImageDataArray();
  register OutputStorageType **igxy=_gxy.GetImageDataArray();
  register OutputStorageType **igyy=_gyy.GetImageDataArray();
  for (y=miny; y<maxy; y++){
    for (x=minx; x<maxx; x++){
      xm=x-1; xp=x+1;
      ym=y-1; yp=y+1; 
      isgxx[y][x]=
        igxx[ym][xm]+igxx[ym][x]+igxx[ym][xp]+
        igxx[y][xm]+igxx[y][x]+igxx[y][xp]+
        igxx[yp][xm]+igxx[yp][x]+igxx[yp][xp];
      isgxy[y][x]=
        igxy[ym][xm]+igxy[ym][x]+igxy[ym][xp]+
        igxy[y][xm]+igxy[y][x]+igxy[y][xp]+
        igxy[yp][xm]+igxy[yp][x]+igxy[yp][xp];
      isgyy[y][x]=
        igyy[ym][xm]+igyy[ym][x]+igyy[ym][xp]+
        igyy[y][xm]+igyy[y][x]+igyy[y][xp]+
        igyy[yp][xm]+igyy[yp][x]+igyy[yp][xp];
    }
  }
#ifdef BIAS_DEBUG
  if (DebugLevelIsSet(D_ST_WRITE_DEBUG_IM)){
    _WriteDebugImage(sgxx, "sgxx-3x3.mip");
    _WriteDebugImage(sgxy, "sgxy-3x3.mip");
    _WriteDebugImage(sgyy, "sgyy-3x3.mip");
  
    CalcStructureTensorValid(gx, gy, sgxx, sgxy, sgyy);
    _WriteDebugImage(sgxx, "sgxx-st.mip");
    _WriteDebugImage(sgxy, "sgxy-st.mip");
    _WriteDebugImage(sgyy, "sgyy-st.mip");
  }
#endif

  return 0;
}

template <class InputStorageType, class OutputStorageType>
int StructureTensor<InputStorageType, OutputStorageType>::
CalcStructureTensor5x5(const Image<OutputStorageType>& gx,
                       const Image<OutputStorageType>& gy, 
                       Image<OutputStorageType>& sgxx, 
                       Image<OutputStorageType>& sgxy,
                       Image<OutputStorageType>& sgyy)
{
#ifdef TIME_MEASURE
  TimeMeasure timer;
  timer.Start();
#endif

#ifdef BIAS_DEBUG
  BIASASSERT(gx.SamePixelAndChannelCount(gy));
  BIASASSERT(gx.SamePixelAndChannelCount(sgxx));
  BIASASSERT(gx.SamePixelAndChannelCount(sgxy));
  BIASASSERT(gx.SamePixelAndChannelCount(sgyy));
#endif  

  const int width=gx.GetWidth();
  const int height=gx.GetHeight();
  if (width>_InternalMemWidth || height>_InternalMemHeight){
    //_DeleteInternalMem();
    _AllocInternalMem(width, height);
  }

  _GradientProducts(gx, gy);

#ifdef TIME_MEASURE
  timer.Stop();
  cerr << "_GradientProducts took "<<timer.GetRealTime()<<" us\n";
  timer.Reset();
  timer.Start();
#endif

  // propagate ROI
  int tlx, tly, brx, bry;
  _gxx.GetROI()->GetCorners(tlx, tly, brx, bry);
  const int miny=tly;
  const int maxy=bry;
  const int minxp=tlx+2;
  const int maxxm=brx-2;
  const int minyp=tly+2;
  const int maxym=bry-2;
  sgxx.GetROI()->SetCorners(minxp, minyp, maxxm, maxym);
  sgxy.GetROI()->SetCorners(minxp, minyp, maxxm, maxym);
  sgyy.GetROI()->SetCorners(minxp, minyp, maxxm, maxym);

  register int x, y, xm, xmm, xp, xpp, ym, ymm, yp, ypp;
  register OutputStorageType **isgxx=sgxx.GetImageDataArray();
  register OutputStorageType **isgxy=sgxy.GetImageDataArray();
  register OutputStorageType **isgyy=sgyy.GetImageDataArray();
  register OutputStorageType **itsgxx=_tsgxx.GetImageDataArray();
  register OutputStorageType **itsgxy=_tsgxy.GetImageDataArray();
  register OutputStorageType **itsgyy=_tsgyy.GetImageDataArray();
  register OutputStorageType **igxx=_gxx.GetImageDataArray();
  register OutputStorageType **igxy=_gxy.GetImageDataArray();
  register OutputStorageType **igyy=_gyy.GetImageDataArray();
  // separate loops are faster
  for (y=miny; y<maxy; y++){
    for (x=minxp; x<maxxm; x++){
      xmm=x-2; xm=x-1; xp=x+1; xpp=x+2; 
      itsgxx[y][x]=
        igxx[y][xmm]+igxx[y][xm]+igxx[y][x]+igxx[y][xp]+igxx[y][xpp];
    }
  }
  for (y=minyp; y<maxym; y++){
    for (x=minxp; x<maxxm; x++){
      ymm=y-2; ym=y-1; yp=y+1; ypp=y+2; 
      isgxx[y][x]=
        itsgxx[ymm][x]+itsgxx[ym][x]+itsgxx[y][x]+itsgxx[yp][x]+itsgxx[ypp][x];
    }
  }
  for (y=miny; y<maxy; y++){
    for (x=minxp; x<maxxm; x++){
      xmm=x-2; xm=x-1; xp=x+1; xpp=x+2;       
      itsgxy[y][x]=
        igxy[y][xmm]+igxy[y][xm]+igxy[y][x]+igxy[y][xp]+igxy[y][xpp];
    }
  }
  for (y=minyp; y<maxym; y++){
    for (x=minxp; x<maxxm; x++){
      ymm=y-2; ym=y-1; yp=y+1; ypp=y+2; 
      isgxy[y][x]=
        itsgxy[ymm][x]+itsgxy[ym][x]+itsgxy[y][x]+itsgxy[yp][x]+itsgxy[ypp][x];
    }
  }
  for (y=miny; y<maxy; y++){
    for (x=minxp; x<maxxm; x++){
      xmm=x-2; xm=x-1; xp=x+1; xpp=x+2;       
      itsgyy[y][x]=
        igyy[y][xmm]+igyy[y][xm]+igyy[y][x]+igyy[y][xp]+igyy[y][xpp];
    }
  }
  for (y=minyp; y<maxym; y++){
    for (x=minxp; x<maxxm; x++){
      ymm=y-2; ym=y-1; yp=y+1; ypp=y+2; 
      isgyy[y][x]=
        itsgyy[ymm][x]+itsgyy[ym][x]+itsgyy[y][x]+itsgyy[yp][x]+itsgyy[ypp][x];
    }
  }
  
#ifdef BIAS_DEBUG
  if (DebugLevelIsSet(D_ST_WRITE_DEBUG_IM)){
    _WriteDebugImage(sgxx, "sgxx-5x5.mip");
    _WriteDebugImage(sgxy, "sgxy-5x5.mip");
    _WriteDebugImage(sgyy, "sgyy-5x5.mip");
  
    CalcStructureTensorValid(gx, gy, sgxx, sgxy, sgyy);
    _WriteDebugImage(sgxx, "sgxx-st.mip");
    _WriteDebugImage(sgxy, "sgxy-st.mip");
    _WriteDebugImage(sgyy, "sgyy-st.mip");
  }
#endif
  
#ifdef TIME_MEASURE
  timer.Stop();
  cerr << "CalcStructureTensor5x5 without _GradientProducts took "<<timer.GetRealTime()<<" us\n";
#endif
  return 0;
}

template <class InputStorageType, class OutputStorageType>
int StructureTensor<InputStorageType, OutputStorageType>::
CalcStructureTensor7x7(const Image<OutputStorageType>& gx,
                       const Image<OutputStorageType>& gy, 
                       Image<OutputStorageType>& sgxx, 
                       Image<OutputStorageType>& sgxy,
                       Image<OutputStorageType>& sgyy)
{
#ifdef TIME_MEASURE
  TimeMeasure timer;
  timer.Start();
#endif

#ifdef BIAS_DEBUG
  BIASASSERT(gx.SamePixelAndChannelCount(gy));
  BIASASSERT(gx.SamePixelAndChannelCount(sgxx));
  BIASASSERT(gx.SamePixelAndChannelCount(sgxy));
  BIASASSERT(gx.SamePixelAndChannelCount(sgyy));
#endif  

  register int width=gx.GetWidth();
  const int height=gx.GetHeight();
  if (width>_InternalMemWidth || height>_InternalMemHeight){
    _AllocInternalMem(width, height);
  }

  _GradientProducts(gx, gy);

#ifdef TIME_MEASURE
  timer.Stop();
  cerr << "_GradientProducts took "<<timer.GetRealTime()<<" us\n";
  timer.Reset();
  timer.Start();
#endif

  // propagate ROI
  int tlx, tly, brx, bry;
  _gxx.GetROI()->GetCorners(tlx, tly, brx, bry);
  const int miny=tly;
  const int maxy=bry;
  const int minxp=tlx+3;
  const int maxxm=brx-3;
  const int minyp=tly+3;
  const int maxym=bry-3;
  sgxx.GetROI()->SetCorners(minxp, minyp, maxxm, maxym);
  sgxy.GetROI()->SetCorners(minxp, minyp, maxxm, maxym);
  sgyy.GetROI()->SetCorners(minxp, minyp, maxxm, maxym);

  {
    register OutputStorageType *ptsgxx, *ptmsgxx, *pmgxx, *ppgxx, *lend, *end;
    register OutputStorageType *ptsgxy, *ptmsgxy, *pmgxy, *ppgxy;
    register OutputStorageType *ptsgyy, *ptmsgyy, *pmgyy, *ppgyy;
    register const int tstep=width-(maxxm-minxp);
    const int offset=(width*miny)+minxp;
    ptmsgxx = _tsgxx.GetImageData() + offset;
    ptmsgxy = _tsgxy.GetImageData() + offset;
    ptmsgyy = _tsgyy.GetImageData() + offset;
    pmgxx = _gxx.GetImageData() + offset-3;
    pmgxy = _gxy.GetImageData() + offset-3;
    pmgyy = _gyy.GetImageData() + offset-3;
    lend = _tsgxx.GetImageData() + (width*miny)+maxxm;
    end = _tsgxx.GetImageData() + ((maxy-1)*width)+maxxm;
    while (ptmsgxx<end){
      ppgxx = pmgxx;
      ptsgxx = ptmsgxx;
      *ptsgxx=(OutputStorageType)0.0;
      *ptsgxx+=*ppgxx++;
      *ptsgxx+=*ppgxx++;
      *ptsgxx+=*ppgxx++;
      *ptsgxx+=*ppgxx++;
      *ptsgxx+=*ppgxx++;
      *ptsgxx+=*ppgxx++;
      *ptsgxx+=*ppgxx++;
      *ptsgxx++;

      ppgxy = pmgxy;
      ptsgxy = ptmsgxy;
      *ptsgxy=(OutputStorageType)0.0;
      *ptsgxy+=*ppgxy++;
      *ptsgxy+=*ppgxy++;
      *ptsgxy+=*ppgxy++;
      *ptsgxy+=*ppgxy++;
      *ptsgxy+=*ppgxy++;
      *ptsgxy+=*ppgxy++;
      *ptsgxy+=*ppgxy++;
      *ptsgxy++;

      ppgyy = pmgyy;
      ptsgyy = ptmsgyy;
      *ptsgyy=(OutputStorageType)0.0;
      *ptsgyy+=*ppgyy++;
      *ptsgyy+=*ppgyy++;
      *ptsgyy+=*ppgyy++;
      *ptsgyy+=*ppgyy++;
      *ptsgyy+=*ppgyy++;
      *ptsgyy+=*ppgyy++;
      *ptsgyy+=*ppgyy++;
      *ptsgyy++;
      while (ptsgxx<lend){
        *ptsgxx++ = *ptmsgxx++ - *pmgxx++ + *ppgxx++;
        *ptsgxy++ = *ptmsgxy++ - *pmgxy++ + *ppgxy++;
        *ptsgyy++ = *ptmsgyy++ - *pmgyy++ + *ppgyy++;
      }
      lend+=width;
      pmgxx+=tstep;
      ptmsgxx+=tstep;
      pmgxy+=tstep;
      ptmsgxy+=tstep;
      pmgyy+=tstep;
      ptmsgyy+=tstep;
    }
  }
  
#ifdef TIME_MEASURE
  timer.Stop();
  cerr << "first convolution took "<<timer.GetRealTime()<<" us\n";
  timer.Reset();
  timer.Start();
#endif
  /*
  //const int max_cache_size=128000;
  //const int dy=max_cache_size/width;
  //int mminy=miny, mminyp=minyp, mmaxy, mmaxym;
  //for (mmaxy=dy, mmaxym=dy-3; mmaxy<maxy; 
  //     mmaxy+=dy, mmaxym+=dy, mminy+=dy, mmindyp+=dy)
  { // this is very slow, maybe because of caches misses if done on the hole 
  // image
  register OutputStorageType *ppsgxx, *pmsgxx, *ptmsgxx, *ptpsgxx, *lend, *end;
  register OutputStorageType *ppsgxy, *pmsgxy, *ptmsgxy, *ptpsgxy;
  register OutputStorageType *ppsgyy, *pmsgyy, *ptmsgyy, *ptpsgyy;
  register const int tstep=(width*(maxym-minyp-1))-1;
  const int offset=(width*miny)+minxp;

  ptmsgxx = _tsgxx.GetImageData() + offset;
  ppsgxx = pmsgxx = sgxx.GetImageData() + (width*minyp)+minxp;
  ptmsgxy = _tsgxy.GetImageData() + offset;
  ppsgxy = pmsgxy = sgxy.GetImageData() + (width*minyp)+minxp;
  ptmsgyy = _tsgyy.GetImageData() + offset;
  ppsgyy = pmsgyy = sgyy.GetImageData() + (width*minyp)+minxp;

  lend = sgxx.GetImageData() + (width*(maxym))+minxp;
  end = sgxx.GetImageData() + (width*(maxym))+maxxm-1;

  while (ppsgxx<end){
  ppsgxx = pmsgxx + width;
  ptpsgxx = ptmsgxx;
  *pmsgxx=(OutputStorageType)0.0;
  *pmsgxx+=*ptpsgxx; ptpsgxx+=width;
  *pmsgxx+=*ptpsgxx; ptpsgxx+=width;
  *pmsgxx+=*ptpsgxx; ptpsgxx+=width;
  *pmsgxx+=*ptpsgxx; ptpsgxx+=width;
  *pmsgxx+=*ptpsgxx; ptpsgxx+=width;
  *pmsgxx+=*ptpsgxx; ptpsgxx+=width;
  *pmsgxx+=*ptpsgxx; ptpsgxx+=width;

  ppsgxy = pmsgxy + width;
  ptpsgxy = ptmsgxy;
  *pmsgxy=(OutputStorageType)0.0;
  *pmsgxy+=*ptpsgxy; ptpsgxy+=width;
  *pmsgxy+=*ptpsgxy; ptpsgxy+=width;
  *pmsgxy+=*ptpsgxy; ptpsgxy+=width;
  *pmsgxy+=*ptpsgxy; ptpsgxy+=width;
  *pmsgxy+=*ptpsgxy; ptpsgxy+=width;
  *pmsgxy+=*ptpsgxy; ptpsgxy+=width;
  *pmsgxy+=*ptpsgxy; ptpsgxy+=width;

  ppsgyy = pmsgyy + width;
  ptpsgyy = ptmsgyy;
  *pmsgyy=(OutputStorageType)0.0;
  *pmsgyy+=*ptpsgyy; ptpsgyy+=width;
  *pmsgyy+=*ptpsgyy; ptpsgyy+=width;
  *pmsgyy+=*ptpsgyy; ptpsgyy+=width;
  *pmsgyy+=*ptpsgyy; ptpsgyy+=width;
  *pmsgyy+=*ptpsgyy; ptpsgyy+=width;
  *pmsgyy+=*ptpsgyy; ptpsgyy+=width;
  *pmsgyy+=*ptpsgyy; ptpsgyy+=width;

  while (ppsgxx<lend){
  *ppsgxx = *pmsgxx - *ptmsgxx + *ptpsgxx;
  ppsgxx+=width;
  pmsgxx+=width;
  ptmsgxx+=width;
  ptpsgxx+=width;

  *ppsgxy = *pmsgxy - *ptmsgxy + *ptpsgxy;
  ppsgxy+=width;
  pmsgxy+=width;
  ptmsgxy+=width;
  ptpsgxy+=width;

  *ppsgyy = *pmsgyy - *ptmsgyy + *ptpsgyy;
  ppsgyy+=width;
  pmsgyy+=width;
  ptmsgyy+=width;
  ptpsgyy+=width;

  }
  lend++;
  ptmsgxx-=tstep;
  pmsgxx-=tstep;
  ptmsgxy-=tstep;
  pmsgxy-=tstep;
  ptmsgyy-=tstep;
  pmsgyy-=tstep;
  }
  }
  
  */

  
  // todo this really fast by pointer arithmetic
  register int x, y, myp, mym, my1;
  register const int ymmm=minyp-3, ymm=minyp-2, ym=minyp-1, yp=minyp+1, 
    ypp=minyp+2, yppp=minyp+3;
  register OutputStorageType **isgxx=sgxx.GetImageDataArray();
  register OutputStorageType **isgxy=sgxy.GetImageDataArray();
  register OutputStorageType **isgyy=sgyy.GetImageDataArray();
  register OutputStorageType **itsgxx=_tsgxx.GetImageDataArray();
  register OutputStorageType **itsgxy=_tsgxy.GetImageDataArray();
  register OutputStorageType **itsgyy=_tsgyy.GetImageDataArray();

  if (width*height>153600){ // the caches are running empty
    // do concolution in two parts
    const int medy=((maxym-minyp)>>1)+minyp;
    const int eymmm=medy-3, eymm=medy-2, eym=medy-1, eyp=medy+1, 
      eypp=medy+2, eyppp=medy+3;
    for (x=minxp; x<maxxm; x++){
      isgxy[minyp][x]=itsgxy[ymmm][x]+itsgxy[ymm][x]+itsgxy[ym][x]+
        itsgxy[minyp][x]+itsgxy[yp][x]+itsgxy[ypp][x]+itsgxy[yppp][x];
      for (y=minyp+1; y<medy; y++){
        myp=y+3; mym=y-4; my1=y-1;
        isgxy[y][x]=isgxy[my1][x]-itsgxy[mym][x]+itsgxy[myp][x];
      }
    }
    for (x=minxp; x<maxxm; x++){
      isgxy[medy][x]=itsgxy[eymmm][x]+itsgxy[eymm][x]+itsgxy[eym][x]+
        itsgxy[medy][x]+itsgxy[eyp][x]+itsgxy[eypp][x]+itsgxy[eyppp][x];
      for (y=medy+1; y<maxym; y++){
        myp=y+3; mym=y-4; my1=y-1;
        isgxy[y][x]=isgxy[my1][x]-itsgxy[mym][x]+itsgxy[myp][x];
      }
    }

    for (x=minxp; x<maxxm; x++){
      isgxx[minyp][x]=itsgxx[ymmm][x]+itsgxx[ymm][x]+itsgxx[ym][x]+
        itsgxx[minyp][x]+itsgxx[yp][x]+itsgxx[ypp][x]+itsgxx[yppp][x];
      for (y=minyp+1; y<medy; y++){
        myp=y+3; mym=y-4; my1=y-1;
        isgxx[y][x]=isgxx[my1][x]-itsgxx[mym][x]+itsgxx[myp][x];
      }
    }
    for (x=minxp; x<maxxm; x++){
      isgxx[medy][x]=itsgxx[eymmm][x]+itsgxx[eymm][x]+itsgxx[eym][x]+
        itsgxx[medy][x]+itsgxx[eyp][x]+itsgxx[eypp][x]+itsgxx[eyppp][x];
      for (y=medy+1; y<maxym; y++){
        myp=y+3; mym=y-4; my1=y-1;
        isgxx[y][x]=isgxx[my1][x]-itsgxx[mym][x]+itsgxx[myp][x];
      }
    }

    for (x=minxp; x<maxxm; x++){
      isgyy[minyp][x]=itsgyy[ymmm][x]+itsgyy[ymm][x]+itsgyy[ym][x]+
        itsgyy[minyp][x]+itsgyy[yp][x]+itsgyy[ypp][x]+itsgyy[yppp][x];
      for (y=minyp+1; y<medy; y++){
        myp=y+3; mym=y-4; my1=y-1;
        isgyy[y][x]=isgyy[my1][x]-itsgyy[mym][x]+itsgyy[myp][x];
      }
    }
    for (x=minxp; x<maxxm; x++){
      isgyy[medy][x]=itsgyy[eymmm][x]+itsgyy[eymm][x]+itsgyy[eym][x]+
        itsgyy[medy][x]+itsgyy[eyp][x]+itsgyy[eypp][x]+itsgyy[eyppp][x];
      for (y=medy+1; y<maxym; y++){
        myp=y+3; mym=y-4; my1=y-1;
        isgyy[y][x]=isgyy[my1][x]-itsgyy[mym][x]+itsgyy[myp][x];
      }
    }
  } else {
    for (x=minxp; x<maxxm; x++){
      isgxy[minyp][x]=itsgxy[ymmm][x]+itsgxy[ymm][x]+itsgxy[ym][x]+
        itsgxy[minyp][x]+itsgxy[yp][x]+itsgxy[ypp][x]+itsgxy[yppp][x];
      for (y=minyp+1; y<maxym; y++){
        myp=y+3; mym=y-4; my1=y-1;
        isgxy[y][x]=isgxy[my1][x]-itsgxy[mym][x]+itsgxy[myp][x];
      }
    }

    for (x=minxp; x<maxxm; x++){
      isgxx[minyp][x]=itsgxx[ymmm][x]+itsgxx[ymm][x]+itsgxx[ym][x]+
        itsgxx[minyp][x]+itsgxx[yp][x]+itsgxx[ypp][x]+itsgxx[yppp][x];
      for (y=minyp+1; y<maxym; y++){
        myp=y+3; mym=y-4; my1=y-1;
        isgxx[y][x]=isgxx[my1][x]-itsgxx[mym][x]+itsgxx[myp][x];
      }
    }

    for (x=minxp; x<maxxm; x++){
      isgyy[minyp][x]=itsgyy[ymmm][x]+itsgyy[ymm][x]+itsgyy[ym][x]+
        itsgyy[minyp][x]+itsgyy[yp][x]+itsgyy[ypp][x]+itsgyy[yppp][x];
      for (y=minyp+1; y<maxym; y++){
        myp=y+3; mym=y-4; my1=y-1;
        isgyy[y][x]=isgyy[my1][x]-itsgyy[mym][x]+itsgyy[myp][x];
      }
    }
  }
#ifdef TIME_MEASURE
  timer.Stop();
  cerr << "second convolution took "<<timer.GetRealTime()<<" us\n";
  timer.Reset();
  timer.Start();
#endif

#ifdef BIAS_DEBUG
  if (DebugLevelIsSet(D_ST_WRITE_DEBUG_IM)){
    _WriteDebugImage(sgxx, "sgxx-7x7.mip");
    _WriteDebugImage(sgxy, "sgxy-7x7.mip");
    _WriteDebugImage(sgyy, "sgyy-7x7.mip");
  
    CalcStructureTensorValid(gx, gy, sgxx, sgxy, sgyy);
    _WriteDebugImage(sgxx, "sgxx-st.mip");
    _WriteDebugImage(sgxy, "sgxy-st.mip");
    _WriteDebugImage(sgyy, "sgyy-st.mip");
  }
#endif

#ifdef TIME_MEASURE
  timer.Stop();
  cerr << "CalcStructureTensor7x7 without _GradientProducts took "<<timer.GetRealTime()<<" us\n";
#endif
  return 0;
}

//////////////////////////////////////////////////////////////////////////
// protected functions
//////////////////////////////////////////////////////////////////////////

template <class InputStorageType, class OutputStorageType>
void StructureTensor<InputStorageType, OutputStorageType>::
_AddParam(Param& para, const int grp, const std::string& postfix)
{
  
  if (para.CheckParam("GradientType"+postfix)){
    _GradientType = para.GetParamEnum("GradientType"+postfix);
  } else {
    vector<string> tt;
    vector<int> tti;
    tt.push_back("GT_Sobel3x3"); 
    tti.push_back(GT_Sobel3x3);
    tt.push_back("GT_Gauss"); 
    tti.push_back(GT_Gauss);
    tt.push_back("GT_GaussAsymmetric"); 
    tti.push_back(GT_GaussAsymmetric);
    
    _GradientType = para.AddParamEnum("GradientType"+postfix,
                                      "type of gradient filter to use",
                                      tt, GT_Gauss, &tti, 0, grp); 
  }
  
  if (para.CheckParam("HalfWinSize"+postfix)){ 
    _HalfWinSize=
      para.GetParamInt("HalfWinSize"+postfix);
  } else { 
    _HalfWinSize=
      para.AddParamInt("HalfWinSize"+postfix, 
                       "half window size for computation of structure tensor", 
                       3, 0, INT_MAX, 0, grp);
  }
}


template <class InputStorageType, class OutputStorageType>
void StructureTensor<InputStorageType, OutputStorageType>::
_AllocInternalMem(const int width, const int height)
{
  if (!_gxx.IsEmpty()) _gxx.Release();
  _gxx.Init(width, height, 1);
  if (!_gxy.IsEmpty()) _gxy.Release();
  _gxy.Init(width, height, 1);
  if (!_gyy.IsEmpty()) _gyy.Release();
  _gyy.Init(width, height, 1);
  if (!_tsgxx.IsEmpty()) _tsgxx.Release();
  _tsgxx.Init(width, height, 1);
  if (!_tsgxy.IsEmpty()) _tsgxy.Release();
  _tsgxy.Init(width, height, 1);
  if (!_tsgyy.IsEmpty()) _tsgyy.Release();
  _tsgyy.Init(width, height, 1);

  _InternalMemWidth=width;
  _InternalMemHeight=height;
}

template <class InputStorageType, class OutputStorageType>
void StructureTensor<InputStorageType, OutputStorageType>::
_UpdateGrad()
{
  _grad = &_gsobel3x3;
  
  switch (*_GradientType){
  case GT_Sobel3x3:
    _grad = &_gsobel3x3;
    BIASCDOUT(D_ST_INIT, "gradient Sobel3x3\n");
    break;
  case GT_Gauss:
    _grad = &_ggauss;
    BIASCDOUT(D_ST_INIT, "gradient Gauss\n");
    break;
  case GT_GaussAsymmetric:
    _grad = &_ggaussasymmetric;
    BIASCDOUT(D_ST_INIT, "gradient GaussAsymmetric\n");
    break;
  default:
    BIASERR("unknown gradient type");
    BIASABORT;
    //_grad=NULL;
    //break;
  } 
}

template <class InputStorageType, class OutputStorageType>
void StructureTensor<InputStorageType, OutputStorageType>::
_WriteDebugImage(Image<OutputStorageType>& im, std::string name)
{
#ifdef BIAS_DEBUG
  if (ImageIO::Write(name, im)!=0){
    BIASERR("error wrting "<<name);
  }
#endif
}


template <class InputStorageType, class OutputStorageType>
void StructureTensor<InputStorageType, OutputStorageType>::
_GradientProducts(const Image<OutputStorageType>& gx, 
                  const Image<OutputStorageType>& gy)
{
#ifdef BIAS_DEBUG
  BIASASSERT(gx.SamePixelAndChannelCount(gy));
  BIASASSERT(gx.SamePixelAndChannelCount(_gxx));
  BIASASSERT(gx.SamePixelAndChannelCount(_gxy));
  BIASASSERT(gx.SamePixelAndChannelCount(_gyy));
#endif  
  int xtlx, xtly, xbrx, xbry, ytlx, ytly, ybrx, ybry;
  gx.GetROI()->GetCorners(xtlx, xtly, xbrx, xbry);
  gy.GetROI()->GetCorners(ytlx, ytly, ybrx, ybry);

  const int width=gx.GetWidth();
  const int height=gx.GetHeight();
  if (width>_InternalMemWidth || height>_InternalMemHeight){
    //_DeleteInternalMem();
    _AllocInternalMem(width, height);
  }
  const int minx=((xtlx<ytlx)?(ytlx):(xtlx));
  const int maxx=((xbrx<ybrx)?(xbrx):(ybrx));
  const int miny=((xtly<ytly)?(ytly):(xtly));
  const int maxy=((xbrx<ybry)?(xbry):(ybry));

  _gxx.GetROI()->SetCorners(minx, miny, maxx, maxy);
  _gxy.GetROI()->SetCorners(minx, miny, maxx, maxy);
  _gyy.GetROI()->SetCorners(minx, miny, maxx, maxy);

  /*
  register OutputStorageType **igx=gx.GetImageDataArray();
  register OutputStorageType **igy=gy.GetImageDataArray();
  register OutputStorageType **igxx=_gxx.GetImageDataArray();
  register OutputStorageType **igxy=_gxy.GetImageDataArray();
  register OutputStorageType **igyy=_gyy.GetImageDataArray();
  register int x, y;
  register OutputStorageType mgx, mgy;
  for (y=miny; y<maxy; y++){
    for (x=minx; x<maxx; x++){
      mgx=igx[y][x];
      mgy=igy[y][x];
      igxx[y][x]=mgx*mgx;
      igxy[y][x]=mgx*mgy;
      igyy[y][x]=mgy*mgy;
    }
  }

  */


  register const int step=width-(maxx-minx);
  const int offset=miny*width+minx;
  register OutputStorageType *igx=gx.GetImageData()+offset;
  register OutputStorageType *igy=gy.GetImageData()+offset;
  register OutputStorageType *igxx=_gxx.GetImageData()+offset;
  register OutputStorageType *igxy=_gxy.GetImageData()+offset;
  register OutputStorageType *igyy=_gyy.GetImageData()+offset;

  register const OutputStorageType *end=gx.GetImageData()+(maxy-1)*width+maxx;
  OutputStorageType *lend=gx.GetImageData()+miny*width+maxx;

  while (igx<end){
    while (igx<lend){
      *igxx++=*igx * *igx;
      *igyy++=*igy * *igy;
      *igxy++=*igx++ * *igy++;
    }
    igx+=step;
    igy+=step;
    igxx+=step;
    igxy+=step;
    igyy+=step;
    lend+=width;
  }


}

template <class InputStorageType, class OutputStorageType>
void StructureTensor<InputStorageType, OutputStorageType>::
GetBordersValid_(int &border_x, int &border_y) const
{
  _grad->GetBorders(border_x, border_y);
  border_x += *_HalfWinSize;
  border_y += *_HalfWinSize;
}

//////////////////////////////////////////////////////////////////////////
//                 instantiation
//////////////////////////////////////////////////////////////////////////
#define FILTER_INSTANTIATION_CLASS StructureTensor
#define FILTER_INSTANTIATION_NO_UNSIGNED_OUTPUT
#include "Filterinst.hh"
