#
# This file generates the bias library config header 
# bias_config.h (in BIAS_BINARY_DIR) automatically 
# from bias_config.h.in (in BIAS_SOURCE_DIR).
#
# from the appropriate ..._FOUND flags
#
# -------------------------------------------------------------------------
# @author Jan Woetzel <http://www.mip.informatik.uni-kiel.de/~jw> (11/2003)
# -------------------------------------------------------------------------

## set the in and out cmake configure files: 
SET(FILE_BIAS_CONFIG_H_IN "${BIAS_SOURCE_DIR}/bias_config.h.in")
SET(FILE_BIAS_CONFIG_H "${BIAS_BINARY_DIR}/bias_config.h")

# save build seting for bias_config.h to switch to correct exansion of dllim/export on WIN32 builds. 
# was BIAS built shared or static?
IF(BUILD_SHARED_LIBS)
  SET(BIAS_BUILD_SHARED_LIBS ON)
ENDIF(BUILD_SHARED_LIBS)


#
# translate cmake ..._FOUND and USE_..  to  BIAS_HAVE_...
#

# generate BIAS_HAVE_* 
FOREACH( EXTLIB
  "BLAS"
  "LAPACK"
  "DAIMLERCHRYSLER"
  "1394CAMERACMU"
  "DC1394"
  "DSHOW"
  "DEVIL"
  "DV"
  "FFTW3"
  "GLEW"
  "GLUT"
  "GSL"
  "GTK"
  "GTK2"
  "IMLIB"
  "IMAGEMAGICKLIB"
  "LIBJPEG"
  "OPENCV"
  "OPENGL"
  "OPENEXR"
  "PTHREADS"
  "STEREOI"
  "TIFF"
  "UEYE"
  "V4L"
  "VLD"
  "WXWINDOWS"
  "XML2" 
  )
    IF   (${EXTLIB}_FOUND AND USE_${EXTLIB})
      SET(BIAS_HAVE_${EXTLIB} ON)
    ENDIF(${EXTLIB}_FOUND AND USE_${EXTLIB})    
ENDFOREACH(EXTLIB)

#
# things that do not map a lib 1:1: 
#

## compiled with class Videsource_DCAM ?
IF   (UNIX AND BIAS_HAVE_DC1394) 
  SET(BIAS_HAVE_DCAM TRUE)
ENDIF(UNIX AND BIAS_HAVE_DC1394) 
IF (WIN32 AND BIAS_HAVE_1394CAMERACMU)
  SET(BIAS_HAVE_DCAM TRUE)
ENDIF (WIN32 AND BIAS_HAVE_1394CAMERACMU)

## compiled lib BIASGui ?
#IF   (WXWINDOWS_FOUND OR GTK_FOUND)
# always built, now.
  SET(BIAS_HAVE_GUI TRUE)
#ENDIF(WXWINDOWS_FOUND OR GTK_FOUND)

## generate .h from .h.in file
#MESSAGE("DBG generating ${FILE_BIAS_CONFIG_H}  (from ${FILE_BIAS_CONFIG_H_IN})")
CONFIGURE_FILE(${FILE_BIAS_CONFIG_H_IN} ${FILE_BIAS_CONFIG_H})
