# This file generates the bias library config shell script: "bias-config"
# from bias-config.in (in BIAS_SOURCE_DIR) 
#
# AUTHOR
# Jan-Friso Evers-Senne and Jan Woetzel 



IF (UNIX)
  SET(FILE_BIAS_CONFIG_SCRIPT_IN "${BIAS_SOURCE_DIR}/bias-config.in")
  SET(FILE_BIAS_CONFIG_SCRIPT "${BIAS_BINARY_DIR}/bias-config")
  
  # where to grab this from (different for installed/notinstall case)
  SET(BIAS_PREFIX "${CMAKE_INSTALL_PREFIX}")

  #
  # extra compiler flags rpaths:
  #
  SET(BIAS_LD_FLAGS "-Wl,-rpath,${BIAS_PREFIX}/lib")
  IF(USE_OPENCV AND HAVE_OPENCV)
    SET(BIAS_LD_FLAGS "${BIAS_LD_FLAGS}:${OPENCV_LINK_DIRECTORIES}")
  ENDIF(USE_OPENCV AND HAVE_OPENCV)
  
  
  #  # ==============================================
  #  #  Recursive dependency solving via *_LIB_DEPENDS
  #  #  is correct but too slow, currently 
  #  #  because of wide recursive foreach tree. 
  #  #  keep the old "handmade" IF USE_*  for now. (JW)
  #  # ==============================================

  FOREACH( EXTLIB 
      FFTW3
      GLEW
      GTK
      GTK2
      OPENGL # is full path correct here (jw) ?
      OPENCV
      OPENEXR
      IMLIB
      IMAGEMAGICKLIB
      LIBXML2
      BLAS
      LAPACK
      # PTHREADS
      WXINDOWS
      WXWIDGETS
      )
    IF   ( USE_${EXTLIB} AND ${EXTLIB}_FOUND )
      SET(BIAS_EXTERN_LIBRARIES  "${BIAS_EXTERN_LIBRARIES} ${${EXTLIB}_LIBRARIES}")
    ENDIF( USE_${EXTLIB} AND ${EXTLIB}_FOUND )

  ENDFOREACH( EXTLIB )

  #  IF(USE_OPENGL AND GTK_GL_FOUND)
  #    SET(BIAS_EXTERN_LIBRARIES ${BIAS_EXTERN_LIBRARIES}  ${OPENGL_LIBRARIES})
  #  ENDIF(USE_OPENGL AND GTK_GL_FOUND)

  IF(USE_OPENCV)
    SET(BIAS_LD_FLAGS "${BIAS_LD_FLAGS}:${OPENCV_LINK_DIRECTORIES}")
  ENDIF(USE_OPENCV)  
  
  ## generate old-style bias-config shell script 
  ## to be used just by NON-CMAKIFIED user projects. 
  ## See PackageConfig/BIASConfig for CMAKE user projects
  
  SET(BIAS_CONFIG_SCRIPT_C_FLAGS "")
  SET(BIAS_CONFIG_SCRIPT_C_FLAGS "${BIAS_CONFIG_SCRIPT_C_FLAGS} -I${CMAKE_INSTALL_PREFIX}/include")
  SET(BIAS_CONFIG_SCRIPT_C_FLAGS "${BIAS_CONFIG_SCRIPT_C_FLAGS} -I${CMAKE_INSTALL_PREFIX}/include/BIAS")
  SET(BIAS_CONFIG_SCRIPT_C_FLAGS "${BIAS_CONFIG_SCRIPT_C_FLAGS} -I${BIAS_BINARY_DIR}")
  SET(BIAS_CONFIG_SCRIPT_C_FLAGS "${BIAS_CONFIG_SCRIPT_C_FLAGS} -I${BIAS_SOURCE_DIR}")
  

  FOREACH(INCDIR ${BIAS_EXTERN_INCLUDE_DIR})
    SET(BIAS_CONFIG_SCRIPT_C_FLAGS "${BIAS_CONFIG_SCRIPT_C_FLAGS} -I${INCDIR} -I${BIAS_SOURCE_DIR}")
  ENDFOREACH(INCDIR)

  # prefix with BIAS own installed include paths
  SET(BIAS_CONFIG_SCRIPT_C_FLAGS "-I${BIAS_PREFIX}/include -I${BIAS_PREFIX}/include/BIAS ${BIAS_CONFIG_SCRIPT_C_FLAGS}")
  # MESSAGE("DBG BIAS_CONFIG_SCRIPT_CFLAGS: ${BIAS_CONFIG_SCRIPT_C_FLAGS}")
  
  
  ##
  ## add "-l" to "real" library names  
  ## skip *.so  and all back-quoted shell scripts
  ##
  SET(BIAS_CONFIG_SCRIPT_EXTERN_LIBRARIES "")
  FOREACH(LIB ${BIAS_EXTERN_LIBRARIES})
    # do not prefix back-quited expressions, e.g/ `/bin/foo-config`
    IF (NOT ${LIB} MATCHES "[`].+[`]")
      IF (NOT ${LIB} MATCHES ".+[.][s][o]")
	IF (NOT ${LIB} MATCHES "[-][l].+")
	  
	  # OK, this a 'real' library to be prefixed: 
	  # add prefix and convert semicolon-separated vector to space-separtaed string 
	  SET(BIAS_CONFIG_SCRIPT_EXTERN_LIBS "${BIAS_CONFIG_SCRIPT_EXTERN_LIBS} -l${LIB}")
	  
	ELSE (NOT ${LIB} MATCHES "[-][l].+")
	  # MESSAGE("DBG skipping  already -l prefixed LIB=${LIB}")
	  SET(BIAS_CONFIG_SCRIPT_EXTERN_LIBS "${BIAS_CONFIG_SCRIPT_EXTERN_LIBS} ${LIB}")	
	ENDIF (NOT ${LIB} MATCHES "[-][l].+")
      ELSE (NOT ${LIB} MATCHES ".+[.][s][o]")
	#MESSAGE("DBG skipping  prefixing of .so LIB=${LIB}")
	SET(BIAS_CONFIG_SCRIPT_EXTERN_LIBS "${BIAS_CONFIG_SCRIPT_EXTERN_LIBS} ${LIB}")	
      ENDIF (NOT ${LIB} MATCHES ".+[.][s][o]")      
    ELSE (NOT ${LIB} MATCHES "[`].+[`]")
      #MESSAGE("DBG skipping prefixing of back-quoted LIB=${LIB}")
      SET(BIAS_CONFIG_SCRIPT_EXTERN_LIBS "${BIAS_CONFIG_SCRIPT_EXTERN_LIBS} ${LIB}")
    ENDIF (NOT ${LIB} MATCHES "[`].+[`]")
  ENDFOREACH(LIB)
  #MESSAGE("DBG  BIAS_CONFIG_SCRIPT_EXTERN_LIBS=${BIAS_CONFIG_SCRIPT_EXTERN_LIBS}")
  
  # prepend with BIAS own lib dir
  SET(BIAS_CONFIG_SCRIPT_EXTERN_LIBS  "-L${BIAS_PREFIX}/lib ${BIAS_CONFIG_SCRIPT_EXTERN_LIBS}")
  # append hard coded system dirs for SM
  SET(BIAS_CONFIG_SCRIPT_EXTERN_LIBS  "${BIAS_CONFIG_SCRIPT_EXTERN_LIBS} -L/usr/X11R6/lib -L/usr/lib -L/usr/local/lib ")
  IF    (LIBRARY_OUTPUT_PATH)
    SET(BIAS_CONFIG_SCRIPT_EXTERN_LIBS  "${BIAS_CONFIG_SCRIPT_EXTERN_LIBS} -L${LIBRARY_OUTPUT_PATH}")
  ELSE  (LIBRARY_OUTPUT_PATH)
    SET(BIAS_CONFIG_SCRIPT_EXTERN_LIBS  "${BIAS_CONFIG_SCRIPT_EXTERN_LIBS} -L${BIAS_BINARY_DIR}")
  ENDIF (LIBRARY_OUTPUT_PATH)

  CONFIGURE_FILE(${FILE_BIAS_CONFIG_SCRIPT_IN} 
    ${FILE_BIAS_CONFIG_SCRIPT}
    IMMEDIATE )  
  IF(UNIX)
    # set executable permissions
    EXEC_PROGRAM(chmod ARGS 755  ${FILE_BIAS_CONFIG_SCRIPT} )
  ENDIF(UNIX)


ELSE(UNIX)
  IF(WIN32)
    # OK, do nothing for now. (JW)
  ELSE(WIN32)
    MESSAGE("GenerateConfigScript works only on Unix/Linux/WIN32, sorry.")
  ENDIF(WIN32)
ENDIF(UNIX)

