/* 
This file is part of the BIAS library (Basic ImageAlgorithmS).

Copyright (C) 2003, 2004    (see file CONTACTS for details)
  Multimediale Systeme der Informationsverarbeitung
  Institut fuer Informatik
  Christian-Albrechts-Universitaet Kiel


BIAS is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or
(at your option) any later version.

BIAS is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with BIAS; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


#ifndef __HMatrix_hh__
#define __HMatrix_hh__
#include "bias_config.h" 

#include <Base/Geometry/HMatrixBase.hh>
#include <Base/Image/Image.hh>


namespace BIAS{

  /** @class HMatrix
      @ingroup g_geometry
      @brief a 3x3 Matrix describing projective transformations between planes
  */
  class BIASGeometry_EXPORT HMatrix : public HMatrixBase
  {
  public:
    HMatrix();

    explicit HMatrix(const MatrixInitType& i)
      : HMatrixBase(i){};

    HMatrix(Matrix3x3<double> Mat);
    
    ~HMatrix();

    HMatrix& operator=(const HMatrix& mat);

    /** maps the rectangle (0,0),(OrigWidth,OrigHeight) with this H
        and computes mapped range as corner + size. JW 07/2003 */
    void GetMappedRange(const unsigned int OrigWidth, 
                        const unsigned int OrigHeight,
                        BIAS::Vector2<double> & corner,
                        double & sizeWidth, double & sizeHeight) const;

    /** @return the squared distance between the mapped point from
        the first picture and the measured point in the second picture
        @author mdunad 02 2004 */
    double GetMappedError(const BIAS::HomgPoint2D &PointPicOne,
                          const BIAS::HomgPoint2D &PointPicTwo) const;

    /** @brief returns jacobian of H: R^2 --> R^2
        
        H as a mapping of 2D to 2D is not always a linear mapping, because of
        the nonlinear homogenization. This function returns the 2x2 Jacobian
        matrix at the point x. You have to pass homogenized points and a
        correctly sized jacobian

        @param x homogenized point (real point in R^2) for the jacobian
        @param Jac (output) 2x2 jacobian at x
        @author koeser 04/2005 */
    void GetJacobian(const HomgPoint2D& x, Matrix<double>& Jac) const;


  };


}// namespace

#endif // __HMatrix_hh__
