/* 
This file is part of the BIAS library (Basic ImageAlgorithmS).

Copyright (C) 2003, 2004    (see file CONTACTS for details)
  Multimediale Systeme der Informationsverarbeitung
  Institut fuer Informatik
  Christian-Albrechts-Universitaet Kiel


BIAS is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or
(at your option) any later version.

BIAS is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with BIAS; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include <Geometry/Pose.hh>
#include <Base/Common/CompareFloatingPoint.hh>

using namespace BIAS;
using namespace std;


BIAS::Pose::
Pose()
  : Cov_(7, 7, MatrixZero)
{}


BIAS::Pose::
~Pose() 
{}



Matrix3x3<POSE_TYPE> BIAS::Pose::
GetCovC() 
{
  Matrix3x3<POSE_TYPE> ret;
  for (unsigned int row=0; row<3; row++) {
    for (unsigned int col=0; col<3; col++) {
      ret[row][col] = Cov_[row][col];
    }
  }
  return ret;
}


Matrix4x4<POSE_TYPE> BIAS::Pose::
GetCovQ() 
{
  Matrix4x4<POSE_TYPE> ret;
  for (unsigned int row=0; row<4; row++) {
    for (unsigned int col=0; col<4; col++) {
      ret[row][col] = Cov_[row+3][col+3];
    }
  }
  return ret;
}   


void BIAS::Pose::
SetQ(const Quaternion<POSE_TYPE>& Q) 
{
  CoordinateTransform3D::SetQ(Q);
  Cov_.SetZero();
}

void BIAS::Pose::
SetC(const Vector3<POSE_TYPE>& C) 
{
  CoordinateTransform3D::SetC(C);
  Cov_.SetZero();
}

void BIAS::Pose::
Set(const Quaternion<POSE_TYPE>& Q, const Vector3<POSE_TYPE>& C) 
{
  CoordinateTransform3D::Set(Q, C);
  Cov_.SetZero();
}


int BIAS::Pose::
SetCovEuclidian(Matrix<POSE_TYPE> Cov) 
{
  if (Cov.GetRows()!=6 || Cov.GetCols()!=6) {
    BIASERR("Euclidian Pose Covariance has to be 7x7");
    return -1;
  } else {
    BIASERR("Not Implemented!!");
    BIASABORT;
    return -2;
  }
  
  //return 0;
}

int BIAS::Pose::
Set(const PoseParametrization& pose) 
{
  int ret=0;

  Set(pose.GetOrientation(), pose.GetPosition());
  ret -= SetCovariance(pose.GetCovarianceMatrix());
  return ret;
}


void BIAS::Pose::
Set(const Quaternion<POSE_TYPE>& Q, 
    const Vector3<POSE_TYPE>& C,
    const Matrix<POSE_TYPE>& Cov) 
{
  Set(Q, C);
  SetCovariance(Cov);
}

bool BIAS::Pose::
Load(const std::string& inputFile){
  bool res = CoordinateTransform3D::Load(inputFile);
  if(res && scale_!=1.0) {
    BIASERR("Loaded transformation is not a pose - resetting scale!\n");
    scale_ = 1.0;
    return false;
  }
  return res;
}

void BIAS::Pose::
TransformToNewGlobalBy(const CoordinateTransform3D& trsf)
{
  Pose result;
  CoordinateTransform3D intermediate;
  trsf.ConcatenateLocalTransform((*this), intermediate);
  result.Set(intermediate.GetQ(), intermediate.GetC());
  (*this) = result;
}

