#ifndef __TFTensorEstimation_hh__
#define __TFTensorEstimation_hh__

#include <Base/Common/BIASpragmaStart.hh>

#include "TrifocalTensor.hh"
#include <Base/Geometry/HomgPoint2D.hh>
#include <Base/Geometry/Normalization.hh>

#include <vector>

namespace BIAS
{
  /** class TFTensorEstimation
      @brief compute a trifocal tensor from image correspondences, not robust
      against outliers
      @author Matthias Dunda
      @ingroup g_estimation */
  class BIASGeometry_EXPORT TFTensorEstimation
  {
  public:
    TFTensorEstimation();
    ~TFTensorEstimation();

    /** @brief Estimate a trifocal tensor given correnspondences
        over three images.
        @param HartleyNormalization normalizes correspondences for
        each view according to Hartley
        @return !=0 on error
        @author Matthias Dunda 01/2004 */
    int Compute( BIAS::TrifocalTensor &T,
                 const std::vector<BIAS::HomgPoint2D> &x1,
                 const std::vector<BIAS::HomgPoint2D> &x2,
                 const std::vector<BIAS::HomgPoint2D> &x3,
                 bool HartleyNormalization = true );
  protected:
    BIAS::Normalization Norm_;
    int AlgebraicMinimization(BIAS::TrifocalTensor &Initial,
                              BIAS::TrifocalTensor &Result);
                              
  }; // class
} // namespace

#include <Base/Common/BIASpragmaEnd.hh>

#endif /* __TFTensorEstimation_hh__ */

