#ifndef __CameraViewController_h__
#define __CameraViewController_h__

// ----------------------------------------------------------
// @author Jan Woetzel   09/2003
#include <bias_config.h>
#ifndef BIAS_HAVE_OPENGL
#  error Please recompile BIAS with USE_OPENGL to use this lib. 
#endif // BIAS_HAVE_OPENGL

//#ifndef BIAS_HAVE_GLUT
//#  error Please recompile BIAS with USE_GLUT to use this lib. 
//#endif // BIAS_HAVE_GLUT

#ifndef BIAS_HAVE_WXWINDOWS
#  error Please recompile BIAS with USE_WXWINDOWS to use this lib. 
#endif // BIAS_HAVE_WXWINDOWS


// must be first
#include <Gui/biasgl.h>

// STD
#include <iostream>

// BIAS
#include <Base/Debug/Error.hh>
#include <Base/Debug/DebugSimple.hh>
#include <Base/Math/Vector2.hh>
#include <Base/Math/Vector3.hh>
#include <Geometry/PMatrix.hh>

#include <Gui/CameraViewExternals.hh>
#include <Gui/CameraViewInternals.hh>

#include <wx/wx.h>


namespace BIAS {

  enum Navigationmode {
    modeAxis1=1<<0
    ,modeAxis2=1<<1
    ,modeAxis3=1<<2
    //Mouse_Left
    //,Mouse_Right
    //ExamineTrackballmode
    //InspectionMoveCAbsolute=0
    //,InspectionMoveViewAtAbsolute
    //,InspectionModeRelative
    //,FlyMode
    //,PMatrixMode
  };


  /** \brief base for (interactively) manipulating a (virtual) camera view. 
  Typically used by OpenGL as a glulookat Update. 
  Navigator would be a good name, too.
  \author Jan Woetzel */
  class CameraViewController {
  public:
    virtual ~CameraViewController(); ///< destructor

    // ctors
    CameraViewController();

    /// any constructor should call this function 
    /// to initialize all data members (e.g. pointers to NULL)
    virtual void InitMembers();

    virtual void DisplayGL();
    virtual void DisplayGLInternal();
    virtual void DisplayGLExternal();

    /** map update of one axis (mouse wheel, delta, key...) 
    maps delta (e.g. mouse dragging motion or wheel) to pose update 
    depending on current mode JW */
    //virtual void Update(const unsigned int axisNo
    //  ,const double & delta
    //  ,const bool useCurrentMode=true
    //  ,const int alternativeMode = -1
    //  );

    //virtual void DisplayViewAt(const float & size=1.0f);

    //virtual void Set(BIAS::PMatrix & P,
    //  const unsigned int w,
    //  const unsigned int h,
    //  const float & zNear,
    //  const float & zFar );

    //void SetAspect();

    std::ostream & Print(std::ostream & os=std::cout );

    /** interaction area size */
    void SetSize(const BIAS::Vector2<double> & sz){
      this->size = sz; // dim
    };

    ///** toplevel entry for all motion controllers */
    //void Motion(const BIAS::Vector2<double> & pos,
    //  const bool & isStart=false);

    /** mouse start position, usually on leftclick */
    void SetStartPos(const BIAS::Vector2<double> & pos);


    inline CameraViewExternals GetExternals() const {
      BIASASSERT(p_externals!=NULL);
      return *p_externals;
    }
    inline CameraViewExternals & GetExternals(){
      BIASASSERT(p_externals!=NULL);
      return *p_externals;
    }

    inline CameraViewInternals GetInternals() const {
      BIASASSERT(p_internals!=NULL);
      return *p_internals;
    }
    inline CameraViewInternals & GetInternals(){
      BIASASSERT(p_internals!=NULL);
      return *p_internals;
    }

    //
    // different controller implementations: 
    // 

    /// simple translation
    void MotionTranslation(const BIAS::Vector2<double> & pos);

    void MotionTranslationForward(const BIAS::Vector2<double> & pos);

    void MotionTrackballRotation(const BIAS::Vector2<double> & pos);

    void MoveToCenter(const double & fraction);

    // deltaN is normalized 2D coord
    void MoveOrbit(const BIAS::Vector2<double> & deltaN);

  protected:
    BIAS::CameraViewExternals * p_externals;
    BIAS::CameraViewInternals * p_internals;

    // Navigationmode mode_;

    BIAS::Vector2<double>  size; // of interaction area, usually viewport

    BIAS::Vector2<double>  startpos; // origin   on click/start of motion
    CameraViewExternals startposExt; // internal on click/start of motion
    CameraViewInternals startposInt;

  }; // CameraViewController
} // namespace BIAS
#endif // __CameraViewController_h__
