#ifndef __CameraViewFrustum_h__
#define __CameraViewFrustum_h__

#include <bias_config.h>
#include <iostream>


namespace BIAS {

  /** \brief describes internal parameter Frustum of an (OpenGL) camera view

  \author Jan Woetzel 09/2003 */
  class BIASGui_EXPORT CameraViewFrustum {
  public:
    virtual ~CameraViewFrustum(){}; ///< destructor

    /// ctors
    CameraViewFrustum();

    /// sets a symmetric, centered frustum like gluPerspective does
    /// fovYdeg is the y field of view in degree.
    void SetPerspective(const double & fovYdeg, 
      const double & fovAspect,
      const double & zNear=1., 
      const double & zFar=1000.);


    /// @return the approximate field of view in degree
    double GetFovY() const;
    double GetFovX() const;

    /// @return aspect ratio x/y of field of image size (NOT field of view)
    double GetAspect() const;


    std::ostream & Print(std::ostream & os=std::cout ) const;

#ifdef BIAS_HAVE_OPENGL
    void DisplayGL() const;
#endif // BIAS_HAVE_OPENGL

    void InitMembers(){
      this->zNear = 0.1;
      this->zFar  = 1000.;

      this->left   = -1. *zNear;
      this->right  = -left;
      this->bottom = -1. *zNear;
      this->top    = -bottom;
    }

  public:
    double left;
    double right;
    double bottom;
    double top;

    double zNear;
    double zFar;

  }; // CameraViewFrustum
} // namespace BIAS
#endif
