#include <wx/wx.h>

#include <Gui/ImageCanvas.hh>
#include <Gui/StringConv.hh>
#include <Base/Image/ImageIO.hh>

using namespace BIAS;
using namespace std;

enum { ID_Quit = 1, ID_About, ID_Load};

class MyApp: public wxApp
{
  virtual bool OnInit();
};

IMPLEMENT_APP(MyApp)

class MyFrame: public wxFrame
{
public:

  MyFrame(const wxString& title, const wxPoint& pos, const wxSize& size);

  void OnQuit(wxCommandEvent& event);
  void OnAbout(wxCommandEvent& event);
  void OnLoad(wxCommandEvent& event);

  ImageCanvas *_ic;

  DECLARE_EVENT_TABLE()
};

bool MyApp::OnInit()
{
  MyFrame *frame = new MyFrame( wxT("ImageCanvas"), wxPoint(50,50), 
                                wxSize(450,340) );
  frame->Show(TRUE);
  SetTopWindow(frame);
  return TRUE;
} 

BEGIN_EVENT_TABLE(MyFrame, wxFrame)
  EVT_MENU(ID_Quit, MyFrame::OnQuit)
  EVT_MENU(ID_About, MyFrame::OnAbout)
  EVT_MENU(ID_Load, MyFrame::OnLoad)
END_EVENT_TABLE()


MyFrame::MyFrame(const wxString& title, const wxPoint& pos, const wxSize& size)
  : wxFrame((wxFrame *)NULL, -1, title, pos, size)
{
  wxMenu *menuFile = new wxMenu;
  
  menuFile->Append( ID_About, wxT("&About...") );
  menuFile->Append( ID_Load, wxT("&Load...") );
  menuFile->AppendSeparator();
  menuFile->Append( ID_Quit, wxT("E&xit") );

  wxMenuBar *menuBar = new wxMenuBar;
  menuBar->Append(menuFile, wxT("&File"));
  SetMenuBar(menuBar);

  CreateStatusBar(2);
  SetStatusText(wxT("ImageName"), 0);

  wxBoxSizer *box=new wxBoxSizer(wxVERTICAL);
  _ic=new ImageCanvas(this, GetStatusBar(), 1);
  box->Add(_ic, 1, wxALL| wxGROW |wxALIGN_CENTER, 5);
  box->SetSizeHints(this);
  SetSizer(box);

  Image<unsigned char> im;
  ImageIO::Load("ucim.pgm", im);
  SetStatusText(wxT("ucim.pgm"), 0);
  _ic->Show(im, "ucim.pgm");
  FitInside();
}

void MyFrame::OnQuit(wxCommandEvent& WXUNUSED(event))
{ Close(TRUE); }

void MyFrame::OnAbout(wxCommandEvent& WXUNUSED(event))
{
  wxMessageBox(wxT("This is a wxWindows Hello world sample"),
             wxT("About Hello World"), wxOK | wxICON_INFORMATION, this);
}

void MyFrame::OnLoad(wxCommandEvent& WXUNUSED(event))
{
  wxFileDialog fileDialog(this, wxT("Choose a file"), wxT("."), wxT("ucim.pgm"));
  if (fileDialog.ShowModal() == wxID_OK){
    string fname=WxToAscii(fileDialog.GetPath());
    Image<unsigned char> im;
    if (ImageIO::Load(fname, im)!=0){
      BIASERR("errro loading "<<fname);
    } else {
      SetStatusText(fileDialog.GetFilename(), 0);
      _ic->Show(im, WxToAscii(fileDialog.GetFilename()));
    }
  } else {
    cerr << "canceled loadin image" << endl;
  }
}

