#ifndef __ProjectionGUI_hh__
#define __ProjectionGUI_hh__

#include <bias_config.h>

#ifndef BIAS_HAVE_WXWINDOWS
#  error BIAS_HAVE_WXWINDOWS not defined but required by ProjectionGUI. Please enable BIAS_HAVE_WXWINDOWS in CMake configure step.
#endif

#include <wx/wx.h>
#include <Geometry/Projection.hh>
#include <vector>

#include <Base/Math/Vector.hh>
#include <Base/Math/Vector2.hh>

namespace BIAS {

  /** @brief Panel for displaying and editing vectors.
      @author jkollmann */
  class BIASGui_EXPORT VectorPanel : public wxPanel
  {
  public:
    VectorPanel(wxWindow* parent, wxWindowID id, bool editable);
    void SetVector(const BIAS::Vector<double>& vector);
    BIAS::Vector<double> GetVector() const;

  private:
    bool editable_;
    std::vector<wxTextCtrl*> controls_;
  };

  /** @brief Panel for displaying and editing matrices.
      @author jkollmann */
  class BIASGui_EXPORT MatrixPanel : public wxPanel
  {
  public:
    MatrixPanel(wxWindow* parent, wxWindowID id, bool editable);
    void SetMatrix(const Matrix<double>& matrix);
    Matrix<double> GetMatrix();

  private:
    bool editable_;
    int rows_, cols_;
    std::vector<wxTextCtrl*> controls_;
  };

  /** @brief Panel for displaying and editing projections.
      @author jkollmann */
  class ProjectionGUI : public wxPanel
  {
  public:
    ProjectionGUI(wxWindow* parent, wxWindowID id = -1,
                  const wxPoint& pos = wxDefaultPosition,
                  const wxSize& size = wxDefaultSize,
                  long style = wxTAB_TRAVERSAL,
                  const wxString& name = wxT("panel"));
    void SetProjection(const Projection& projection);
    const Projection& GetProjection() const;

  private:
    VectorPanel* AddVectorPanel(const wxString& name, wxWindowID id, bool editable);
    MatrixPanel* AddMatrixPanel(const wxString& name, wxWindowID id, bool editable);

    void UpdateGUI();

    void OnNormalizeQuaternion(wxCommandEvent& event);
    void OnModified(wxCommandEvent& event);
    void OnNew(wxCommandEvent& event);
    void OnLoad(wxCommandEvent& event);
    void OnSave(wxCommandEvent& event);

    bool selfModifying_;
    Projection projection_;
    wxScrolledWindow* projectionPanel_;

    VectorPanel* panelCenter_;
    VectorPanel* panelQuaternion_;
    MatrixPanel* panelRMatrix_;
    VectorPanel* panelImageSize_;
    VectorPanel* panelAspectRatio_;
    VectorPanel* panelPrincipal_;
    MatrixPanel* panelKMatrix_;
    VectorPanel* panelFocalLength_;
    VectorPanel* panelUndistortion_;
    VectorPanel* panelRadius_;

    DECLARE_EVENT_TABLE()
  };

  /** @brief Dialog containing a ProjectionGUI.
      @author jkollmann */
   class BIASGui_EXPORT ProjectionDialog : public wxDialog
   {
   public:
    ProjectionDialog(const Projection& projection,
                     wxWindow* parent = NULL, wxWindowID id = -1,
                     const wxString& title = wxT("Projection"));
    const Projection& GetProjection() const;
   private:
    ProjectionGUI* projgui_;
   };

} // namespace BIAS

#endif // __ProjectionGUI_hh__
