#ifndef __StringConv_hh__
#define __StringConv_hh__

#include <wx/string.h>
#include <string>

namespace BIAS {

/*
inline const char* WxToAscii(const wxString& string)
{
  return (const char*)string.mb_str(wxConvUTF8);
}
*/

/// Converts a wxString to a std::string.
#define WxToAscii(s) (const char*)(wxString(s).mb_str(wxConvUTF8))

/// Converts a C string to a wxString.
inline wxString AsciiToWx(const char* string)
{
  return wxString(string, wxConvUTF8);
}

/// Converts a std::string to a wxString.
inline wxString AsciiToWx(const std::string& string)
{
  return wxString(string.c_str(), wxConvUTF8);
}
/**
  Class for converting an array of wxStrings to an array
  of non-const ASCII strings. Required for using getopt
  in UNICODE wx tools.
*/
class WxToAsciiArray
{
public:
  WxToAsciiArray(wxChar** strings, int size)
  {
    _AsciiStrings = new char*[size];
    for (int i = 0; i < size; i++)
    {
      _AsciiStrings[i] = strdup(WxToAscii(strings[i]));
    }
    _Size = size;
  }

  ~WxToAsciiArray()
  {
    for (int i = 0; i < _Size; i++)
    {
      free(_AsciiStrings[i]);
    }
    delete[] _AsciiStrings;
  }

  operator char**()
  {
    return _AsciiStrings;
  }

private:
  char** _AsciiStrings;
  int    _Size;
};

} // namespace BIAS

#endif // __StringConv_hh__
