#ifndef _glcheck_h_
#define _glcheck_h_

/** macros to check gl status.

To be used like this:
\verbatim
  CHECK_GL:ERROR
\endverbatim
author: Jan Woetzel
*/

#include <bias_config.h>

#ifndef BIAS_HAVE_OPENGL
#  error Please recompile BIAS with USE_OPENGL to use OpenGL.
#endif // BIAS_HAVE_OPENGL 

#include <Gui/biasgl.h>
#include <GL/glu.h> // for gluErrorString



// BIAS ERR stream: 
#include <Base/Debug/Error.hh>

// STD for output of macros:   
#include <string>
#include <sstream>
#include <iostream>


// ---------------------------------------------------
///use this macro in your code to check for OpenGL errors with "click+jump" 
///@author Jan Woetzel
#if defined(BIAS_DEBUG) && defined(WIN32)

#define CHECK_GL_ERROR \
{\
  GLenum lastErr = glGetError(); \
  if (lastErr!=GL_NO_ERROR) { \
    std::stringstream ss; \
    ss<<std::endl<<__FILE__<<"("<<__LINE__<<") : "<<FUNCNAME<<std::endl \
      <<"\tOpenGL Error: "<<(char *)gluErrorString(lastErr)<<std::endl; \
    ERROR_STREAM<<ss.str(); \
    OutputDebugString(ss.str().c_str()); \
  } \
} 

#elif defined(BIAS_DEBUG) 

#define CHECK_GL_ERROR \
{\
  GLenum lastErr = glGetError(); \
  if (lastErr!=GL_NO_ERROR) { \
    std::stringstream ss; \
    ss<<std::endl<<__FILE__<<":"<<__LINE__<<" : "<<FUNCNAME<<std::endl \
      <<"\tOpenGL Error: "<<(char *)gluErrorString(lastErr)<<std::endl; \
    ERROR_STREAM<<ss.str(); \
  } \
} 
#else
// release: no output 
#  define CHECK_GL_ERROR 
#endif



#endif // _glcheck_h_ 
// DO NOT ADD CODE AFTER THIS LINE

