/*
$Id: CameraParam.cpp,v 2.29 2006/05/05 11:45:50 streckel Exp $
*/
/* 
This file is part of the BIAS library (Basic ImageAlgorithmS).

Copyright (C) 2003, 2004    (see file CONTACTS for details)
  Multimediale Systeme der Informationsverarbeitung
  Institut fuer Informatik
  Christian-Albrechts-Universitaet Kiel


BIAS is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or
(at your option) any later version.

BIAS is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with BIAS; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#include "CameraParam.hh"
#include <iostream>
#include <math.h>
#include <Base/Image/ImageAttributes.hh>

using namespace BIAS;
using namespace std;
  

/*
std::string CameraParam::CameraModelName[]={
  "SONY DFW-X700",
  "SONY DFW-V500",
  "PTGREY DRAGONFLY COLOR",
  "SONY EVI-D31",
  "CAMERA_SONY_DV",
  "UNKNOWN",
};
   
std::string CameraParam::LensModelName[]={
  "SONY EVID31 BUILTIN",
  "SONY DV BUILTIN",
  "PENTAX CMOUNT 4.8",
  "PENTAX CMOUNT 8.0",
  "PENTAX CMOUNT 4.2",
};
*/


CameraParam::CameraParam() {
  Init();
}


CameraParam::CameraParam(const CameraParam &cp) {
  Init(cp);
}


#ifdef BIAS_HAVE_XML2
CameraParam::CameraParam(const std::string &filename)
{
  IsInitialized_=false;
  Init(filename);
}
#endif //BIAS_HAVE_XML2


#ifdef BIAS_HAVE_XML2
CameraParam::CameraParam(XMLIO &myXML, xmlNodePtr &cameraNode){
  IsInitialized_=false;
  Init(myXML, cameraNode);
}
#endif //BIAS_HAVE_XML2


CameraParam::~CameraParam(){
}


void CameraParam::Init()
{
  MaxCamAngle_=0;
  CamModel_ = "Default Initialization";
  CamID_ = 1;
  LensModel_ = "Default Initialization";
  LensIsSpherical_ = false;

  width_=height_=0;
  aspectratio_ = 1;
  focallengthDef_ = 0;
  kc1Def_ = kc2Def_ = kc3Def_ = kc4Def_ = 0;
  cellSizeX_ = cellSizeY_ = 0;
  principalX_ = principalY_ =0;
  IlluCorrX_.clear();
  IlluCorrY_.clear();
  knownparams_vect_.clear();
  cellSizeX_ = cellSizeY_ = 1;
  zoomCamera_=false;  
  AngleCorrX_.clear();
  AngleCorrY_.clear();
  IlluCorrX_.clear();
  IlluCorrY_.clear(); 
  radius_ = 0;
  IsInitialized_=false;
  zoom_ = 0;
  focallengthZoom_ = 0;
  downsamplingFactor_ = 1.0;
}

void CameraParam::Init(const CameraParam &cp) {
  LensIsSpherical_ = cp.LensIsSpherical_;
  IsInitialized_ = cp.IsInitialized_;
  CamModel_ = cp.CamModel_;
  CamID_ = cp.CamID_;
  LensModel_ = cp.LensModel_;
  LensIsSpherical_ = cp.LensIsSpherical_;
  zoomCamera_ = cp.zoomCamera_;
  width_ = cp.width_;
  height_ = cp.height_;
  knownparams_vect_ = cp.knownparams_vect_;
  aspectratio_ = cp.aspectratio_;
  cellSizeX_ = cp.cellSizeX_;
  cellSizeY_ = cp.cellSizeY_;
  principalX_ = cp.principalX_;
  principalY_ = cp.principalY_;
  focallengthDef_ = cp.focallengthDef_;
  kc1Def_ = cp.kc1Def_;
  kc2Def_ = cp.kc2Def_;
  kc3Def_ = cp.kc3Def_;
  kc4Def_ = cp.kc4Def_;
  IlluCorrX_ = cp.IlluCorrX_;
  IlluCorrY_ = cp.IlluCorrY_;
  AngleCorrX_ = cp.AngleCorrX_;
  AngleCorrY_ = cp.AngleCorrY_;
  if (AngleCorrX_.size()>0) 
    MaxCamAngle_ = AngleCorrX_[AngleCorrX_.size()-1];
  else 
    MaxCamAngle_ = 0;
  radius_ = cp.radius_;
  downsamplingFactor_ = cp.downsamplingFactor_;
  if (zoomCamera_) {
    zoom_ =  knownparams_vect_[0].zoom;
    focallengthZoom_ = knownparams_vect_[0].focallength;
  }
}

#ifdef BIAS_HAVE_XML2
int CameraParam::Init(const std::string &filename) {
  int res = ReadCameraData(filename);
  if (res <0) return res;
  IsInitialized_=true;
  return 0;
}
#endif //BIAS_HAVE_XML2


#ifdef BIAS_HAVE_XML2
int CameraParam::Init(XMLIO &myXML, xmlNodePtr &cameraNode)
{
  int res = ReadCameraData(myXML, cameraNode);
  if (res <0) return res;
  IsInitialized_=true;
  return 0;
}
#endif //BIAS_HAVE_XML2


bool CameraParam::IsInitialized() const{
    return IsInitialized_;
}

KMatrix CameraParam::GetKMatrix() const {
  KMatrix K;
  K.SetIdentity();
  if (IsPerspective()) {
    K[0][0] = GetFocalLength();
    K[1][1] = aspectratio_ * GetFocalLength();
    K[0][1] = 0; //skew_;
    K[0][2] = principalX_;
    K[1][2] = principalY_;
  } else {
    BIASERR("No perspective camera-> No K-Matrix!");
  }
  return K;
} 


void CameraParam::SetPrincipal(double x, double y) {
  principalX_ = x;
  principalY_ = y;
}

double CameraParam::GetPrincipalX() const {
  if (!IsInitialized_){
    BIASERR("Not inited!");
    return 0;
  }
  //  cout<<"Get PrincipalX returns: "<<principalX_/downsamplingFactor_+0.000000001<<endl;
  return principalX_/downsamplingFactor_+0.000000001;
}

double CameraParam::GetPrincipalY() const {
  if (!IsInitialized_){
    BIASERR("Not inited!");
    return 0;
  }
  //  cout<<"Get PrincipalY returns: "<<principalY_/downsamplingFactor_+0.000000001<<endl;
  return principalY_/downsamplingFactor_+0.000000001;
}

unsigned int CameraParam::GetHeight() const {
  if (!IsInitialized_){
    BIASERR("Not inited!");
    return 0;
  }
  return (int)(height_/downsamplingFactor_+0.000000001);
}

unsigned int CameraParam::GetWidth() const {
  if (!IsInitialized_){
    BIASERR("Not inited!");
    return 0;
  }
  return (int)(width_/downsamplingFactor_+0.000000001);
}

void CameraParam::SetCellSize(const double CellSizeX, 
                              const double AspectRatio) {
  cellSizeX_ = CellSizeX;
  if (AspectRatio!=0) {
    cellSizeY_ = CellSizeX / AspectRatio;
    aspectratio_ = AspectRatio;
  } else {
    cellSizeY_ = CellSizeX;
    aspectratio_ = 1;
  }
}

void CameraParam::SetZoomCamera(const bool zoomCam) {
  zoomCamera_=zoomCam;
}


bool CameraParam::IsZoomCamera() const  {
  if (!IsInitialized_){
    BIASERR("Not inited!");
    return false;
  }
  return zoomCamera_;
}

double CameraParam::GetZoom() const {
  if (!IsInitialized_){
    BIASERR("Not inited!");
    return -1;
  }
  if (IsPerspective()) {
    if (zoomCamera_) {
      return zoom_;
    } 
    return 0;
  } else {
    BIASERR("No perspective camera!");
    return -1;
  }
}

double CameraParam::GetFocalLength() const {
  if (!IsInitialized_){
    BIASERR("Not inited!");
    return -1;
  }
  if (IsPerspective()) {
    if (zoomCamera_) {
      return focallengthZoom_/downsamplingFactor_+0.000000001;
    }
    return focallengthDef_/downsamplingFactor_+0.000000001;
  } else {
    BIASERR("No perspective camera!");
    return -1;
  }
}


double CameraParam::GetDownsamplingFactor(){
  if (!IsInitialized_){
  	BIASERR("Not inited!");
  	return -1.0;
  } 
  return downsamplingFactor_;
}

void CameraParam::SetZoom(const double abszoom){
  if (!IsInitialized_){
    BIASERR("Not inited!");
    return;
  }
  if (zoomCamera_ && IsPerspective()) {
    zoom_=abszoom;      
    focallengthZoom_ = InterpolateFocalLength_();
    cout << "interpolating zoom:" << abszoom << " to: " <<
      focallengthZoom_ << endl;
  } else {
    BIASERR("No perspective/zoom camera!");
    return;
  }
}


void CameraParam::SetFocalLength(const double focallength){
  if (!IsInitialized_){
    BIASERR("Not inited!");
    return;
  }
  if (IsPerspective()) 
	  if (zoomCamera_){
		  focallengthZoom_=focallength;
		  zoom_=InterpolateAbsZoom_();
	  }
	  else
		  focallengthDef_ = focallength;
  else 
    BIASERR("No perspective camera!");
}

void CameraParam::SetDownsampling(const double factor) {
  downsamplingFactor_ = factor;
}

void CameraParam::SetUnzoomed(){
  if (!IsInitialized_){
    BIASERR("Not inited!");
    return;
  }
  if (zoomCamera_ && IsPerspective()) {
    if (focallengthDef_==0) {
      zoom_=knownparams_vect_[0].zoom;
      focallengthZoom_=knownparams_vect_[0].focallength;
    } else {
      zoom_=0;
      focallengthZoom_=focallengthDef_;
    }
  }
}


#ifdef BIAS_HAVE_XML2
int CameraParam::WriteCameraData(const std::string& Filename) const {

  if (!IsInitialized_){
    BIASERR("Not inited!");
    return -1;
  }

  XMLIO myXML;
  xmlNodePtr rootNode;
  rootNode = myXML.create("camera");
  myXML.addAttribute(rootNode, "version", 
                     CAMERA_PARAMETER_CURRENT_XML_VERSION);
  //  myXML.addAttribute(rootNode, "name", CamModel_);
  AddCameraDataToXMLTree(myXML, rootNode);
  myXML.write(Filename); 
  return 0;

}
#endif //BIAS_HAVE_XML2


#ifdef BIAS_HAVE_XML2
int CameraParam::AddCameraDataToRig(XMLIO &myXML, 
                                    xmlNodePtr &cameraDataNode) const {

  if (!IsInitialized_){
    BIASERR("Not inited!");
    return -1;
  }
  xmlNodePtr childNode;
  childNode = myXML.addChildNode(cameraDataNode, "camera");
  myXML.addAttribute(childNode, "version", 
      std::string("2.0") );
  //  myXML.addAttribute(childNode, "name", CamModel_);
  AddCameraDataToXMLTree(myXML, childNode);
  return 0;

}
#endif //BIAS_HAVE_XML2


#ifdef BIAS_HAVE_XML2
int CameraParam::AddCameraDataToXMLTree(XMLIO &myXML, 
                                       xmlNodePtr &rootNode) const {


  if (!IsInitialized_){
    BIASERR("Not inited!");
    return -1;
  }

  unsigned int i;
  xmlNodePtr  childNode2, childNode3, childNode4;
  myXML.addComment(rootNode, "\"Model\" is an arbitrary camera name.\n \"SerialID\" is i.e. the camera firewire-bus ID.\n \"ImageWidth\" and \"ImageHeight\" are in pixel.\n \"CellSizeX\" is the size of one CCD-cell in meter.\n \"AspectRatio\" is CellsizeX/CellSizeY (same as FocalLengthY/FocalLengthX).");
  childNode2 = myXML.addChildNode(rootNode, "Sensor");
  myXML.addAttribute(childNode2, "Model", CamModel_ );
  myXML.addAttribute(childNode2, "SerialID", CamID_);
  myXML.addAttribute(childNode2, "ImageWidth", (int) width_);
  myXML.addAttribute(childNode2, "ImageHeight", (int) height_);
  myXML.addAttribute(childNode2, "CellSizeX", cellSizeX_);
  myXML.addAttribute(childNode2, "AspectRatio", aspectratio_);
  myXML.addComment(rootNode, "\"Model\" is an arbitrary name.\n \"Type\" can be either \"Spherical\" or \"Perspective\"");
  childNode2 = myXML.addChildNode(rootNode, "Lens");
  myXML.addAttribute(childNode2, "Model", LensModel_ );
  string LensType="";
  if (LensIsSpherical_) {
    LensType="Spherical";
  } else {
    LensType="Perspective";
  }
  myXML.addAttribute(childNode2, "Type", LensType);
  if (LensIsSpherical_) {
    childNode3 = myXML.addChildNode(childNode2, "AngleCalib");
    myXML.addComment(childNode3, "Measurement points of angle calibration curve.\n \"AngRay\" is the angle of the viewing ray, \"AngDeformed\" is the Angle deformed by the camera.\n The maximium \"AngRay\" is the cameras FoV.");
    for (i=0; i<AngleCorrX_.size(); i++) {
      childNode4 = myXML.addChildNode(childNode3, "MeasPt");
      myXML.addAttribute(childNode4, "AngRay", AngleCorrX_[i]);
      myXML.addAttribute(childNode4, "AngDeformed", AngleCorrY_[i]);
    }
  } else {
    childNode3 = myXML.addChildNode(childNode2, "PerspectiveCalib");
    myXML.addComment(childNode3, "\"FocalLength\" is FocalLengthX in meters, if you want to give it in pixel set \"CellSizeX\"=1.\n\"k1\" and \"k2\" are radial distortion; \"p1\", \"p2\" are tangential distortion.\n Camera calibration constants coming from the matlab toolbox 'http://www.vision.caltech.edu/bouguetj/calib_doc/'.\n Distortion is calculated as x'=x*(1+k1*r^2+k2*r^4)+2*p1*x*y+p2*(r^2+2*x^2) and y'=y*(1+k1*r^2+k2*r^4)+2*p1*(r^2+2*y^2)+2*p2*x*y with r^2=(x^2+y^2).");
    if (focallengthDef_!=0) {
      childNode4 = myXML.addChildNode(childNode3, "Default");
      //convert to mm
      double fl = focallengthDef_ * cellSizeX_; 
      myXML.addAttribute(childNode4, "FocalLength", fl);
      myXML.addAttribute(childNode4, "k1", kc1Def_);
      myXML.addAttribute(childNode4, "k2", kc2Def_);
      myXML.addAttribute(childNode4, "p1", kc3Def_);
      myXML.addAttribute(childNode4, "p2", kc4Def_);
    }
    for (i=0; i<knownparams_vect_.size(); i++) {
      childNode4 = myXML.addChildNode(childNode3, "ZoomStep");
      myXML.addAttribute(childNode4, "Zoom", knownparams_vect_[i].zoom);
      //convert to mm
      double fl = knownparams_vect_[i].focallength * cellSizeX_; 
      myXML.addAttribute(childNode4, "FocalLength", fl);
  //       myXML.addComment(childNode2,"Focal length in meter, if you want to use the pixel value set pixelSpacing to 1.");
    myXML.addAttribute(childNode4, "k1", knownparams_vect_[i].kc1);
      myXML.addAttribute(childNode4, "k2", knownparams_vect_[i].kc2);
      myXML.addAttribute(childNode4, "p1", knownparams_vect_[i].kc3);
      myXML.addAttribute(childNode4, "p2", knownparams_vect_[i].kc4);
    }
  }
  childNode3 = myXML.addChildNode(childNode2, "VignetteCalib");
  if (LensIsSpherical_) {
    myXML.addComment(childNode3, "The measurements for vignette calibration relate the angle of the viewing ray to an absorbtion factor for the corresponding image pixels.");
    for (i=0; i<IlluCorrX_.size(); i++) {
      childNode4 = myXML.addChildNode(childNode3, "MeasPt");
      myXML.addAttribute(childNode4, "AngRay", IlluCorrX_[i]);
      myXML.addAttribute(childNode4, "Fac", IlluCorrY_[i]);
    }
  } else {
    myXML.addComment(childNode3, "The measurements for vignette calibration relate the distance of a pixel from the image principal point to an absorbtion factor for the corresponding image pixels.");
    for (i=0; i<IlluCorrX_.size(); i++) {
      childNode4 = myXML.addChildNode(childNode3, "MeasPt");
      myXML.addAttribute(childNode4, "Dist", IlluCorrX_[i]);
      myXML.addAttribute(childNode4, "Fac", IlluCorrY_[i]);
    }
  }
   
  myXML.addComment(rootNode, "The \"PrincipalPoint\" is given as a fraction of \"ImageWidth\"/\"ImageHeight\" with (0.5,0.5) is the image center.");
  childNode2 = myXML.addChildNode(rootNode, "PrincipalPoint");
  myXML.addAttribute(childNode2, "X", principalX_/width_);
  myXML.addAttribute(childNode2, "Y", principalY_/height_);
  if (LensIsSpherical_) {
    myXML.addComment(childNode2, "The \"SphereRadius\" is given as a fraction of \"ImageWidth\".");
    myXML.addAttribute(childNode2, "SphereRadius", radius_/width_);
  }
  
  return 0;


 //   unsigned int i;
//   xmlNodePtr  childNode2, childNode3, childNode4, childNode5, childNode6;
//   myXML.addComment(rootNode,"e.g. ID from the firewire-bus");
//   childNode2 = myXML.addChildNode(rootNode, "id");
//   myXML.addContent(childNode2, CamID_);
    
//   childNode2 = myXML.addChildNode(rootNode, "sensor");

//   childNode3 = myXML.addChildNode(childNode2, "numberOfPixels");
//   childNode4 = myXML.addChildNode(childNode3, "x");
//   myXML.addContent(childNode4, (int)width_);
//   childNode4 = myXML.addChildNode(childNode3, "y");
//   myXML.addContent(childNode4, (int)height_);
//   childNode3 = myXML.addChildNode(childNode2, "pixelSpacing");
//   myXML.addComment(childNode3,"Size of the CCD-pixels in meter, if x and y differ, the aspect ratio is set to x/y.");
//   childNode4 = myXML.addChildNode(childNode3, "x");
//   myXML.addContent(childNode4, cellSizeX_);
//   childNode4 = myXML.addChildNode(childNode3, "y");
//   myXML.addContent(childNode4, cellSizeY_);
//   childNode3 = myXML.addChildNode(childNode2, "principalPoint");
//   myXML.addComment(childNode3,"Principal point relative to image center in meter.");
//   // convert to meter
//   double px = (principalX_-width_/2.0) * cellSizeX_;
//   double py = (principalY_-height_/2.0) * cellSizeY_;
//   childNode4 = myXML.addChildNode(childNode3, "x");
//   myXML.addContent(childNode4, px);
//   childNode4 = myXML.addChildNode(childNode3, "y");
//   myXML.addContent(childNode4, py);

//   if (LensIsSpherical_) {
//     childNode2 = myXML.addChildNode(rootNode, "sphericalLens");
//     myXML.addAttribute(childNode2, "name", LensModel_);
//     childNode3 = myXML.addChildNode(childNode2, "angleCalibration");
//     for (i=0; i<AngleCorrX_.size(); i++) {   
//       childNode4 = myXML.addChildNode(childNode3, "measPoint");
//       childNode5 = myXML.addChildNode(childNode4, "angleRay");
//       myXML.addContent(childNode5, AngleCorrX_[i]);
//       childNode5 = myXML.addChildNode(childNode4, "angleDeformed");
//       myXML.addContent(childNode5, AngleCorrY_[i]);
//     }
//     childNode3 = myXML.addChildNode(childNode2, "vignetteCalibration");
//     for (i=0; i<IlluCorrX_.size(); i++) {
//       childNode4 = myXML.addChildNode(childNode3, "measPoint");
//       childNode5 = myXML.addChildNode(childNode4, "angleRay");
//       myXML.addContent(childNode5, IlluCorrX_[i]);
//       childNode5 = myXML.addChildNode(childNode4, "factor");
//       myXML.addContent(childNode5, IlluCorrY_[i]);
//     }
//     childNode3 = myXML.addChildNode(childNode2, "sphereRadius");
//     myXML.addContent(childNode3, radius_);
//   } else {
//     if (knownparams_vect_.size()==0) {
//       childNode2 = myXML.addChildNode(rootNode, "lens");
//       myXML.addAttribute(childNode2, "name", LensModel_);
//       myXML.addComment(childNode2,"Focal length in meter, if you want to use the pixel value set pixelSpacing to 1.");
//       childNode3 = myXML.addChildNode(childNode2, "focalLength");

//       //convert to Pixel
//       double fl = focallengthDef_ * cellSizeX_; 
//       myXML.addContent(childNode3, fl);
//       childNode3 = myXML.addChildNode(childNode2, "distortion");
//       myXML.addComment(childNode3,"Distortion is calculated as x'=x*(1+k1*r^2+k2*r^4)+2*p1*x*y+p2*(r^2+2*x^2) and y'=y*(1+k1*r^2+k2*r^4)+2*p1*(r^2+2*y^2)+2*p2*x*y with r^2=(x^2+y^2)");
//       childNode4 = myXML.addChildNode(childNode3, "radial");
//       childNode5 = myXML.addChildNode(childNode4, "k1");
//       myXML.addContent(childNode5, kc1Def_);
//       childNode5 = myXML.addChildNode(childNode4, "k2");
//       myXML.addContent(childNode5, kc2Def_);
//       childNode4 = myXML.addChildNode(childNode3, "tangential");
//       childNode5 = myXML.addChildNode(childNode4, "p1");
//       myXML.addContent(childNode5, kc3Def_);
//       childNode5 = myXML.addChildNode(childNode4, "p2");
//       myXML.addContent(childNode5, kc4Def_);
//     } else {
//       childNode2 = myXML.addChildNode(rootNode, "zoomLens");
//       myXML.addAttribute(childNode2, "name", LensModel_);
//       for (i=0; i<knownparams_vect_.size(); i++) {
//         childNode3 = myXML.addChildNode(childNode2, "zoomStep");
//         childNode4 = myXML.addChildNode(childNode3, "zoom");
//         myXML.addContent(childNode4, knownparams_vect_[i].zoom);
//         childNode4 = myXML.addChildNode(childNode3, "focalLength");
//         //convert to Pixel
//         double fl = knownparams_vect_[i].focallength * cellSizeX_;
//         myXML.addContent(childNode4, fl);
//         childNode4 = myXML.addChildNode(childNode3, "distortion");
//         if (i==0) {
//           myXML.addComment(childNode4,"Distortion is calculated as x'=x*(1+k1*r^2+k2*r^4)+2*p1*x*y+p2*(r^2+2*x^2) and y'=y*(1+k1*r^2+k2*r^4)+2*p1*(r^2+2*y^2)+2*p2*x*y with r^2=(x^2+y^2)");
//         }
//         childNode5 = myXML.addChildNode(childNode4, "radial");
//         childNode6 = myXML.addChildNode(childNode5, "k1");
//         myXML.addContent(childNode6, knownparams_vect_[i].kc1);
//         childNode6 = myXML.addChildNode(childNode5, "k2");
//         myXML.addContent(childNode6, knownparams_vect_[i].kc2);
//         childNode5 = myXML.addChildNode(childNode4, "tangential");
//         childNode6 = myXML.addChildNode(childNode5, "p1");
//         myXML.addContent(childNode6, knownparams_vect_[i].kc3);
//         childNode6 = myXML.addChildNode(childNode5, "p2");
//         myXML.addContent(childNode6, knownparams_vect_[i].kc4);
//       }
//     }
 //    myXML.addComment(childNode2,"Used only for BBC software.");
//     childNode3 = myXML.addChildNode(childNode2, "iris");
//     myXML.addContent(childNode3, iris_);
//     myXML.addComment(childNode2,"Used only for BBC software.");
//     childNode3 = myXML.addChildNode(childNode2, "reciprocalFocus");
//     myXML.addContent(childNode3, reciprocalFocus_);
//  }
//   myXML.addComment(rootNode,"Used only for BBC software.");
//   childNode2 = myXML.addChildNode(rootNode, "exposureTime");
//   myXML.addContent(childNode2, exposureTime_);  
//  return 0;
}
#endif //BIAS_HAVE_XML2


#ifdef BIAS_HAVE_XML2
int CameraParam::ReadCameraData(const std::string& Filename) {

  XMLIO myXML;
  xmlNodePtr rootNode;
  rootNode = myXML.read(Filename);
  if (rootNode==NULL) {
    BIASERR("Camera-Parameter-File not found or parse error: " << Filename);
    return -1;
  }
 
  if (myXML.getNodeName(rootNode)!="camera" &&
      myXML.getNodeName(rootNode)!="Camera") {
    BIASERR("Root node has to be \"camera\"");
    return -1;
  }
  ReadCameraData (myXML, rootNode);

  return 0;
}
#endif //BIAS_HAVE_XML2


#ifdef BIAS_HAVE_XML2

int CameraParam::ReadCameraData(XMLIO &myXML, xmlNodePtr &cameraNode) {

  //reset this
  Init();
  double vers = 0;

  vers = myXML.getAttributeValueDouble(cameraNode, "Version");
  if (vers==0) {
    vers = myXML.getAttributeValueDouble(cameraNode, "version");
  }
  if (vers!=CAMERA_PARAMETER_CURRENT_XML_VERSION) {
    BIASWARN("Camera-Parameter Version (" << vers << ") is Deprecated, "
             << "current version is " << CAMERA_PARAMETER_CURRENT_XML_VERSION);
    if (vers==0.8) {
      ReadCameraDataV0_8(myXML, cameraNode);
      return 0;
    } else {
      if (vers==1.0) {
        ReadCameraDataV1_0(myXML, cameraNode);
        return 0;
      } else {
        return -1;
      }
    } 
  }

  xmlNodePtr  childNode2, childNode3, childNode4;

  childNode2 = myXML.getChild(cameraNode, "Sensor");
  if (childNode2!=NULL) {
    CamModel_ = myXML.getAttributeValueString(childNode2, "Model");
    CamID_ = myXML.getAttributeValueInt(childNode2, "SerialID");
    width_ = myXML.getAttributeValueInt(childNode2, "ImageWidth");
    height_ = myXML.getAttributeValueInt(childNode2, "ImageHeight");
    if (CamModel_=="" || CamID_==0 || width_==0 || height_==0) {
      BIASERR("Sensor-data incomplete. Camera Model=" << CamModel_ 
              << ", Camera Serial=" << CamID_ 
              << ", Image Width=" << width_ 
              << ", Image Height" << height_);
      return -1;  
    }
    cellSizeX_ = myXML.getAttributeValueDouble(childNode2, "CellSizeX");
    aspectratio_ = myXML.getAttributeValueDouble(childNode2, "AspectRatio");
    if (cellSizeX_==0 || aspectratio_==0) {
      aspectratio_=1;
      cellSizeX_=1;
      cellSizeY_=1;
    } else {
     cellSizeY_ = cellSizeX_ / aspectratio_;
    }
  } else {
    BIASERR("Node \"Sensor\" missing.");
    return -1;  
  }

  childNode2 = myXML.getChild(cameraNode, "Lens");
  if (childNode2==NULL) {
    BIASERR("Node \"Lens\" missing.");
    return -1;  
  }

  LensModel_ = myXML.getAttributeValueString(childNode2, "Model");
  string LensType = myXML.getAttributeValueString(childNode2, "Type");

  if (LensType!="Spherical" && LensType!="Perspective") {
    BIASERR("Unknown Lens-Type, has to be \"Spherical\" or \"Perspective\".");
    return -1;  
  }
  if (LensType=="Spherical") {
    LensIsSpherical_ = true;
    zoomCamera_=false;
    childNode3 = myXML.getChild(childNode2, "AngleCalib");
    if (childNode3==NULL) {
      BIASERR("Node \"AngleCalib\" missing.");
      return -1;  
    }
    childNode4 = myXML.getFirstChild(childNode3);
    if (childNode4 == NULL) {
      BIASERR("No angle calibration measurement points.");
      return -1;        
    }  else {
      do {
        if (myXML.getNodeName(childNode4)=="MeasPt") {
          // TODO: Write these into SphereParameters class
          double x,y;
          x = myXML.getAttributeValueDouble(childNode4, "AngRay");
          y = myXML.getAttributeValueDouble(childNode4, "AngDeformed");
          AngleCorrX_.push_back(x); AngleCorrY_.push_back(y); 
        }
        childNode4 = myXML.getNextChild(); 
      } while (childNode4!=NULL);
      if (AngleCorrX_.size()>0) 
        MaxCamAngle_ = AngleCorrX_[AngleCorrX_.size()-1];
      else 
        MaxCamAngle_ = 0;
    }
  }
  if (LensType=="Perspective") { 
    LensIsSpherical_ = false;
    zoomCamera_=false;
    childNode3 = myXML.getChild(childNode2, "PerspectiveCalib");
    if (childNode3 != NULL) {
      childNode4 = myXML.getFirstChild(childNode3);
      if (childNode4 != NULL) {
        do {
          if (myXML.getNodeName(childNode4)=="Default") {
            focallengthDef_ =
              myXML.getAttributeValueDouble(childNode4, "FocalLength");
            //convert to Pixel
            if (cellSizeX_!=1) {
              focallengthDef_ = focallengthDef_ / cellSizeX_;
            }
            kc1Def_ = myXML.getAttributeValueDouble(childNode4, "k1");
            kc2Def_ = myXML.getAttributeValueDouble(childNode4, "k2");
            kc3Def_ = myXML.getAttributeValueDouble(childNode4, "p1");
            kc4Def_ = myXML.getAttributeValueDouble(childNode4, "p2");
          }
          if (myXML.getNodeName(childNode4)=="ZoomStep") {
            CPDiscreteParam dp;    
            dp.zoom=myXML.getAttributeValueDouble(childNode4, "Zoom");
            dp.focallength=myXML.getAttributeValueDouble(childNode4, 
                                                         "FocalLength");
             //convert to Pixel
            if (cellSizeX_!=1) {
              dp.focallength = dp.focallength / cellSizeX_;
            }
            dp.kc1=myXML.getAttributeValueDouble(childNode4, "k1");
            dp.kc2=myXML.getAttributeValueDouble(childNode4, "k2");
            dp.kc3=myXML.getAttributeValueDouble(childNode4, "p1");
            dp.kc4=myXML.getAttributeValueDouble(childNode4, "p2");
            knownparams_vect_.push_back(dp);
            zoomCamera_=true;
          }
          childNode4 = myXML.getNextChild(); 
        } while (childNode4!=NULL);
        if (knownparams_vect_.size()>0 && 
            focallengthDef_==0) {
            focallengthDef_ = knownparams_vect_[0].focallength;
            kc1Def_ = knownparams_vect_[0].kc1;
            kc2Def_ = knownparams_vect_[0].kc2;
            kc3Def_ = knownparams_vect_[0].kc3;
            kc4Def_ = knownparams_vect_[0].kc4;
        }
      }
    }
  }
  
  childNode3 = myXML.getChild(childNode2, "VignetteCalib");
  if (childNode3 != NULL) {
    childNode4 = myXML.getFirstChild(childNode3);
    if (childNode4 != NULL) {
      do {
        if (myXML.getNodeName(childNode4)=="MeasPt") {
          // TODO: Write these into SphereParameters & cameradata class
          double x,y;
          if (LensIsSpherical_) {
            x = myXML.getAttributeValueDouble(childNode4, "AngRay");
          } else {
            x = myXML.getAttributeValueDouble(childNode4, "Dist");
          }
          y = myXML.getAttributeValueDouble(childNode4, "Fac");
          IlluCorrX_.push_back(x); IlluCorrY_.push_back(y); 
        }
        childNode4 = myXML.getNextChild(); 
      } while (childNode4!=NULL);
    }
  }
      
  childNode2 = myXML.getChild(cameraNode, "PrincipalPoint");
  if (childNode2 != NULL) {
    principalX_ = myXML.getAttributeValueDouble(childNode2, "X");
    principalX_ = width_ * principalX_;
    principalY_ = myXML.getAttributeValueDouble(childNode2, "Y");
    principalY_ = height_ * principalY_;
    if (LensIsSpherical_) {
      radius_ = myXML.getAttributeValueDouble(childNode2, "SphereRadius");
      radius_ = width_ * radius_;
    }
  } else {
    principalX_ = width_/2.0;
    principalY_ = height_/2.0;
    if (LensIsSpherical_) {
      radius_ = height_/2.0;
    }   
  }

  IsInitialized_=true;
  return 0;

}


int CameraParam::ReadCameraDataV1_0(XMLIO &myXML, xmlNodePtr &cameraNode) {
  
  //reset this
  Init();
  CamModel_ = myXML.getAttributeValueString(cameraNode, "name");
  
  xmlNodePtr  childNode2, childNode3, childNode4, childNode5, childNode6;

  childNode2 = myXML.getChild(cameraNode, "id");
  CamID_ = myXML.getNodeContentInt(childNode2);
  if ( CamID_==0 ) {
       BIASERR("Sensor-data incomplete. Camera ID must not be 0:" << CamID_);
      return -1;  
    }
   
  childNode2 = myXML.getChild(cameraNode, "sensor");
  if (childNode2!=NULL) {
    childNode3 = myXML.getChild(childNode2, "numberOfPixels");
    childNode4 = myXML.getChild(childNode3, "x");
    width_ = myXML.getNodeContentInt(childNode4);
    childNode4 = myXML.getChild(childNode3, "y");
    height_ = myXML.getNodeContentInt(childNode4);
    if ( width_==0 || height_==0) {
      BIASERR("Sensor-data incomplete. Camera ID=" << CamID_ 
              << ", Image Width= " << width_ 
              << ", Image Height= " << height_);
      return -1;  
    }

    childNode3 = myXML.getChild(childNode2, "pixelSpacing");
    childNode4 = myXML.getChild(childNode3, "x");
    cellSizeX_ = myXML.getNodeContentDouble(childNode4);
    childNode4 = myXML.getChild(childNode3, "y");
    cellSizeY_ = myXML.getNodeContentDouble(childNode4);
    if (cellSizeX_==0 || cellSizeY_==0) {
      aspectratio_=1;
      cellSizeX_=1;
      cellSizeY_=1;
    } else {
      aspectratio_ = cellSizeX_/cellSizeY_;
      cout <<"========== aspectratio: "<<aspectratio_<<endl;
    }

    childNode3 = myXML.getChild(childNode2, "principalPoint");
    if (childNode3 != NULL) {
      childNode4 = myXML.getChild(childNode3, "x");
      principalX_ = myXML.getNodeContentDouble(childNode4);
      childNode4 = myXML.getChild(childNode3, "y");
      principalY_ = myXML.getNodeContentDouble(childNode4);
      //convert to Pixel
      principalX_ = width_/2.0  + principalX_/cellSizeX_;
      principalY_ = height_/2.0 + principalY_/cellSizeY_; 
      //      cout<<"Principal point is: "<<principalX_<<","<<principalY_<<"in ReadDatav1.0"<<endl;
    } else {
      principalX_ = width_/2.0;
      principalY_ = height_/2.0;
      //  cout<<"Principal point is: "<<principalX_<<","<<principalY_<<"in ReadDatav1.0"<<endl;
    }
    
  } else {
    BIASERR("Node \"sensor\" missing.");
    return -1;  
  }

  childNode2 = myXML.getChild(cameraNode, "lens");
  if (childNode2!=NULL) {
    LensIsSpherical_=false;
    zoomCamera_=false;
  } else {
    childNode2 = myXML.getChild(cameraNode, "zoomLens");
    if (childNode2!=NULL) {
      LensIsSpherical_=false;
      zoomCamera_=true;
    } else {
      childNode2 = myXML.getChild(cameraNode, "sphericalLens");
      if (childNode2!=NULL) {
        LensIsSpherical_=true;
        zoomCamera_=false;
      } else {
        BIASERR("Camera has no lens-, zoomLens- or sphericalLens-Tag");
        return -1;          
      }
    }
  }

  LensModel_ = myXML.getAttributeValueString(childNode2, "name");

  if (LensIsSpherical_) {
 
    zoomCamera_=false;
    childNode3 = myXML.getChild(childNode2, "angleCalibration");
    if (childNode3==NULL) {
      BIASERR("Node \"angleCalibration\" missing.");
      return -1;  
    }
    childNode4 = myXML.getFirstChild(childNode3);
    if (childNode4 == NULL) {
      BIASERR("No angle calibration measurement points.");
      return -1;        
    }  else {
      do {
        if (myXML.getNodeName(childNode4)=="measPoint") {
          double x,y;
          childNode5 = myXML.getChild(childNode4, "angleRay");
          x = myXML.getNodeContentDouble(childNode5);
          childNode5 = myXML.getChild(childNode4, "angleDeformed");
          y = myXML.getNodeContentDouble(childNode5);
          AngleCorrX_.push_back(x); AngleCorrY_.push_back(y); 
        }
        childNode4 = myXML.getNextChild(childNode4); 
      } while (childNode4!=NULL);
      if (AngleCorrX_.size()>0) 
        MaxCamAngle_ = AngleCorrX_[AngleCorrX_.size()-1];
      else 
        MaxCamAngle_ = 0;
    }

    childNode3 = myXML.getChild(childNode2, "vignetteCalibration");
    if (childNode3 != NULL) {
      childNode4 = myXML.getFirstChild(childNode3);
      if (childNode4 != NULL) {
        do {
          if (myXML.getNodeName(childNode4)=="measPoint") {
            double x,y;
            childNode5 = myXML.getChild(childNode4, "angleRay");
            x = myXML.getNodeContentDouble(childNode5);
            childNode5 = myXML.getChild(childNode4, "factor");
            y = myXML.getNodeContentDouble(childNode5);
            IlluCorrX_.push_back(x); IlluCorrY_.push_back(y); 
          }
          childNode4 = myXML.getNextChild(childNode4); 
        } while (childNode4!=NULL);
      }
    }

    childNode3 = myXML.getChild(childNode2, "sphereRadius");
    radius_ = myXML.getNodeContentDouble(childNode3);
    if (childNode3==NULL) {
      BIASERR("Sphere radius missing!");
      return -1;  
    }
    
  } else {  //  if (!LensIsSpherical_)
    if (!zoomCamera_) {
      childNode3 = myXML.getChild(childNode2, "focalLength");
      focallengthDef_ = myXML.getNodeContentDouble(childNode3);
      //convert to Pixel
      focallengthDef_ = focallengthDef_/cellSizeX_;
      childNode3 = myXML.getChild(childNode2, "distortion");
      childNode4 = myXML.getChild(childNode3, "radial");
      childNode5 = myXML.getChild(childNode4, "k1");
      kc1Def_ = myXML.getNodeContentDouble(childNode5);
      childNode5 = myXML.getChild(childNode4, "k2");
      kc2Def_ = myXML.getNodeContentDouble(childNode5);
      childNode4 = myXML.getChild(childNode3, "tangential");
      childNode5 = myXML.getChild(childNode4, "p1");
      kc3Def_ = myXML.getNodeContentDouble(childNode5);
      childNode5 = myXML.getChild(childNode4, "p2");
      kc4Def_ = myXML.getNodeContentDouble(childNode5);
    } else { // if (zoomCamera)
      childNode3 = myXML.getFirstChild(childNode2);
      if (childNode3 != NULL) {
        do {
          if (myXML.getNodeName(childNode3)=="zoomStep") {
            CPDiscreteParam dp;    
            childNode4 = myXML.getChild(childNode3, "zoom");
            dp.zoom = myXML.getNodeContentDouble(childNode4);
            childNode4=myXML.getChild(childNode3, "focalLength");
            dp.focallength = myXML.getNodeContentDouble(childNode4);
             //convert to Pixel
            dp.focallength = dp.focallength/cellSizeX_;
            childNode4=myXML.getChild(childNode3, "distortion");
            childNode5=myXML.getChild(childNode4, "radial");
            childNode6=myXML.getChild(childNode5, "k1");
            dp.kc1 = myXML.getNodeContentDouble(childNode6);
            childNode6=myXML.getChild(childNode5, "k2");
            dp.kc2 = myXML.getNodeContentDouble(childNode6);
            childNode5=myXML.getChild(childNode4, "tangential");
            childNode6=myXML.getChild(childNode5, "p1");
            dp.kc3 = myXML.getNodeContentDouble(childNode6);
            childNode6=myXML.getChild(childNode5, "p2");
            dp.kc4 = myXML.getNodeContentDouble(childNode6);
            knownparams_vect_.push_back(dp);
          }
          if (myXML.getNodeName(childNode3)=="defaultZoom") {
            BIASERR("defaultZoom is deprecated and has no effect, use different param-file version");
          }
          childNode3 = myXML.getNextChild(childNode3); 
        } while (childNode3!=NULL);
        if (knownparams_vect_.size()>0) {
          focallengthDef_ = knownparams_vect_[0].focallength;
          kc1Def_ = knownparams_vect_[0].kc1;
          kc2Def_ = knownparams_vect_[0].kc2;
          kc3Def_ = knownparams_vect_[0].kc3;
          kc4Def_ = knownparams_vect_[0].kc4;
        }
      }
    }
  }
  IsInitialized_=true;
  return 0;
}
#endif //BIAS_HAVE_XML2


#ifdef BIAS_HAVE_XML2
int CameraParam::ReadCameraDataV0_8(XMLIO &myXML, xmlNodePtr &cameraNode) {
  
  //reset this
  Init();
 
  xmlNodePtr  childNode2, childNode3, childNode4;

  childNode2 = myXML.getChild(cameraNode, "Sensor");
  if (childNode2!=NULL) {
    CamModel_ = myXML.getAttributeValueString(childNode2, "Model");
    CamID_ = myXML.getAttributeValueInt(childNode2, "SerialID");
    width_ = myXML.getAttributeValueInt(childNode2, "ImageWidth");
    height_ = myXML.getAttributeValueInt(childNode2, "ImageHeight");
    if (CamModel_=="" || CamID_==0 || width_==0 || height_==0) {
      BIASERR("Sensor-data incomplete. Camera Model=" << CamModel_ 
              << ", Camera Serial=" << CamID_ 
              << ", Image Width=" << width_ 
              << ", Image Height" << height_);
      return -1;  
    }
    cellSizeX_ = myXML.getAttributeValueDouble(childNode2, "CellSizeInMicrX");
    cellSizeY_ = myXML.getAttributeValueDouble(childNode2, "CellSizeInMicrY");
    if (cellSizeX_>100 ||cellSizeY_>100) {
      cellSizeX_ /= 1000.0;
      cellSizeY_ /= 1000.0;
    } else {
      cellSizeX_ /= 1000000.0;
      cellSizeY_ /= 1000000.0;
    }
    if (cellSizeX_==0 || cellSizeY_==0) {
      aspectratio_=1;
      cellSizeX_=1;
      cellSizeY_=1;
    } else {
      aspectratio_ = cellSizeX_/cellSizeY_;
    }
  } else {
    BIASERR("Node \"Sensor\" missing.");
    return -1;  
  }

  childNode2 = myXML.getChild(cameraNode, "Lens");
  if (childNode2==NULL) {
    BIASERR("Node \"Lens\" missing.");
    return -1;  
  }

  LensModel_ = myXML.getAttributeValueString(childNode2, "Model");
  string LensType = myXML.getAttributeValueString(childNode2, "Type");

  if (LensType!="Spherical" && LensType!="Perspective") {
    BIASERR("Unknown Lens-Type, has to be \"Spherical\" or \"Perspective\".");
    return -1;  
  }
  if (LensType=="Spherical") {
    LensIsSpherical_ = true;
    zoomCamera_=false;
    childNode3 = myXML.getChild(childNode2, "AngleCalib");
    if (childNode3==NULL) {
      BIASERR("Node \"AngleCalib\" missing.");
      return -1;  
    }
    childNode4 = myXML.getFirstChild(childNode3);
    if (childNode4 == NULL) {
      BIASERR("No angle calibration measurement points.");
      return -1;        
    }  else {
      do {
        if (myXML.getNodeName(childNode4)=="MeasPt") {
          // TODO: Write these into SphereParameters class
          double x,y;
          x = myXML.getAttributeValueDouble(childNode4, "AngRay");
          y = myXML.getAttributeValueDouble(childNode4, "AngDeformed");
          AngleCorrX_.push_back(x); AngleCorrY_.push_back(y); 
        }
        childNode4 = myXML.getNextChild(); 
      } while (childNode4!=NULL);
      if (AngleCorrX_.size()>0) 
        MaxCamAngle_ = AngleCorrX_[AngleCorrX_.size()-1];
      else 
        MaxCamAngle_ = 0;
    }
  }
  if (LensType=="Perspective") { 
    LensIsSpherical_ = false;
    zoomCamera_=false;
    childNode3 = myXML.getChild(childNode2, "PerspectiveCalib");
    if (childNode3 != NULL) {
      childNode4 = myXML.getFirstChild(childNode3);
      if (childNode4 != NULL) {
        do {
          if (myXML.getNodeName(childNode4)=="Default") {
            focallengthDef_ =
              myXML.getAttributeValueDouble(childNode4, "FocalLengthInMM");
            //convert to Pixel
            if (cellSizeX_!=1) {
              focallengthDef_ = focallengthDef_/1000.0 / cellSizeX_;
            }
            kc1Def_ = myXML.getAttributeValueDouble(childNode4, "KC1");
            kc2Def_ = myXML.getAttributeValueDouble(childNode4, "KC2");
            kc3Def_ = myXML.getAttributeValueDouble(childNode4, "KC3");
            kc4Def_ = myXML.getAttributeValueDouble(childNode4, "KC4");
          }
          if (myXML.getNodeName(childNode4)=="ZoomStep") {
            CPDiscreteParam dp;    
            dp.zoom=myXML.getAttributeValueDouble(childNode4, "Zoom");
            dp.focallength=myXML.getAttributeValueDouble(childNode4, 
                                                         "FocalLengthInMM");
             //convert to Pixel
            if (cellSizeX_!=1) {
              dp.focallength = dp.focallength/1000.0 / cellSizeX_;
            }
            dp.kc1=myXML.getAttributeValueDouble(childNode4, "KC1");
            dp.kc2=myXML.getAttributeValueDouble(childNode4, "KC2");
            dp.kc3=myXML.getAttributeValueDouble(childNode4, "KC3");
            dp.kc4=myXML.getAttributeValueDouble(childNode4, "KC4");
            knownparams_vect_.push_back(dp);
            zoomCamera_=true;
          }
          childNode4 = myXML.getNextChild(); 
        } while (childNode4!=NULL);
        if (knownparams_vect_.size()>0 && 
            focallengthDef_==0) {
            focallengthDef_ = knownparams_vect_[0].focallength;
            kc1Def_ = knownparams_vect_[0].kc1;
            kc2Def_ = knownparams_vect_[0].kc2;
            kc3Def_ = knownparams_vect_[0].kc3;
            kc4Def_ = knownparams_vect_[0].kc4;
        }
      }
    }
  }
  
  childNode3 = myXML.getChild(childNode2, "VignetteCalib");
  if (childNode3 != NULL) {
    childNode4 = myXML.getFirstChild(childNode3);
    if (childNode4 != NULL) {
      do {
        if (myXML.getNodeName(childNode4)=="MeasPt") {
          // TODO: Write these into SphereParameters & cameradata class
          double x,y;
          if (LensIsSpherical_) {
            x = myXML.getAttributeValueDouble(childNode4, "AngRay");
          } else {
            x = myXML.getAttributeValueDouble(childNode4, "Dist");
          }
          y = myXML.getAttributeValueDouble(childNode4, "Fac");
          IlluCorrX_.push_back(x); IlluCorrY_.push_back(y); 
        }
        childNode4 = myXML.getNextChild(); 
      } while (childNode4!=NULL);
    }
  }
      
  childNode2 = myXML.getChild(cameraNode, "PrincipalPoint");
  if (childNode2 != NULL) {
    principalX_ = myXML.getAttributeValueDouble(childNode2, "X");
    principalY_ = myXML.getAttributeValueDouble(childNode2, "Y");
    if (LensIsSpherical_) {
      radius_ = myXML.getAttributeValueDouble(childNode2, "SphereRadius");
    }
  } else {
    principalX_ = width_/2.0;
    principalY_ = height_/2.0;
    if (LensIsSpherical_) {
      radius_ = height_/2.0;
    }   
  }

  IsInitialized_=true;
  return 0;
}
#endif //BIAS_HAVE_XML2

void CameraParam::InitSpherical(const string CamModel, const int CamID, 
                                const string LensModel,
                                const int imgWidth, const int imgHeight, 
                                const double principalX, 
                                const double principalY, 
                                const double radius,
                                const vector<double> &AngleCorrX,
                                const vector<double> &AngleCorrY,
                                const vector<double> &IlluCorrX, 
                                const vector<double> &IlluCorrY) {

  //reset this
  Init();
  downsamplingFactor_=1;

  CamModel_ = CamModel;
  CamID_ = CamID;
  LensModel_=LensModel;

  width_  = imgWidth;
  height_ = imgHeight;
  principalX_ = principalX;
  principalY_ = principalY;
  radius_ = radius;

  aspectratio_=1;
  cellSizeX_=1;
  cellSizeY_=1;
  LensIsSpherical_ = true;
  zoomCamera_=false;

  AngleCorrX_ = AngleCorrX;
  AngleCorrY_ = AngleCorrY;
  IlluCorrX_  = IlluCorrX;
  IlluCorrY_  = IlluCorrY;

  IsInitialized_=true;

}


void CameraParam::InitPerspective(const string CamModel, const int CamID, 
                                const string LensModel,
                                const int imgWidth, const int imgHeight, 
                                const double principalX, 
                                const double principalY, 
                                const double focallength, 
                                const double aspectratio,
                                const double k1, const double k2, const double k3, const double k4,
                                const vector<double> &IlluCorrX, 
                                const vector<double> &IlluCorrY) {

  //reset this
  Init();
  downsamplingFactor_=1;
  CamModel_ = CamModel;
  CamID_ = CamID;
  LensModel_=LensModel;
  width_  = imgWidth;
  height_ = imgHeight;
  principalX_ = principalX;
  principalY_ = principalY;
  radius_ = 0.0;
  focallengthDef_ = focallength;
  aspectratio_=aspectratio;
  cellSizeX_=1;
  cellSizeY_=1/aspectratio;
  LensIsSpherical_ = false;
  zoomCamera_=false;
  kc1Def_ = k1; kc2Def_ = k2; kc3Def_ = k3; kc4Def_ = k4;
  IlluCorrX_  = IlluCorrX;
  IlluCorrY_  = IlluCorrY;
  IsInitialized_=true;
}

void CameraParam::DebugOut() const {
  cout << "Camera Parameters:" << endl;
  cout << "IsInitialized_ = " << IsInitialized_ << endl;
  cout << "CamModel_ = " << CamModel_ << endl;
  cout << "CamID_ = " << CamID_ << endl;
  cout << "LensModel_ = " << LensModel_ << endl;
  cout << "LensIsSpherical_ = " << LensIsSpherical_ << endl;
  cout << "zoomCamera_ = " << zoomCamera_ << endl;
  cout << "width_ = " << width_ << endl;
  cout << "height_ = " << height_ << endl;
  for (int i=0; i<(int)knownparams_vect_.size();i++) {
    cout << "knownparams_vect_[" << i << "] = " << endl;
    CPDiscreteParam dp;
    dp=knownparams_vect_[i];
    cout << "  zoom = " << dp.zoom << endl;
    cout << "  focallength = " << dp.focallength << endl;
    cout << "  kc1 = " << dp.kc1 << endl;
    cout << "  kc2 = " << dp.kc2 << endl;
    cout << "  kc3 = " << dp.kc3 << endl;
    cout << "  kc4 = " << dp.kc4 << endl;
  };
  for (int i=0; i<(int) AngleCorrX_.size();i++) {
    cout << "AngleCorrX_[" << i << "] = " << AngleCorrX_[i] << endl;
  }
  for (int i=0; i<(int) AngleCorrY_.size();i++) {
    cout << "AngleCorrY_[" << i << "] = " << AngleCorrY_[i] << endl;
  }
  for (int i=0; i<(int) IlluCorrX_.size();i++) {
    cout << "IlluCorrX_[" << i << "] = " << IlluCorrX_[i] << endl;
  }
  for (int i=0; i<(int) IlluCorrY_.size();i++) {
    cout << "IlluCorrY_[" << i << "] = " << IlluCorrY_[i] << endl;
  }

  cout << "aspectratio_ = " << aspectratio_ << endl;
  cout << "cellSizeX_ = " << cellSizeX_ << endl;
  cout << "cellSizeY_ = " << cellSizeY_ << endl;
  cout << "principalX_ = " << principalX_ << endl;
  cout << "principalY_ = " << principalY_ << endl;
  cout << "focallengthDef_ = " << focallengthDef_ << endl;
  cout << "kc1Def_ = " << kc1Def_ << endl;
  cout << "kc2Def_ = " << kc2Def_ << endl;
  cout << "kc3Def_ = " << kc3Def_ << endl;
  cout << "kc4Def_ = " << kc4Def_ << endl;
  cout << "downsamplingFactor_ = " << downsamplingFactor_ << endl;
  cout << endl << endl;
}


void CameraParam::SetData_EVID31_147094(){

  //reset this
  Init();

  CPDiscreteParam dp;    
  CamModel_ = "SONY EVI-D31";
  CamID_ = 147094;
  LensModel_ = "LENS_SONY_EVID31_BUILTIN";
  LensIsSpherical_ = false;

  cellSizeX_=1;
  cellSizeY_=1;
  
  width_=768;
  height_=576;
  
  aspectratio_=1;
  
  principalX_=384;
  principalY_=288;
  
//   exposureTime_ = 0.08;
//   iris_ = 1;
//   reciprocalFocus_ = 1.234;
  
  knownparams_vect_.clear();
  
  // must be in descending order regarding the zoom!
  
  dp.zoom=47;
  dp.focallength=910.781402852481733;
  dp.kc1=-0.268280770809586;
  dp.kc2=0.487451704517498;
  dp.kc3=-0.000762042258562;
  dp.kc4=-0.001130194838780;
  knownparams_vect_.push_back(dp);
  
  dp.zoom=45;
  dp.focallength=942.276371802197332;
  dp.kc1=-0.222325918868402;
  dp.kc2=0.303803084817878;
  dp.kc3=-0.000220828107747;
  dp.kc4=-0.002379174938453;
  knownparams_vect_.push_back(dp);
  
  dp.zoom=43;
  dp.focallength=989.223042668483913;
  dp.kc1=-0.230499856719837;
  dp.kc2=0.632569092436950;
  dp.kc3=0.000645301367934;
  dp.kc4=-0.002245120579513;
  knownparams_vect_.push_back(dp);
  
  dp.zoom=41;
  dp.focallength=1046.743156411611380;
  dp.kc1=-0.175147739150835;
  dp.kc2=0.399855896767651;
  dp.kc3=0.001819727735452;
  dp.kc4=-0.004642715440390;
  knownparams_vect_.push_back(dp);
  
  dp.zoom=39;
  dp.focallength=1113.877613258150632;
  dp.kc1=-0.199908717507440;
  dp.kc2=0.735328948045900;
  dp.kc3=0.001857139856879;
  dp.kc4=-0.001726332328807;
  knownparams_vect_.push_back(dp);
  
  dp.zoom=37;
  dp.focallength=1195.129672009601336;
  dp.kc1=-0.194029932886528;
  dp.kc2=1.052137931539009;
  dp.kc3=0.002556010034664;
  dp.kc4=-0.002664280436511;
  knownparams_vect_.push_back(dp);
  
  dp.zoom=35;
  dp.focallength=1284.915674699713918;
  dp.kc1=-0.222381373429422;
  dp.kc2=1.627430874083972;
  dp.kc3=0.001072744905158;
  dp.kc4=-0.001628105221342;
  knownparams_vect_.push_back(dp);
  
  dp.zoom=33;
  dp.focallength=1381.682287410254276;
  dp.kc1=-0.221923278642680;
  dp.kc2=1.734685282276320;
  dp.kc3=0.002486050695331;
  dp.kc4=-0.001291711925732;
  knownparams_vect_.push_back(dp);
  
  dp.zoom=31;
  dp.focallength=1493.437763776483052;
  dp.kc1=-0.224447467694984;
  dp.kc2=1.863840161034580;
  dp.kc3=0.003709677616807;
  dp.kc4=-0.001150445035362;
  knownparams_vect_.push_back(dp);
  
  dp.zoom=29;
  dp.focallength=1616.993704040472039;
  dp.kc1=-0.202923793671622;
  dp.kc2=1.473981778670278;
  dp.kc3=0.003684677768293;
  dp.kc4=-0.002550288087126;
  knownparams_vect_.push_back(dp);
  
  dp.zoom=27;
  dp.focallength=1757.081611652665060;
  dp.kc1=-0.217240281709651;
  dp.kc2=2.140024501420107;
  dp.kc3=0.001114854630183;
  dp.kc4=-0.001295578479362;
  knownparams_vect_.push_back(dp);
  
  dp.zoom=25;
  dp.focallength=1902.05452;
  dp.kc1=-0.23863;
  dp.kc2=1.61900;
  dp.kc3=0.00195;
  dp.kc4=-0.00185;
  knownparams_vect_.push_back(dp);
  
  dp.zoom=23;
  dp.focallength=2074.378333106370974;
  dp.kc1=-0.255567875389573;
  dp.kc2=1.732924024277849;
  dp.kc3=0.001245946585972;
  dp.kc4=-0.000594821663798;
  knownparams_vect_.push_back(dp);
  
  dp.zoom=21;
  dp.focallength=2282.63949;
  dp.kc1=-0.26368;
  dp.kc2=0.44519;
  dp.kc3=0.00174;
  dp.kc4=-0.00076;
  knownparams_vect_.push_back(dp);
  
  dp.zoom=19;
  dp.focallength=2519.628084135987592;
  dp.kc1=-0.332071279316112;
  dp.kc2=0.944781789471998;
  dp.kc3=0.000746973436601;
  dp.kc4=-0.001772703465946;
  knownparams_vect_.push_back(dp);
  
  dp.zoom=17;
  dp.focallength=2824.507670379169667;
  dp.kc1=-0.404343435901478;
  dp.kc2=0.453162060457514;
  dp.kc3=-0.000861720421368;
  dp.kc4=-0.001149130726447;
  knownparams_vect_.push_back(dp);
  
  dp.zoom=15;
  dp.focallength=3226.589266051306367;
  dp.kc1=-0.594320179301222;
  dp.kc2=12.959736505367889;
  dp.kc3=-0.001868658432064;
  dp.kc4=-0.000676806079268;
  knownparams_vect_.push_back(dp);
  
  dp.zoom=13;
  dp.focallength=3788.529115119004928;
  dp.kc1=-0.823072669196453;
  dp.kc2=27.510513372014003;
  dp.kc3=-0.006928032867432;
  dp.kc4=0.002134868203333;
  knownparams_vect_.push_back(dp);
  
  dp.zoom=11;
  dp.focallength=4592.039263971351829;
  dp.kc1=-1.121702159939672;
  dp.kc2=36.994236469912934;
  dp.kc3=-0.014620822228192;
  dp.kc4=0.004860766468908;
  knownparams_vect_.push_back(dp);
  
  dp.zoom=9;
  dp.focallength=5834.063334730769384;
  dp.kc1=-1.473018481165970;
  dp.kc2=41.598117903225528;
  dp.kc3=-0.028692464501407;
  dp.kc4=0.007739406251353;
  knownparams_vect_.push_back(dp);

  dp.zoom=7;
  dp.focallength=5877.390940425404551;
  dp.kc1=-1.786184809649660;
  dp.kc2=109.639598242766894;
  dp.kc3=-0.011555140535335;
  dp.kc4=-0.002211119391028;
  knownparams_vect_.push_back(dp);
  
  // this is a safety-buffer
  dp.zoom=1000;
  knownparams_vect_.push_back(dp);
  IsInitialized_=true;
  
}



int CameraParam::ReadFromEXIF(const std::string& filename) {
  //reset this
  Init();

  downsamplingFactor_=1;
  ImageAttributes exifdata;
  exifdata.InitExifTags();
  int result = exifdata.Read(filename);
  if (result!=0) return result;

 
  string make, model;
  exifdata.GetHardwareName(make, model);
  CamModel_ = make + "___"+model;

  CamID_ = 0;
  LensModel_ = "unknown";
  LensIsSpherical_ = false;

  result = exifdata.GetPixelSizeXMeter(cellSizeX_);
  if (result<0) return result;
  result = exifdata.GetPixelSizeYMeter(cellSizeY_);
  if (result<0) return result;

  int width=0, height=0;
  result = exifdata.GetImageDimensions(width, height);
  if (result<0) return result;
  width_ = width;
  height_ = height;
  
  aspectratio_ = cellSizeX_ / cellSizeY_;

  result = exifdata.GetPrincipalPoint(principalX_, principalY_);
  if (result<0) return result;

  result = exifdata.GetFocalLengthXPixel(focallengthDef_);
  if (result<0) return result;
  
  zoomCamera_ = false;
  knownparams_vect_.clear();
  IsInitialized_=true;

  return 0;
}

CameraParam::CameraParam(const KMatrix& K, const int w, const int h) {
  //reset this
  Init();
 
  width_    = w;
  height_   = h;
  CamModel_ = "fromkmatrix";

  // you can set any value here but not zero !
  CamID_ = 42;
  LensModel_ = "unknown";
  LensIsSpherical_ = false;

  cellSizeX_  = 1;
  cellSizeY_  = K[0][0] / K[1][1];
  principalX_ = K[0][2];
  principalY_ = K[1][2];

  focallengthDef_ = K[0][0];
   
  aspectratio_ = cellSizeX_/cellSizeY_;
 
  zoomCamera_ = false;
  knownparams_vect_.clear();
  IsInitialized_=true;

}


void CameraParam::SetData_OMNITECH_ORIFL_190_3 () { 

  //reset this
  Init();

  AngleCorrX_.clear();
  AngleCorrY_.clear();
  IlluCorrX_.clear();
  IlluCorrY_.clear(); 

  CamModel_ = "Fisheye Camera";
  CamID_ = 0440;
  LensModel_ = "OMNITECH_ORIFL_190_3";
  LensIsSpherical_ = true;

  width_=640;
  height_=480;
  
  principalX_ = 364;
  principalY_ = 245;

  aspectratio_=1;
  
  knownparams_vect_.clear();

  cellSizeX_=1;
  cellSizeY_=1;

  double x,y;
  x=0.000; y=1.000*x;
  AngleCorrX_.push_back(x); AngleCorrY_.push_back(y); 
  x=0.138; y=1.000*x;
  AngleCorrX_.push_back(x); AngleCorrY_.push_back(y); 
  x=0.424; y=0.993*x;
  AngleCorrX_.push_back(x); AngleCorrY_.push_back(y); 
  x=0.757; y=0.974*x;
  AngleCorrX_.push_back(x); AngleCorrY_.push_back(y); 
  x=1.045; y=0.941*x;
  AngleCorrX_.push_back(x); AngleCorrY_.push_back(y); 
  x=1.342; y=0.899*x;
  AngleCorrX_.push_back(x); AngleCorrY_.push_back(y); 
  x=1.436; y=0.881*x;
  AngleCorrX_.push_back(x); AngleCorrY_.push_back(y); 
  x=1.652; y=0.828*x;
  AngleCorrX_.push_back(x); AngleCorrY_.push_back(y);
  
  // Vignette correction x=angle, y=darkening factor
  x=0.005; y=1.000;
  IlluCorrX_.push_back(x); IlluCorrY_.push_back(y); 
  x=0.092; y=1.000;
  IlluCorrX_.push_back(x); IlluCorrY_.push_back(y); 
  x=0.179; y=1.000;
  IlluCorrX_.push_back(x); IlluCorrY_.push_back(y); 
  x=0.266; y=1.003;
  IlluCorrX_.push_back(x); IlluCorrY_.push_back(y); 
  x=0.354; y=1.003;
  IlluCorrX_.push_back(x); IlluCorrY_.push_back(y); 
  x=0.439; y=1.003;
  IlluCorrX_.push_back(x); IlluCorrY_.push_back(y); 
  x=0.527; y=1.003;
  IlluCorrX_.push_back(x); IlluCorrY_.push_back(y); 
  x=0.615; y=1.005;
  IlluCorrX_.push_back(x); IlluCorrY_.push_back(y); 
  x=0.701; y=1.005;
  IlluCorrX_.push_back(x); IlluCorrY_.push_back(y); 
  x=0.788; y=1.005;
  IlluCorrX_.push_back(x); IlluCorrY_.push_back(y); 
  x=0.875; y=1.005;
  IlluCorrX_.push_back(x); IlluCorrY_.push_back(y); 
  x=0.962; y=1.003;
  IlluCorrX_.push_back(x); IlluCorrY_.push_back(y); 
  x=1.049; y=0.998;
  IlluCorrX_.push_back(x); IlluCorrY_.push_back(y); 
  x=1.135; y=0.993;
  IlluCorrX_.push_back(x); IlluCorrY_.push_back(y); 
  x=1.223; y=0.983;
  IlluCorrX_.push_back(x); IlluCorrY_.push_back(y); 
  x=1.310; y=0.966;
  IlluCorrX_.push_back(x); IlluCorrY_.push_back(y); 
  x=1.398; y=0.941;
  IlluCorrX_.push_back(x); IlluCorrY_.push_back(y); 
  x=1.485; y=0.897;
  IlluCorrX_.push_back(x); IlluCorrY_.push_back(y); 
  x=1.572; y=0.828;
  IlluCorrX_.push_back(x); IlluCorrY_.push_back(y); 
  x=1.660; y=0.716;
  IlluCorrX_.push_back(x); IlluCorrY_.push_back(y); 
  
  // TODO: Implement Max Angle
  //double MaxCamAngle = 1.6581; // 95 deg
    
  radius_ = 226;
  IsInitialized_=true;
}


double CameraParam::InterpolateFocalLength_(){
  
  double relnorm;
  double absdist;
  
  bool found;
  unsigned int i;
  found=false;

  if (knownparams_vect_[0].zoom<zoom_){ // zoom is higher than the 
    // known values:
    zoom_=knownparams_vect_[0].zoom;  // set to highest available
  }
  
  for (i=0;i<knownparams_vect_.size();i++){
    if (knownparams_vect_[i].zoom<=zoom_){
	    found=true;
	    break;
    }
  }
  
  if (!found){ // zoom is lower than the known values
    focallengthZoom_=knownparams_vect_[i-1].focallength;
    return knownparams_vect_[i-1].focallength;
  }
  
  
  if (knownparams_vect_[i].zoom==zoom_){
    return knownparams_vect_[i].focallength;
  }

  relnorm=(zoom_ - knownparams_vect_[i].zoom) /
    (knownparams_vect_[i-1].zoom - knownparams_vect_[i].zoom);
  
  relnorm=1.0-relnorm;
  
  absdist=relnorm*(knownparams_vect_[i].focallength-
                   knownparams_vect_[i-1].focallength);
  
  return (absdist+knownparams_vect_[i-1].focallength);
}


double CameraParam::InterpolateAbsZoom_(){
  
  double relnorm;
  double absdist;
  
  bool found;
  unsigned int i;
  found=false;
  
  if (knownparams_vect_[0].focallength<focallengthZoom_){
    focallengthZoom_=knownparams_vect_[0].focallength;
  }
  
  for (i=0;i<knownparams_vect_.size();i++){
    if (knownparams_vect_[i].focallength<=focallengthZoom_){
	    found=true;
	    break;
    }
  }
  if (!found){
    return -1;
  }
  
  if (knownparams_vect_[i].focallength==focallengthZoom_){
    // interpolate distortion parameters
    return knownparams_vect_[i].zoom;
  }
  
  relnorm=(focallengthZoom_ - knownparams_vect_[i].focallength) /
    (knownparams_vect_[i-1].focallength - knownparams_vect_[i].focallength);
  
  relnorm=1.0-relnorm;
  
  absdist=relnorm*(knownparams_vect_[i].zoom-
                   knownparams_vect_[i-1].zoom);
  
  return (absdist+knownparams_vect_[i-1].zoom);
}
