#ifndef _ColorSegmentation_hh_
#define _ColorSegmentation_hh_ 

#include <bias_config.h>
#ifndef BIAS_HAVE_PTHREADS
#  error You need pthreads. Please recompile BIAS wih USE_PTHREADS
#endif // BIAS_HAVE_PTHREADS

#include <Base/Image/Image.hh>



namespace BIAS {

  /** @class ColorSegmentation
  @brief
  @author africk */
  class ColorSegmentation 
    : public BIAS::Debug 
  {
  public:
    ColorSegmentation();
    void SetColorSpace(BIAS::ImageBase::EColorModel farbraum);
    void Segment(BIAS::Image<unsigned char> &src, 
      BIAS::Image<unsigned char> &res, 
      int hue,int hueIntervall,int minSaturation);
    void GetHueImage(BIAS::Image<unsigned char> &imgHue);
    void GetSatImage(BIAS::Image<unsigned char> &imgSat);
    void GetGreyImage(BIAS::Image<unsigned char> &imgGrey);

    inline void Segment(const unsigned char **src, unsigned char **dst,
                        const int tl[2], const int br[2], 
                        const int hue, const int hueIntervall,
                        const int minSaturation)
    {
      register int x, y;
      int c, s, d;
      for (y = tl[1]; y < br[1]; y++) {
        for (x = tl[0]; x < br[0]; x++) {
          c = (int)src[y][3*x];
          s = (int)src[y][3*x+1];
          d = abs(hue-c);
          //if (d>=128) d=256-d;
          if (d<=hueIntervall && s>=minSaturation) 
            dst[y][x] = 255;
          else 
            dst[y][x] = 0; 
        }
      }
    };

  protected:
    BIAS::Image<unsigned char> image;
    BIAS::ImageBase::EColorModel stdFarbRaum;

  }; 

  class ColorSegmentationThreaded
  {
  public:
    void Segment(BIAS::Image<unsigned char> &src, 
      BIAS::Image<unsigned char> &res, 
      int hue,int hueIntervall,int minSaturation);
  protected:
    static void *Segment_(void *data);

  };

}

#endif
