
#include <bias_config.h>

#ifndef BIAS_HAVE_XML2
#  error You need XML2 to compile this code. Please enable USE_XML2 and recompile BIAS. 
#endif // BIAS_HAVE_XML2


#include <Image/CameraParam.hh>
#include <Image/UndistortionMapping.hh>
#include <Image/MatlabTBUndistortion.hh>
#include <Base/Image/ImageIO.hh>
#include <iostream>

using namespace BIAS;
using namespace std;



int main(int argc, char** argv )
{
  if (argc<2 ) {
    cout <<"Usage: ExampleUndistort <image>"<<endl;
    exit(1);
  }
  CameraParam cp;
  UndistortionMapping<unsigned char, unsigned char> um;

  Image<unsigned char> src, dst, dst2, dstum, dstum2;
  // double xDst, yDst;
  string camfile = "MuseumNorwayCam.xml";
  cp.ReadCameraData(camfile);
  if (cp.IsInitialized() && cp.IsPerspective()) {
    cout << camfile << " used for initialization." << endl;
    um.SetCameraParam(cp);
  } else {
    cout << "Error: could not read camera data file or no perspective lens!" 
      << endl;
    BIASABORT;
  }

  // source image is distorted
  if (argc>1) {
    if (ImageIO::Load(argv[1], src)!=0){
      BIASERR("error loading image "<<argv[1]);
      return -1;
    }
  }

  um.UndistortPerspImage(src,dstum);
  ImageIO::Write("UndistortedImage", dstum);
  cout << " created: "<<"UndistortedImage.mip"<<endl;

  um.DistortPerspImage(dstum,dstum2);
  ImageIO::Write("RedistortedImage.pgm", dstum2);
  cout << " created: "<<"RedistortedImage.mip"<<endl;

  return 0;  
}
