/* 
This file is part of the BIAS library (Basic ImageAlgorithmS).

Copyright (C) 2003, 2004    (see file CONTACTS for details)
Multimediale Systeme der Informationsverarbeitung
Institut fuer Informatik
Christian-Albrechts-Universitaet Kiel


BIAS is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or
(at your option) any later version.

BIAS is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with BIAS; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "Image/ProjectionMapping.hh"
#include <Geometry/RMatrix.hh>
#include <Base/Geometry/HomgPoint2D.hh>
#include <Geometry/Projection.hh>

using namespace BIAS;
using namespace std;


template <class InputStorageType, class OutputStorageType>
ProjectionMapping<InputStorageType, OutputStorageType>::
~ProjectionMapping() 
{}

template <class InputStorageType, class OutputStorageType>
ProjectionMapping<InputStorageType, OutputStorageType>::
ProjectionMapping()
: SourceP_(ProjectionParametersPerspective()), 
  SinkP_(ProjectionParametersPerspective())
{}


template <class InputStorageType, class OutputStorageType>
ProjectionMapping<InputStorageType, OutputStorageType>::
ProjectionMapping(const ProjectionMapping & src) 
{
    (*this) = src;
}

template <class InputStorageType, class OutputStorageType>
ProjectionMapping<InputStorageType, OutputStorageType> & 
ProjectionMapping<InputStorageType, OutputStorageType>::
operator=(const ProjectionMapping & src)
{
     SourceP_ = src.SourceP_;
     SinkP_ = src.SinkP_;     
     return *this;
}

template <class InputStorageType, class OutputStorageType>
int ProjectionMapping<InputStorageType, OutputStorageType>::
GetSourceCoordinates_(const HomgPoint2D & sink,
                      HomgPoint2D & source) const {
  HomgPoint3D point3D;

  point3D = SinkP_.UnProjectToPoint(sink,1.0);
  source = SourceP_.Project(point3D);  
  return 0;
}

// TODO FIXME win32 template instantiation problem (JW)
template class ProjectionMapping<unsigned char, unsigned char>;
template class ProjectionMapping<float, float>;
template class ProjectionMapping<unsigned char, float>;

#if defined(BUILD_IMAGE_CHAR)
// TODO FIXME win32 template instantiation problem (JW)
template class ProjectionMapping<unsigned char, char>;
template class ProjectionMapping<char, char>;
#endif

#if defined(BUILD_IMAGE_USHORT)
template class ProjectionMapping<unsigned short,unsigned short>;
#endif

#if defined(BUILD_IMAGE_SHORT)
template class ProjectionMapping<short, short>;
#endif

#if defined(BUILD_IMAGE_SHORT)&&defined(BUILD_IMAGE_USHORT)
template class ProjectionMapping<unsigned short, short>;
#endif

#if defined(BUILD_IMAGE_INT)
template class ProjectionMapping<int,int>;
#endif

#if defined(BUILD_IMAGE_USHORT)
template class ProjectionMapping<unsigned short,float>;
#endif

#if defined(BUILD_IMAGE_USHORT) && defined(BUILD_IMAGE_INT)
template class ProjectionMapping<unsigned short,int>;
#endif

#if defined(BUILD_IMAGE_DOUBLE)
template class ProjectionMapping<double,double>;
#endif
