/* 
This file is part of the BIAS library (Basic ImageAlgorithmS).

Copyright (C) 2003, 2004    (see file CONTACTS for details)
Multimediale Systeme der Informationsverarbeitung
Institut fuer Informatik
Christian-Albrechts-Universitaet Kiel


BIAS is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or
(at your option) any later version.

BIAS is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with BIAS; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include <Base/Common/BIASpragmaStart.hh>

#ifndef __ProjectionMapping_hh__
#define __ProjectionMapping_hh__

#include <Image/BackwardMapping.hh>
#include <Geometry/HMatrix.hh>
#include <Geometry/Projection.hh>


namespace BIAS {

  /** @class ProjectionMapping
      @ingroup g_filter
      @brief  Maps source pixel to sink pixel of given projections
      @author niemann
  */ 
  template <class InputStorageType, class OutputStorageType> 
  class BIASImage_EXPORT ProjectionMapping 
    : public  BackwardMapping<InputStorageType, OutputStorageType>  {
  public:

    virtual ~ProjectionMapping();

    ProjectionMapping();

    /**  required because assignment is 
         @author Jan Woetzel */
    ProjectionMapping(const ProjectionMapping & src);

    /**  required because of const members 
         @author Jan Woetzel */
    ProjectionMapping<InputStorageType, OutputStorageType> & 
    operator=(const ProjectionMapping & src);

    /** @brief set your source projection before calling Map() */
    inline void SetSourceCam(const Projection& P) { SourceP_ = P;};
    /** @brief set your sink projection before calling Map() */
    inline void SetSinkCam(const Projection& P) { SinkP_ = P;};

  protected:
    /** @brief reimplementation for Pixel transformation, 
        takes sink and computes coords in source */
    virtual int GetSourceCoordinates_(const HomgPoint2D& sink,
                                      HomgPoint2D& source) const;

    /// the projections for backward mapping: run over sink and compute
    /// source
    Projection SourceP_, SinkP_;

  };
} // namespace 

#endif

#include <Base/Common/BIASpragmaEnd.hh>

