/* 
This file is part of the BIAS library (Basic ImageAlgorithmS).

Copyright (C) 2003, 2004    (see file CONTACTS for details)
  Multimediale Systeme der Informationsverarbeitung
  Institut fuer Informatik
  Christian-Albrechts-Universitaet Kiel


BIAS is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or
(at your option) any later version.

BIAS is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with BIAS; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
/** @example ExampleTracker2
    This little example demonstrates the 


    @ingroup g_tracker
    @author woelk */

#include <bias_config.h>

#include <Matcher2D/Tracker.hh>
#include <Base/Image/ImageDraw.hh>
#include <Base/Image/ImageConvert.hh>
#include <Base/Image/ImageIO.hh>
#include <FeatureDetector/CornerDetectorSusan.hh>
#include <FeatureDetector/CornerDetectorKLT.hh>
#include <Base/Debug/TimeMeasure.hh>
#include <Image/PyramidImage.hh>

#include <deque>

using namespace BIAS;
using namespace std;

#define DEFAULT_PARA_FILE_NAME "Tracker.ini"

#define type 1
#if type == 0
#define StorageType unsigned char
#define CalculationType int
#elif type == 1
#define StorageType float
#define CalculationType float
#endif

CornerDetectorBase<StorageType> *cd = NULL;
Tracker<StorageType, CalculationType> *tr = NULL;
PyramidImage<CalculationType> *pim[2], *gx[2], *gy[2];
int act=0, *cdt, *cdfreq, *pyind;
deque<vector<HomgPoint2D> > p;
deque<vector<KLT_TYPE> > residui;
deque<vector<int> > results;
vector<HomgPoint2D> mp;
vector<QUAL> qual;
vector<HomgPoint2D> cp;
vector<KLT_TYPE> r;
vector<int> res;
int tnum=0, cdnum=0;
TimeMeasure otimer, ttimer, cdtimer, pytimer;
bool draw_tracks=true;
const int max_num_points=150;

void Draw(Image<unsigned char>& im, deque<vector<HomgPoint2D> >& p, int imnum)
{
  unsigned char col[]={255, 0, 0};
  const int nump=p[0].size();
  unsigned start[2], end[2];
  const unsigned radius=1;
  int maxhist=0;

  if (imnum>0){
    for (int l=0; l<nump; l++){
      if (!p[imnum][l].IsAtInfinity() && !p[imnum-1][l].IsAtInfinity()){
        // circel around corner in actual image
        start[0]=(unsigned)rint(p[imnum][l][0]);
        start[1]=(unsigned)rint(p[imnum][l][1]);
        ImageDraw<unsigned char>::CircleCenterFilled(im, start[0], start[1], 
                                                     radius, col);
        if (draw_tracks){
          // lines for tracks
          for (int i=imnum; i>0; i--){
            if (p[i-1][l].IsAtInfinity()) {
              if ((imnum-i)>maxhist) maxhist=imnum-i;
              break;
            }
            start[0]=(unsigned)rint(p[i][l][0]);
            start[1]=(unsigned)rint(p[i][l][1]);
            end[0]=(unsigned)rint(p[i-1][l][0]);
            end[1]=(unsigned)rint(p[i-1][l][1]);
            if (i-1==0) maxhist=imnum;
            //cerr << "line: ("<<start[0]<<", "<<start[1]<<") -> ("<<end[0]<<", "<<end[1]<<")\n";
            ImageDraw<unsigned char>::Line(im, start, end, col);
          }
        }
      } 
    }
  }
  //cerr << "max track length "<<maxhist<<endl;
}



void DumpTracks(deque<vector<HomgPoint2D> >& p)
{
  int numim=0;
  while (p[numim].size()>0 && numim<(int)p.size()) numim++;
  for (unsigned l=0; l<p[0].size(); l++){
    cout <<setw(3)<<l<<" : ";
    for (int i=0; i<numim; i++){
      cout.precision(4);
      fprintf(stdout, "%i:(%5.1f, %5.1f) ", i, p[i][l][0], p[i][l][1]);
      //      cout <<setw(10)<<i<<":("<<p[i][l][0]<<", "<<p[i][l][1]<<") ";
    }
    cout << endl;
  }
}




/** the drawing algorithm needs one point at infinity between subsequent 
    tracks, therefor the last point of the last track is set to (0, 0, 0) */
void InsertZeroPoints(deque<vector<HomgPoint2D> >& p, int i)
{
  if (i>0){
    const int nump=p[i].size();
   
    for (int k=0; k<nump; k++){
      if (p[i][k].IsAtInfinity() && !p[i-1][k].IsAtInfinity()){
        p[i-1][k].Set(0.0, 0.0, 0.0);
      }
    }
  }
}

void Track(const Image<CalculationType>& im, Image<unsigned char>& rgbim, int i)
{
  otimer.Start();
  TimeMeasure mcdtimer;


  pytimer.Start();
  tr->PreparePyramide(im, *pim[act], *gx[act], *gy[act]);
  pytimer.Stop();

  ttimer.Start();
  tr->Track(*pim[act], *gx[act], *gy[act]);
  ttimer.Stop();
  tnum++;

  if (i==0){
    cdtimer.Start();
    CornerDetectorGradient<StorageType, CalculationType> *cdgrad
      = dynamic_cast<CornerDetectorGradient<StorageType,CalculationType>*>(cd);
    if (cdgrad){
      cdgrad->Detect(*(*gx[act])[*pyind], *(*gy[act])[*pyind], mp, qual);
    } else {
      cd->Detect(*(*pim[act])[*pyind], mp, qual);
    } 
    //cerr << "found "<<mp.size()<<" points\n";
    //    mp.resize(*numfeat);
    //qual.resize(*numfeat);
    tr->AddNumPoints(mp, qual, max_num_points);
    cerr << "added "<<mp.size()<<" points\n";
    cdtimer.Stop();
    cdnum++;
  }

  tr->GetPoints(mp);
  p.push_back(mp);

#if type == 0
  ImageConvert::Convert(*(*pim[act])[*pyind], rgbim, ImageBase::CM_RGB);
#elif type == 1
  Image<unsigned char> ucim;
  ImageConvert::ConvertST(*(*pim[act])[*pyind], ucim, ImageBase::ST_unsignedchar);
  ImageConvert::Convert(ucim, rgbim, ImageBase::CM_RGB);
#endif
  Draw(rgbim, p, i);
    

  if ((i % *cdfreq)==0){
    //DumpTracks(p);
    cdtimer.Start();
    //mcdtimer.Start();
    cd->Detect(*(*pim[act])[*pyind], cp, qual);
    //mcdtimer.Stop();
    //cerr << "corner detection took "<<mcdtimer.GetRealTime()<<" us\n";
    //mcdtimer.Reset();
    //mcdtimer.Start();
 
    InsertZeroPoints(p, i);
    //DumpTracks(p);
    int num=tr->FillUpPoints(cp, qual);
    cerr << "filled up "<<num<<" points\n";
    tr->GetPoints(mp);
    p.pop_back();
    p.push_back(mp);
    //mcdtimer.Stop();
    //cerr << "corner insertion took "<<mcdtimer.GetRealTime()<<" us\n";
    cdtimer.Stop();
    //DumpTracks(p);
    cdnum++;
  }
  tr->GetResiduiMAD(r);
  residui.push_back(r);
  tr->GetResults(res);
  results.push_back(res);
  otimer.Stop();
}



/// the main function
int main(int argc, char *argv[])
{
  TimeMeasure timer;
  timer.Start();
  Param para;
  //  para.AddDebugLevel(D_PARAM_CONF_FILE);

  Image<unsigned char> ucim, rgbim;
  Image<StorageType> im, im2;
  int grp=para.GetFreeGroupID();
  tr = new Tracker<StorageType, CalculationType>(para, grp, "");
  para.SetGroupName(grp, "Tracker");

  grp=para.GetFreeGroupID();
  for (int i=0; i<2; i++){
    pim[i] = new PyramidImage<CalculationType>(para, grp, "");
    gx[i] = new PyramidImage<CalculationType>(para, grp, "");
    gy[i] = new PyramidImage<CalculationType>(para, grp, "");
  }
  para.SetGroupName(grp, "PyramidImage");

  bool DEBUG=true;
  act=0;

  grp=para.GetFreeGroupID();
#if type == 0
  CornerDetectorSusan<StorageType> cds(para, grp, "");
#endif
  CornerDetectorKLT<StorageType, CalculationType> cdklt(para, grp, "");
  cdt=CornerDetectorBase<StorageType>::
    AddCornerDetectorType(para, "CornerDetectorType", grp);
  para.SetGroupName(grp, "CornerDetector");
  cdfreq=para.AddParamInt("CornerDetectorFreq", "corner detector fills up points every n th frame", 5, 0, INT_MAX, 0, GRP_ALL);
  //numfeat=para.AddParamInt("NumFeatures", "track n features", 250, 0, INT_MAX, 0, GRP_ALL);
                                     

  int argind = para.UpdateParameter(argc, argv, DEFAULT_PARA_FILE_NAME);
  pyind = para.GetParamInt("PyIndex-Tracker");

  switch (*cdt){
  case CornerDetectorBase<StorageType>::CDT_Susan:
#if type == 0
    cd = &cds;
#else
    BIASERR("susan only implemented for unsigned char");
    cd = NULL;
#endif
    break;
  case CornerDetectorBase<StorageType>::CDT_KLT:
    cd = &cdklt;
    break;
  default:
    BIASERR("unknown corner detector type");
    cd = NULL;
    break;
  }
  //cd->AddDebugLevel(D_CD_WRITE_DEBUG_IM);
  //cd->AddDebugLevel(D_CD_FEATURES);

  //tr->AddDebugLevel(D_TRACKER_INIT);
  //tr->AddDebugLevel(D_TRACKER_PARAM);
  //tr->AddDebugLevel(D_TRACKER_TRACK);
  //tr->AddDebugLevel(D_TRACKER_WRITE_IM);
  //tr->AddDebugLevel(D_TRACKER_FILL_UP);

  if (argc-argind<2 || argind<1){
    cerr << argv[0] << " [parameter] <im1> <im2> [ <im3> ... ] \n";
    para.Usage(cerr);
  para.DisableDestructorWarning();
    return -2;
  } 

  tr->ParameterChanged();

  Rescale<float, float> resc;

  for (int i=argind; i<argc; i++){
    timer.Stop();
    cerr << 1e6/(double)timer.GetRealTime() << "fps\n";
    timer.Reset();
    timer.Start();
    act=!act;
#if type == 0
    if (ImageIO::Load(argv[i], im)!=0){
#else
    if (ImageIO::Load(argv[i], ucim)!=0){
#endif
      BIASERR("error loading image "<<argv[i]);
      para.DisableDestructorWarning();
      return -1;
    } else {
      if (DEBUG) cerr << "read "<<argv[i]<<endl;
    }

#if type != 0
    if (ImageConvert::ConvertST(ucim, im, ImageBase::ST_float)!=0){
      BIASERR("error converting image "<<argv[i]);
    }
#endif

    //im.SetROI(0, 80, 640, 400);
    //  im.SetROI(160, 160, 480, 320); 
    
    //if (i==argind) im2.Init(im.GetWidth()>>1, im.GetHeight()>>1, 1);
    //resc.DownsampleBy2Grey(im, im2);
    //im.SetROI(0, 40, 320, 190);
    Track(im, rgbim, i-argind);

    ostringstream name;
    name << "track-"<<setw(4)<<setfill('0')<<i-argind<<".mip";
    ImageIO::Write(name.str(), rgbim);
    
  }
  
  tr->WriteTrackFile(p, results, "features.txt");
    
  //cout << "\nTracks: "<<endl;
  //DumpTracks(p);
  //cout << "\n";

  cout << "preparing pyramid images took :"
       <<pytimer.GetRealTime()/(float)tnum
       <<" us / image\n";
  
  cout << "tracking took :"<<ttimer.GetRealTime()/(float)tnum
       <<" us / image\n";
  
  cout << "detecting corners took: "<<cdtimer.GetRealTime()/(float)cdnum
       <<" us / detection\n";

  cout << "overall tracking corners took: "<<otimer.GetRealTime()/(float)tnum
       <<" us / detection\n";

  para.DisableDestructorWarning();
  return 0;
}

