/* 
This file is part of the BIAS library (Basic ImageAlgorithmS).

Copyright (C) 2003, 2004    (see file CONTACTS for details)
  Multimediale Systeme der Informationsverarbeitung
  Institut fuer Informatik
  Christian-Albrechts-Universitaet Kiel


BIAS is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or
(at your option) any later version.

BIAS is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with BIAS; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#ifndef __TrackerBaseWeighted_hh__
#define __TrackerBaseWeighted_hh__

namespace BIAS {

  /** @class TrackerBase
      @ingroup g_tracker
      @brief  
      @author woelk/kollmann 06/2006 */
  template <class StorageType>
  class BIASMatcher2D_EXPORT TrackerBaseWeighted 
    : public TrackerBaseInterface<StorageType>
  {
  public :
    TrackerBaseWeighted();

  protected:
    Matrix<KLT_TYPE> _weight; // the weight matrix used in TrackWeighted_

    /** p1 has length 2, 2d coo
        p2 has length 2, 2d coo
        result has length 2, 2d coo
        @author woelk/kollmann 06/2006 */
    virtual int Track_(Vector<KLT_TYPE>& p1, Vector<KLT_TYPE>& p2,
                       Vector<KLT_TYPE>& result, KLT_TYPE& error, 
                       int &iter);

    /** Same as TrackSimple_ but also uses the weighting matrix _weight.
        @author woelk */
    int TrackWeighted_(KLT_TYPE p1[2], KLT_TYPE p2[2], KLT_TYPE result[2], 
                       KLT_TYPE& error, int &iter);

    /** Constructs the weight matrix _weight with size _WinSize, _WinSize 
        and fills it with gaussian weights
        @author woelk */
    void CalcGaussianWeightMatrix_();


  }; 
}

#endif // __TrackerBaseWeighted_hh__
