/* 
This file is part of the BIAS library (Basic ImageAlgorithmS).

Copyright (C) 2003, 2004    (see file CONTACTS for details)
  Multimediale Systeme der Informationsverarbeitung
  Institut fuer Informatik
  Christian-Albrechts-Universitaet Kiel


BIAS is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or
(at your option) any later version.

BIAS is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with BIAS; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/



  /** This is an example file on how to use the RANSAC algorithm.
      It tries to find a dominant plane in a number of 3d points.
      
      RANSAC:
      In GetSampleSolutions, 3 of the points are picked and a plane is 
      constructed from them.

      In EvaluateSolution, it is checked, howe many points are "inliers" to 
      that plane.

      In RefineSiolution, least squares is done on all inliers to get a
      better plane. 

      You have to set the inlier threshold (max distance to plane to accept
      a point as being on the plane) and the fraction of valid 3d points,
      which defines the number of samples performed by RANSAC.

      When you call SolveMaster then, you will get the best Plane according 
      to your input.
  */

#include <Base/Common/BIASpragma.hh>

#include "Base/Math/Random.hh"
#include <vector>
#include <iostream>
#include <fstream>
#include <iomanip>
#include <float.h>
#include "ExampleRANSACPlane.hh"
  // #include "Utils/ThreeDOut.hh"

using namespace BIAS;
using namespace std;

void readFile(char *filename, std::vector<HomgPoint3D> &points) 
{
  ifstream infile(filename);
  if (!infile) {
    BIASERR("couldnt open:"<<filename);
    return;
  }
  char c;
  HomgPoint3D point(0,0,0,1);
  while (!infile.eof()) {
    infile >> point[0] >> point[1]>>point[2]>>c;
    if (!infile.eof()) {
      points.push_back(point);
      cout <<" reading point "<<point<<endl;
    }
  }
}

int main(int argc, char *argv[])
{ 
  int res =0;
  double inlier_fraction=0.1;
  bool refine_solution=true;
  double  maxsquareddistance = 0.00001;
 
  HomgPlane3D solution;
  vector<bool> inliers;
  bool debug=false;
  PlaneRANSAC dr;
 

  if (argc<2) {
    cout<<"enter filename to read points from..."<<endl;
    exit(-1);
  }
  vector<HomgPoint3D> points;
  readFile(argv[1], points);

  if (debug){
    dr.SetDebugLevel(dr.GetDebugLevel() | D_RANSAC_SAMPLES);
    dr.SetDebugLevel(dr.GetDebugLevel() | D_RANSAC_SOLVE_MASTER);
    //   dr.SetDebugLevel(dr.GetDebugLevel() | D_DOUBLE_RANSAC_SOLUTION);
  }


 
  dr.Init(points, points.size(), maxsquareddistance, refine_solution, 3,
          inlier_fraction, -1.0);
  
  dr.SolveMaster(inlier_fraction, solution, inliers);
    

  /* // compute covmatrix to draw in vrml style
  Vector3<double> center(0,0,0);
  unsigned int numinliers = 0;
  for (unsigned int i=0; i<inliers.size(); i++) {
    if (inliers[i]) {
      center[0] += points[i][0];
      center[1] += points[i][1];
      center[2] += points[i][2];
      numinliers++;
    }
  }
  if (numinliers>0) center /= (double) numinliers;
  CovMatrix3x3 C;
  C.SetZero();
  for (unsigned int i=0; i<inliers.size(); i++) {
    if (inliers[i]) {
      Vector3<double> diff;
      for (unsigned int j=0; j<3; j++) diff[j] = points[i][j]-center[j];
      CovMatrix3x3 CCurrent(diff.OuterProduct(diff));
      for (unsigned int l=0;l<3;l++) {
        for (unsigned int k=0;k<3;k++) {
          C[l][k] += CCurrent[l][k];
        }
      }
    }
  }
  C /= (double)numinliers;

  
  BIAS::ThreeDOutParameters tparams;  
  ThreeDOut T(tparams);
  
  T.AddEllipsoid(C,center, RGBA(255,255,0,0), 1.0);
  T.VRMLOutEllipsoids(cout);
  */
  cout <<"solution is "<<solution<<endl;
  return res;
}
