/* 
This file is part of the BIAS library (Basic ImageAlgorithmS).

Copyright (C) 2003, 2004    (see file CONTACTS for details)
  Multimediale Systeme der Informationsverarbeitung
  Institut fuer Informatik
  Christian-Albrechts-Universitaet Kiel


BIAS is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or
(at your option) any later version.

BIAS is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with BIAS; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "bias_config.h"
#include <MathAlgo/GSL.hh>

#ifdef BIAS_HAVE_GSL
#include <gsl/gsl_cdf.h>
#include <gsl/gsl_randist.h>
#include <gsl/gsl_sf_gamma.h>

double 
BIAS::ChiSquareCulmProbFun(double x, int deg_freedom)
{
  return gsl_cdf_chisq_P(x, deg_freedom);
}

double 
BIAS::InvChiSquareCulmProbFun(double x, int deg_freedom)
{
  return gsl_cdf_chisq_Pinv(x, deg_freedom);
}

double 
BIAS::Gamma(double x)
{
  return gsl_sf_gamma (x);
}

double 
BIAS::InvGamma(double x)
{
  return gsl_sf_gammainv (x);
}

double 
BIAS::BetaDistribution(double a, double b, double x)
{
  return gsl_ran_beta_pdf(x, a, b);
}

#endif // BIAS_HAVE_GSL



