/* 
This file is part of the BIAS library (Basic ImageAlgorithmS).

Copyright (C) 2003, 2004    (see file CONTACTS for details)
  Multimediale Systeme der Informationsverarbeitung
  Institut fuer Informatik
  Christian-Albrechts-Universitaet Kiel


BIAS is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or
(at your option) any later version.

BIAS is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with BIAS; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifndef __GSL_hh__
#define __GSL_hh__

#include <bias_config.h>

namespace BIAS{

#ifndef BIAS_HAVE_GSL
#error recompile BIAS with gsl support enabled
#else

  /** @addtogroup mathalgo
      @brief C++ Wrappers for gnu scientific library (GSL)
      @author woelk 02/2006 */
  BIASMathAlgo_EXPORT
  double ChiSquareCulmProbFun(double x, int deg_freedom);

  /** @addtogroup mathalgo
      @brief C++ Wrappers for gnu scientific library (GSL)
      @author woelk 02/2006 */
  BIASMathAlgo_EXPORT 
  double InvChiSquareCulmProbFun(double x, int deg_freedom);

  /** @addtogroup mathalgo
      @brief C++ Wrappers for gnu scientific library (GSL)
      \Gamma(x) = \int_0^\infty dt  t^{x-1} \exp(-t)
      The gamma function, x cannot be greater than 171.0 and x cannot be 
      a negative integer
      @author woelk 02/2006 */
  BIASMathAlgo_EXPORT 
  double Gamma(double x);

  /** @addtogroup mathalgo
      @brief C++ Wrappers for gnu scientific library (GSL)
      1/Gamma(x)
      @author woelk 02/2006 */
  BIASMathAlgo_EXPORT 
  double InvGamma(double x);

  /** @addtogroup mathalgo
      @brief C++ Wrappers for gnu scientific library (GSL)
      p(x) dx = {\Gamma(a+b) \over \Gamma(a) \Gamma(b)} x^{a-1} (1-x)^{b-1} dx
      for 0 <= x <= 1.
      @author woelk 02/2006 */
  BIASMathAlgo_EXPORT 
  double BetaDistribution(double a, double b, double x);
   
  
#endif // BIAS_HAVE_GSL

} // namespace 
#endif // __GSL_hh__
