#include <bias_config.h>

#include <Gui/ImageCanvas.hh>
#include <Gui/StringConv.hh>
#include <Image/Camera.hh>
#include <Image/StereoRedGreen.hh>
#include <VideoSource/VideoSource_Base.hh>

#include <VideoSource/VideoSource_Net.hh>
#include <VideoSource/VideoSource_Shm.hh>

// cideo for linux
#ifdef BIAS_HAVE_V4L
#  include <VideoSource/VideoSource_V4L.hh>
#endif // BIAS_HAVE_V4L

//include direct show and ueye interface?
#ifdef BIAS_HAVE_DSHOW
#  include <VideoSource/VideoSource_DSHOW.hh>
#endif

#  ifdef BIAS_HAVE_UEYE
#    include <VideoSource/VideoSource_uEye.hh>
#  endif // BIAS_HAVE_UEYE

#ifdef BIAS_HAVE_DCAM
#  include <VideoSource/VideoSource_DCAM.hh>
#endif

#include <iostream>
#include <Base/Image/ImageIO.hh>
#include <Base/Debug/TimeMeasure.hh>
#include <wx/wx.h>
#include <wx/listbox.h>

using namespace std;
using namespace BIAS;


enum wxIDs {id_quit = 42,id_record,id_config, id_timer,id_save,id_key, id_save2,id_pin};  


ofstream logstr;

int delay_ =20;
int dSaveCounter_;
bool bSaveImages_;
bool bShowStereo_ = false;

class MyFrame : public wxFrame{
public:
  MyFrame(const wxString& title, const wxPoint& pos, const wxSize& size, VideoSource *cam, VideoSource *cam2)
    :wxFrame((wxFrame *)NULL, -1, title, pos, size)
  {
    Timer_.SetOwner(this, id_timer);
    FrameCounter_ = 0;

    Cam_ = cam;
#ifdef BIAS_HAVE_DSHOW
    if( (dynamic_cast<VideoSource_DSHOW*>(Cam_)) != NULL)
    {
      (dynamic_cast<VideoSource_DSHOW*>(Cam_))->InitImage(CamImg_);
    }
    else
#endif
      Cam_->InitImage(CamImg_);
    if (cam2){
      Cam2_ = cam2;
      Cam2_->InitImage(CamImg2_);
    } else {
      Cam2_ = NULL;
    }
    wxMenu *menuFile = new wxMenu;

    menuFile->Append( id_save, wxT("Grab single image") ); 
    menuFile->Append( id_save2, wxT("Grab continuously") );

    menuFile->Append( id_quit, wxT("E&xit") );
    wxMenuBar *menuBar = new wxMenuBar;
    menuBar->Append(menuFile, wxT("&File"));
#ifdef BIAS_HAVE_DSHOW
    if( (dynamic_cast<VideoSource_DSHOW*>(Cam_)) != NULL)
    {
      wxMenu * menuDShow  = new wxMenu;
      menuDShow->Append( id_config, wxT("Display DSHOW camera configuration"));
      menuDShow->Append( id_pin, wxT("Display DSHOW source pin configuration"));
      menuBar->Append(menuDShow, wxT("&DShow"));
    } 
#endif
    SetMenuBar(menuBar);
    CreateStatusBar(5);
    lastwidth = CamImg_.GetWidth();
    lastheight = CamImg_.GetHeight();
    IC_ = new ImageCanvas(this,GetStatusBar(), 0,-1,wxDefaultPosition,
      //wxSize(lastwidth,lastheight));
      wxSize(640,480));
    wxBoxSizer *vbox = new wxBoxSizer(wxVERTICAL);
    vbox->Add(IC_,1,  wxGROW |wxALL , 5);
    vbox->SetSizeHints(this);
    SetSizer(vbox);
    SetAutoLayout(true);
    Fit();  
    Show(TRUE);

    wxString text;
    text.Printf(wxT("size: %4dx%4d"),CamImg_.GetWidth(), CamImg_.GetHeight());
    SetStatusText(text, 2);


    bSaveImages_ = false;
    dSaveCounter_ = 0;
    cout <<"Setting timer to "<<delay_<<" ms"<<endl;
    Timer_.Start(delay_,true);
    StopWatch_.Start();
  }

  void OnKey(wxKeyEvent& event)
  {
    int code = event.GetKeyCode();
    switch(code){
case '1':
#ifdef BIAS_HAVE_DSHOW
  if(Cam_!=NULL)
  {
    if((dynamic_cast<BIAS::VideoSource_DSHOW *> (Cam_)) != NULL)
      (dynamic_cast<BIAS::VideoSource_DSHOW *> (Cam_))->DisplayCameraProperties();
  }
#endif
  break;
case 's':
  bSaveImages_ = !bSaveImages_;
  cout<<"saving images...= "<<bSaveImages_<<endl;
  break;
    }  
  }

  void OnQuit(wxCommandEvent& WXUNUSED(event))
  {
    Timer_.Stop();
    if(Cam_){
      Cam_->PostGrab();     
      Cam_->CloseDevice();
      //delete Cam_;
    }
    if (Cam2_){
      Cam2_->PostGrab();     
      Cam2_->CloseDevice();
    }
    //if(IC_)
    //delete IC_;
    Close(TRUE);
  }

  void OnGrab(wxCommandEvent& WXUNUSED(event))
  { 
    wxString text;
    text.Printf(wxT("ShowCamWX-%05d.mip"),FrameCounter_);
    ImageIO::Write(WxToAscii(text),CamImg_);
    cout <<"Saved image: "<<text<<endl;
  }

  void OnTimer(wxTimerEvent & /*evt*/)
  {
    float fps;
    Timer_.Stop();
    StopWatch_.Stop();
#ifdef WIN32
    fps = 1000.0 / StopWatch_.GetRealTime();
#else
    fps = 1000000.0 / StopWatch_.GetRealTime();
#endif
    StopWatch_.Reset();
    StopWatch_.Start();
    FrameCounter_++;
    wxString text;
    text.Printf(wxT("%05d %2.1f fps "),FrameCounter_,fps);
    SetStatusText(text, 1);
    Cam_->GrabSingle(CamImg_);
    if (Cam2_){
      Cam2_->GrabSingle(CamImg2_);
      static StereoRedGreen comb;
      static Camera<unsigned char> stereo;
      comb.Combine(CamImg_, CamImg2_, stereo);
      ShowImg_ = stereo;
    } else {
      ShowImg_ = CamImg_;
    }
#ifdef BIAS_HAVE_DSHOW
    if( (dynamic_cast<VideoSource_DSHOW*>(Cam_)) != NULL)
    {
      ShowImg_.Flip();
    }
#endif
    IC_->Show(ShowImg_,"foo"); 
    unsigned int w = ShowImg_.GetWidth();
    unsigned int h = ShowImg_.GetHeight();
    text.Printf(wxT("size: %4dx%4d"),w, h);
    SetStatusText(text, 2);

    if (w != lastwidth || h != lastheight) {
      lastwidth = w; lastheight = h;
      IC_->SetSize(w,h);
      Fit();
      cout <<"Refit"<<endl;
    }

    if(bSaveImages_){
      cout<<"Saving image: "<<dSaveCounter_<<endl;
      text.Printf(wxT("image_%05d"),dSaveCounter_++);
      BIAS::ImageIO::Write(WxToAscii(text),ShowImg_);

    }
    Timer_.Start(delay_,true);
  }   


  void OnGrabContinuous(wxCommandEvent& WXUNUSED(event))
  {    
    bSaveImages_ = !bSaveImages_;
    cout<<"Saving images= "<<bSaveImages_;
  }

  void OnConfig(wxCommandEvent& WXUNUSED(event))
  {
#ifdef BIAS_HAVE_DSHOW
    if(Cam_!=NULL)
    {
      if((dynamic_cast<BIAS::VideoSource_DSHOW *> (Cam_)) != NULL)
        (dynamic_cast<BIAS::VideoSource_DSHOW *> (Cam_))->DisplayCameraProperties();
    }
#endif
  }

  void OnConfigPin(wxCommandEvent& WXUNUSED(event))
  {
#ifdef BIAS_HAVE_DSHOW
    if(Cam_!=NULL)
    {
      if((dynamic_cast<BIAS::VideoSource_DSHOW *> (Cam_)) != NULL)
        (dynamic_cast<BIAS::VideoSource_DSHOW *> (Cam_))->DisplaySourcePinProperties();
    }
#endif
  }

protected:
  VideoSource *Cam_;
  VideoSource *Cam2_;
  ImageCanvas  *IC_;
  wxStatusBar *StatusBar_;
  TimeMeasure StopWatch_; 
  unsigned int FrameCounter_;
  unsigned int lastwidth,lastheight;
  Camera<unsigned char> CamImg_, CamImg2_, ShowImg_;
  wxTimer Timer_;
  DECLARE_EVENT_TABLE()

};



BEGIN_EVENT_TABLE(MyFrame, wxFrame)
EVT_MENU(id_quit, MyFrame::OnQuit)
EVT_MENU(id_config, MyFrame::OnConfig)
EVT_MENU(id_pin, MyFrame::OnConfigPin)
EVT_MENU(id_save, MyFrame::OnGrab)
EVT_TIMER(id_timer, MyFrame::OnTimer)
EVT_KEY_DOWN(MyFrame::OnKey)
EVT_MENU(id_save2, MyFrame::OnGrabContinuous)
END_EVENT_TABLE()  









// ================= MyApp =========================

class MyApp: public wxApp
{
public:
  virtual bool OnInit();
  ~MyApp();
  MyFrame *ToplevelWindow_;
};


void Usage()
{
  wxMessageDialog 
    usage(NULL,wxT("Usage: \n"
    "ShowCamWX -n | -N ip-string -P port |-v|-d [-w width] [-h height] [-i <string>]\n "
    "  -n: VideoSource_Net (UDP)\n"
    "  -N string (ip addr.): VideoSource_Net (TCP)\n"
    "         -P int (port): VideoSource_Net (TCP)\n"
    "  -m: VideoSource_Shm\n"
    "  -D: string (WDM device name) VideoSource_DSHOW\n"
    "  -d: VideoSource_DCam\n"
    "  -u: VideoSource_Ueye, only available if Windows compiled and with Option BUILD_UEYE\n" 
    "  -p: Portnumber to listen to\n"
    "  -v: VideoSource_V4L\n"
    "  -i string : device/interface\n"
    "  -I string : 2nd device/interface"));
  usage.ShowModal();

  exit(0);
}

bool MyApp::OnInit()
{
  logstr.open("ShowCamWx.log");
  logstr <<"Start"<<endl;
  VideoSource *Cam=NULL;
  VideoSource *Cam2=NULL;
  bool dcam,net,v4l,TCP,shm,dshow,ueye;
  dcam=v4l=net=TCP=shm=dshow=ueye= false;
  unsigned int w,h;
  unsigned int port=6002;
  w=h=0; 
  float fps=0;
  int c=0;
  string device,ip_addr;
  string device2;

  WxToAsciiArray argvAscii(argv, argc);

  while ((c = getopt(argc, argvAscii, "nN:P:mdvx:y:f:i:I:D:u")) != -1)
  {
    switch (c){
case 'd': dcam = true ;break;
case 'm': shm = true;break;
case 'n': net = true;break;
case 'N': net = true;TCP=true;ip_addr=optarg;break;
case 'P': port=atoi(optarg);break;
case 'v': v4l = true;break;
case 'x': w=atoi(optarg); break;
case 'y': h=atoi(optarg); break;
case 'f': fps=atof(optarg); break;
case 'i': device = optarg; break;
case 'I': device2 = optarg; break;
case 'p': port = atoi(optarg); break;
case 'u':
#ifdef BIAS_HAVE_UEYE
  ueye =true;
#else
  Usage();
#endif
  break;
case 'D': device = optarg; dshow = true; 
#ifndef WIN32
  Usage();
#endif
#ifndef BIAS_HAVE_DSHOW
  Usage();
#endif
  break;
case '?': 
case 'h': 
case ':': 
  Usage(); break;

default:
  break;
    }
  }
  logstr <<"getopt done"<<endl;

  if (dcam && Cam==NULL) {
#ifdef BIAS_HAVE_DCAM
    VideoSource_DCAM *vsd = new VideoSource_DCAM;
    vsd->SetDebugLevel(D_DCAM);
    Cam=vsd;
    if (device2 != ""){
      Cam2 = new VideoSource_DCAM;
      Cam2->SetDebugLevel(D_DCAM);
    }
#else
    cerr <<"DCam support disabled during compilation of BIAS"<<endl;
    exit(1);
#endif
  }


  if (net && Cam==NULL) {
    logstr <<"VS_NET"<<endl;
    VideoSource_Net *vsn = new VideoSource_Net;
    vsn->OpenDevice( ip_addr,port);
    //      vsn->SetUseTCP(TCP);
    Cam=vsn;
  }

  if (shm && Cam==NULL) {
    logstr <<"VS_SHM"<<endl;
    VideoSource_Shm *vsshm = new VideoSource_Shm;
    Cam=vsshm;
    if (device2 != ""){
      Cam2 = new VideoSource_Shm;
    }
  }


  if (v4l && Cam==NULL) {
#ifdef BIAS_HAVE_V4L
    logstr <<"VS_V4L"<<endl;
    VideoSource_V4L *vsv4l = new VideoSource_V4L;
    Cam=vsv4l;
#else
    exit(1);
#endif
  }

#ifdef WIN32
#ifdef BIAS_HAVE_DSHOW
  if(dshow && Cam==NULL) {
    logstr <<"VS_DSHOW "<<endl;
    VideoSource_DSHOW *dsh = new VideoSource_DSHOW(false);
    //dsh->SetSizeAndColorModel(320,240,3,BIAS::ImageBase::CM_YUV420P);
    Cam=dsh;
  }
#endif
#endif //WIN32

#ifdef BIAS_HAVE_UEYE
  if(ueye && Cam==NULL) {
    logstr <<"VS_UEYE "<<endl;
    VideoSource_uEye *eye = new VideoSource_uEye;
    eye->SetSize(640,480);
    eye->SetLeftTop(56,0);
    Cam=eye;
  }
#endif // BIAS_HAVE_UEYE


  if (Cam==NULL) Usage();
  if (fps>0) {
    Cam->SetFPS(fps);
    if (Cam2) Cam2->SetFPS(fps);
  }
  if (!ueye && !dshow){
    Cam->SetSize(w,h);
    if (Cam2) Cam2->SetSize(w,h);
  }



  if (device != ""){ 
    Cam->OpenDevice(device.c_str());
    logstr <<"We have a specified device"<<endl;
  }
  else{
    Cam->OpenDevice();
    logstr <<"No device specified, opened default"<<endl;	
  }

  if (device2 != ""){
    Cam2->OpenDevice(device2.c_str());
    logstr <<"We have a specified device"<<endl;
  } else if (Cam2) {
    BIASERR("something funny happened");
    BIASABORT;
  }

  cout << "--------------------------------PREGRAB" << endl;
  Cam->PreGrab();
  if (Cam2) Cam2->PreGrab();

  ToplevelWindow_ = new MyFrame(wxT("ShowCamWX"),wxPoint(50,50),
    wxSize(400,340),Cam, Cam2);

  //   char *Server;
  //   if (VideoSource_Net *vsn = dynamic_cast<VideoSource_Net *>(Cam)) 
  //   {
  //      Server = vsn->WaitForServerAddr();
  // 	 logstr <<"VS_Server is"<<Server<<endl;
  //      cout <<"Server is: "<<Server<<endl;
  //   }


  Cam->SetDebugLevel(0);
  if (Cam2) Cam2->SetDebugLevel(0);
  return TRUE;
}

MyApp::~MyApp()
{  
}


IMPLEMENT_APP(MyApp)
