/**
\todo implement command line switch to terminate biasviewwx automatically 
with useful return code for dart ADD_TEST to be able to test biasvviewwx 
run in nightly cmake Dart dashboard (JW)
*/

#include <wx/wx.h>
#include <wx/spinctrl.h>
#include <wx/dnd.h>
#include <wx/filename.h>
#include <wx/arrstr.h>


#include <string>
#include <sstream>
#include <limits>

#include <Gui/ImageCanvas.hh>
#include <Gui/ConsoleRedirectIO.hh>
#include <Gui/StringConv.hh>
#include <Base/Image/ImageIO.hh>
#include <Base/Common/FileHandling.hh>
#include <Image/Camera.hh>


#define FRAME_DEFAULT_SIZE  wxSize(700,500)

using namespace std;
using namespace BIAS;

#undef WXWIN_COMPATIBILITY


#if !wxUSE_DRAG_AND_DROP
#  error The dndFileTarget class requires drag and drop support in the WX library. Please recompile wxWidgets. __FILE__: __LINE__
#endif

/*
===============================================================================
MainFrame
===============================================================================
*/

enum
{
  // menu item IDs
  ID_Load,
  ID_Info,
  ID_MetaData,
  ID_Exit,
  ID_KeyInfo,
  ID_About,

  // toolbar item IDs
  ID_Next,
  ID_Prev,
  ID_RunStop,
  ID_Delay,
  ID_Fit,

  // timer ID
  ID_Timer
};

class MainFrame : public wxFrame
{

public:
  friend class DragAndDropTarget;

  MainFrame(const wxString& title, const wxPoint& pos, const wxSize& size);

  void SetFileList(const vector<string>& fileNames);
  void LoadImages(const wxArrayString &paths);

  /// fits the frame size if it is too small for curent image
  void FitSizetIfTooSmall();

  /// workaround adapting client canvases size
  void RefitClients();
  
protected:
  void LoadImageFromList(int index);

  ImageCanvas*        _ImageCanvasUC;
  ScaledImageCanvas*  _ImageCanvasFloat;
  vector<string>      _FileNames;
  int                 _CurrentImage;
  string              _MetaData;
  wxTimer             _Timer;
  bool                _BlockAnimation;
  wxSpinCtrl*         _DelaySpin;
  wxCheckBox*         _FitCheckBox;
  wxCheckBox*         _ReverseCheckBox;
  wxButton*           _RunStopButton;
  bool                _NeedReload;

  // for file open dialog
  wxString            _DefaultFile;
  wxString            _DefaultDir;

  // event handlers
  void OnLoad(wxCommandEvent& event);
  void OnInfo(wxCommandEvent& event);
  void OnMetaData(wxCommandEvent& event);
  void OnExit(wxCommandEvent& event);
  void OnKeyInfo(wxCommandEvent& event);
  void OnAbout(wxCommandEvent& event);

  void OnNext(wxCommandEvent& event);
  void OnPrev(wxCommandEvent& event);
  void OnRunStop(wxCommandEvent& event);
  void OnDelay(wxSpinEvent& event);
  void OnFit(wxCommandEvent& event);

  void OnTimer(wxTimerEvent& event);
  void OnIdle(wxIdleEvent& event);
  void OnSize(wxSizeEvent& event);

  DECLARE_EVENT_TABLE()
};

BEGIN_EVENT_TABLE(MainFrame, wxFrame)
EVT_MENU    (ID_Load,     MainFrame::OnLoad)
EVT_MENU    (ID_Info,     MainFrame::OnInfo)
EVT_MENU    (ID_MetaData, MainFrame::OnMetaData)
EVT_MENU    (ID_Exit,     MainFrame::OnExit)

EVT_MENU    (ID_KeyInfo,  MainFrame::OnKeyInfo)
EVT_MENU    (ID_About,    MainFrame::OnAbout)

EVT_BUTTON  (ID_Next,     MainFrame::OnNext)
EVT_BUTTON  (ID_Prev,     MainFrame::OnPrev)
EVT_BUTTON  (ID_RunStop,  MainFrame::OnRunStop)
EVT_SPINCTRL(ID_Delay,    MainFrame::OnDelay)
EVT_CHECKBOX(ID_Fit,      MainFrame::OnFit)

EVT_MENU    (ID_Next,     MainFrame::OnNext)
EVT_MENU    (ID_Prev,     MainFrame::OnPrev)
EVT_MENU    (ID_RunStop,  MainFrame::OnRunStop)

EVT_TIMER   (ID_Timer, MainFrame::OnTimer)
EVT_IDLE    (MainFrame::OnIdle)
EVT_SIZE    (MainFrame::OnSize)
END_EVENT_TABLE()


// ctor 
MainFrame::MainFrame(const wxString& title, const wxPoint& pos, const wxSize& size)
: wxFrame((wxFrame*)NULL, -1, title, pos, size)
,_ImageCanvasUC(NULL)
,_ImageCanvasFloat(NULL)
,_FitCheckBox(NULL)
{
  _CurrentImage     = 0;
  _DefaultFile      = wxT("ucim.pgm");
  _DefaultDir       = wxT("");
  _BlockAnimation   = true;

  // init timer
  _Timer.SetOwner(this, ID_Timer);

  // create file menu
  wxMenu* fileMenu = new wxMenu;
  fileMenu->Append(ID_Load, wxT("&Open/Load...\tCtrl-O"), wxT("Open a file from disk for loading"));
  fileMenu->Append(ID_Info, wxT("Image Info...\tCtrl-I"));
  fileMenu->Append(ID_MetaData, wxT("Meta Data...\tCtrl-M"));
  fileMenu->AppendSeparator();
  fileMenu->Append(ID_Exit, wxT("E&xit"), wxT("Exit this application"));

  // create help menu
  wxMenu* helpMenu = new wxMenu;
  helpMenu->Append(ID_KeyInfo, wxT("&Keys...\tF1"));
  helpMenu->Append(ID_About, wxT("&About...\tCtrl-F1"));

  // create menu bar
  wxMenuBar* menuBar = new wxMenuBar;
  menuBar->Append(fileMenu, wxT("&File"));
  menuBar->Append(helpMenu, wxT("&Help"));
  SetMenuBar(menuBar);

  // create status bar
  const int numFields = 2;
  CreateStatusBar(numFields);
  int widths[] = { -3, -1 }; // sizes of fields in array, negative means relative *variable* size JW
  SetStatusWidths(numFields, widths);
  SetStatusText(wxT("Welcome to BiasViewWX"), 0);

  // create tool bar
  wxToolBar* toolBar = CreateToolBar();
  toolBar->AddControl(new wxButton(toolBar, ID_Next, wxT("Next")));
  toolBar->AddControl(new wxButton(toolBar, ID_Prev, wxT("Previous")));
  _RunStopButton = new wxButton(toolBar, ID_RunStop, wxT("Run"));
  toolBar->AddControl(_RunStopButton);
  _ReverseCheckBox = new wxCheckBox(toolBar, -1, wxT("Reverse"));
  toolBar->AddControl(_ReverseCheckBox);
  toolBar->AddSeparator();

  _DelaySpin = new wxSpinCtrl(toolBar, ID_Delay, wxT("100"),
                              wxDefaultPosition, wxDefaultSize,
                              wxSP_ARROW_KEYS, 1, 200);
  toolBar->AddControl(_DelaySpin);

  toolBar->AddSeparator();
  _FitCheckBox = new wxCheckBox(toolBar, ID_Fit, wxT("Fit to window"));
  toolBar->AddControl(_FitCheckBox);
  toolBar->Realize();

  // set the shortcuts
  wxAcceleratorEntry entries[3];
  entries[0].Set(wxACCEL_NORMAL, WXK_RETURN, ID_Next);
  entries[1].Set(wxACCEL_NORMAL, WXK_BACK,   ID_Prev);
  entries[2].Set(wxACCEL_CTRL,   WXK_SPACE,  ID_RunStop);
  wxAcceleratorTable accel(3, entries);
  SetAcceleratorTable(accel);
}

void MainFrame::SetFileList(const vector<string>& fileNames)
{
  _FileNames = fileNames;
  if (_FileNames.size() > 0)
  {
    LoadImageFromList(0);
  }
}

void MainFrame::LoadImageFromList(int index)
{
  if (_FileNames.size() == 0)
  {
    return;
  }

  _CurrentImage = index;
  const string& fileName = _FileNames[index];

  // load image
  ImageBase image;
  if (ImageIO::Load(fileName.c_str(), image)!=0)
  {
    stringstream msg;
    msg << "Error loading " << fileName;
    BIASERR(msg.str());
    SetStatusText(AsciiToWx(msg.str()));
    return;
  }

  // save meta data for image info dialog
  stringstream info;
  switch (image.GetStorageType())
  {
  case ImageBase::ST_unsignedchar:
    {
      Camera<unsigned char> camera(image);
      camera.ParseMetaData();     
      camera.PrintAppData(info);
    } break;

  case ImageBase::ST_float:
    {
      Camera<float> camera(image);
      camera.ParseMetaData();
      camera.PrintAppData(info);
    } break;

  default:
    break;
  }
  _MetaData = info.str();

  // create appropriate image canvas based on storage type
  if (image.GetStorageType() == ImageBase::ST_unsignedchar)
  {
    // delete float image canvas
    if (_ImageCanvasFloat != NULL)
    {
      _ImageCanvasFloat->Destroy();
      _ImageCanvasFloat = NULL;
    }

    // create uc image canvas
    if (_ImageCanvasUC == NULL)
    {
      _ImageCanvasUC = new ImageCanvas(this, GetStatusBar(), 1);
    }

    // convert image
    Image<unsigned char> ucImage;
    ucImage.StealImage(image);
    ucImage.SetUID(image.GetUID());

    if (_FitCheckBox->IsChecked())
    {
      // get size available for image canvas
      int width, height;
      GetClientSize(&width, &height);

      // compute scale that keeps the aspect ratio and the image is fully visible
      float xscale = (float)width / image.GetWidth();
      float yscale = (float)height / image.GetHeight();
      float scale = xscale < yscale ? xscale : yscale;

      _ImageCanvasUC->SetScaledImageSize((int)(image.GetWidth() * scale), (int)(image.GetHeight() * scale));
      _ImageCanvasUC->SetShowScaled(true);
    }
    else
    {
      _ImageCanvasUC->SetShowScaled(false);
    }

    // show image in canvas
    _ImageCanvasUC->Show(ucImage, fileName);

    // don't show scrollbars if image is maximized
    if (_FitCheckBox->IsChecked())
    {
      _ImageCanvasUC->SetScrollbars(0, 0, 0, 0);
    }
  }
  else
  {
    // delete uc image canvas
    if (_ImageCanvasUC != NULL)
    {
      _ImageCanvasUC->Destroy();
      _ImageCanvasUC = NULL;
    }

    // create float image canvas
    if (_ImageCanvasFloat == NULL)
    {
      _ImageCanvasFloat = new ScaledImageCanvas(this, GetStatusBar(), 1);
    }

    // set appropriate scale and offset range
    switch (image.GetStorageType())
    {
    case ImageBase::ST_unsignedshortint:
      _ImageCanvasFloat->SetScaleRange(1.0/16.0, 1.0);
      _ImageCanvasFloat->SetOffsetRange(-4095.0, 0.0);
      break;

    case ImageBase::ST_char:
      _ImageCanvasFloat->SetScaleRange(0.5, 5.0);
      _ImageCanvasFloat->SetOffsetRange(-128.0, 128.0);
      break;

    case ImageBase::ST_float:
      _ImageCanvasFloat->SetScaleRange(0.5, 50.0);
      _ImageCanvasFloat->SetOffsetRange(-100.0, 100.0);
      break;

    default:
      cout << "Warning: Unknown storage type" << endl;
      // TODO: set default ranges? don't show image at all?
      break;
    }

    if (_FitCheckBox->IsChecked())
    {
      // get size available for image canvas
      int width, height;
      GetClientSize(&width, &height);
      width = max(1, width - 10); // little fix so the image is really fully visible
      height = max(1, height - 40); // larger fix because of sliders in float canvas

      // compute scale that keeps the aspect ratio and the image is fully visible
      float xscale = (float)width / image.GetWidth();
      float yscale = (float)height / image.GetHeight();
      float scale = xscale < yscale ? xscale : yscale;

      _ImageCanvasFloat->GetImageCanvas()->SetScaledImageSize((int)(image.GetWidth() * scale), (int)(image.GetHeight() * scale));
      _ImageCanvasFloat->GetImageCanvas()->SetShowScaled(true);
    }
    else
    {
      _ImageCanvasFloat->GetImageCanvas()->SetShowScaled(false);
    }

    // show image in canvas
    _ImageCanvasFloat->Show(image, fileName);

    // don't show scrollbars if image is maximized
    if (_FitCheckBox->IsChecked())
    {
      _ImageCanvasFloat->GetImageCanvas()->SetScrollbars(0, 0, 0, 0);
    }
  }

  stringstream msg;
  msg << "File '" << fileName << "' (" << image.GetWidth() << "x" << image.GetHeight() << ") loaded";
  SetStatusText(AsciiToWx(msg.str()));

  // make the shortcuts work without the need to click the canvas manually (on linux)
  if (_ImageCanvasUC) _ImageCanvasUC->SetFocus();
  if (_ImageCanvasFloat) _ImageCanvasFloat->SetFocus();

  this->RefitClients();
}


void MainFrame::LoadImages(const wxArrayString &paths)
{
  // add file names to internal std::string list
  bool ok=true;
  for (int i = 0; i < (int)paths.GetCount(); i++) {
    wxString extUP=wxFileName(paths[i]).GetExt().Upper();
    if (wxFileName(paths[i]).IsDir()) {
      ok=false;
    } else if ((extUP==AsciiToWx("LST")) || (extUP==AsciiToWx("TXT"))) {
      // special handling for .lst and .txt files. 
      // they are no images but contain a list of line separated image 
      //filenames
      vector<string> lstFilenames;
      int result=BIAS::Param::ParseListFile(WxToAscii(paths[i]), 
                                            lstFilenames);
      if (result!=0){
        BIASERR("could not load image list file "<<paths[i].c_str()
                <<" result="<<result<<endl);
      } else {
        // append filenames
        _FileNames.insert(_FileNames.end(), lstFilenames.begin(), 
                          lstFilenames.end() );
      }
    } else {
      _FileNames.push_back( string(WxToAscii(paths[i])) );
    }
  }

  // load first file and remember file name and directory
  if (!_FileNames.empty()) {
    //LoadImageFromList(_FileNames.size() - 1);
    //_DefaultFile = AsciiToWx(_FileNames[_FileNames.size() - 1]);
    LoadImageFromList(0);
    _DefaultFile = AsciiToWx(_FileNames[0]);
  }

  if (!ok) {
    wxMessageBox(wxT("Dropping directories is unsupported. Please drop only "
                     "filenames!"),
                 wxT("Drag&Drop Error"), wxOK | wxICON_EXCLAMATION);
  }
}

void MainFrame::FitSizetIfTooSmall()
{

  int w=0, h=0;
  int imgW=0, imgH=0;
  if (_ImageCanvasUC!=NULL){
    if (_ImageCanvasUC->GetOrigImage()==NULL) 
      return;
    imgW = _ImageCanvasUC->GetOrigImage()->GetWidth();
    imgH = _ImageCanvasUC->GetOrigImage()->GetHeight();
  } else if (_ImageCanvasFloat!=NULL){
    imgW = _ImageCanvasFloat->GetOrigimg().GetWidth();
    imgH = _ImageCanvasFloat->GetOrigimg().GetHeight();
  } else {
    // no valid canvas, nothing to do.
    return;
  }
  // get current frame size, TODO: query client area of canvas instead.
  this->GetClientSize(&w, &h);

  // query display screen dim
  int screenW=0, screenH=0;
  ::wxDisplaySize(&screenW, &screenH);
  if ( (w<imgW) || (h<imgH)) {
    // expand limited by screen size
    // TODO: adpat Client size, to avoid the FitInside...
    // problem: ImageCanves do not handle sizers correctly...
    // this is an approximation working for now: 
    this->SetSize( 
      0,0,
      // -1, -1, /* keep old pos. */
      min(imgW+40, screenW), min(imgH+100, screenH) );
  }
  this->RefitClients();
}


void MainFrame::RefitClients()
{
  // canvas needs to be refitted on windows after loading (not sure why)
#ifdef WIN32
  int w, h;
  GetClientSize(&w, &h);
  if (_ImageCanvasUC!=NULL)
    _ImageCanvasUC->SetSize(0, 0, w, h);
  if (_ImageCanvasFloat!=NULL)
    _ImageCanvasFloat->SetSize(0, 0, w, h);
#endif
}


void MainFrame::OnLoad(wxCommandEvent& /*event*/)
{
  wxFileDialog fileDialog(
    this,                          // parent
    wxT("Choose file(s) to load"), // message
    _DefaultDir,                   // default directory
    _DefaultFile,                  // default file
    wxT("*.*"),                    // wildcard
    wxOPEN | wxMULTIPLE);          // style

  if (fileDialog.ShowModal() == wxID_OK)
  {
    // get file names
    wxArrayString paths;
    fileDialog.GetPaths(paths);
    // load them
    LoadImages(paths);
  } else {
    SetStatusText(wxT("Loading canceled"));
    cout << "Canceled loading image" << endl;
  }

  _DefaultDir = fileDialog.GetDirectory();
}

void MainFrame::OnInfo(wxCommandEvent& /*event*/)
{
  // get currently displayed image
  ImageBase* image = NULL;
  if (_ImageCanvasUC != NULL)
  {
    image = _ImageCanvasUC->GetOrigImage();
  }
  else if (_ImageCanvasFloat != NULL)
  {
    image = &_ImageCanvasFloat->GetOrigimg();
  }
  if (image == NULL)
  {
    wxMessageBox(wxT("No image loaded"), wxT("Image Info"), wxOK | wxICON_ERROR, this);
    return;
  }

  // collect information
  stringstream info;
  info << "Width: " << image->GetWidth() << endl;
  info << "Height: " <<  image->GetHeight() << endl;
  info << "Depth: " << image->GetDepth() << endl;
  info << "BitDepth: " << image->GetBitDepth() << endl;
  info << "Size: " << image->GetSize() << " bytes" << endl;
  info << "Pixel Count: " << image->GetPixelCount() << endl;
  info << "Channels: " << image->GetChannelCount() << endl;
  info << "ColorModel: "  << image->GetColorModel()  << endl;
  info << "StorageType: " << image->GetStorageType() << endl;
  info << "IsPlanar: " << image->IsPlanar() << endl;
  info << "IsInterleaved: " << image->IsInterleaved() << endl;
  info << "UUID: " << image->GetUID() << endl;
  info << "Version: " << image->GetVersionNumber() << endl;

  // show information in message box
  wxMessageBox(AsciiToWx(info.str()), wxT("Image Info"), wxOK, this);
}

void MainFrame::OnMetaData(wxCommandEvent& /*event*/)
{
  wxFont font(10, wxMODERN, wxNORMAL, wxNORMAL, false, wxT("Courier"));
  wxDialog* dlg = new wxDialog(this, -1, wxT("Meta Data"), wxDefaultPosition, wxDefaultSize,
    wxDEFAULT_DIALOG_STYLE | wxRESIZE_BORDER);
  wxTextCtrl* text = new wxTextCtrl(dlg, -1, wxT(""),
    wxDefaultPosition, wxSize(650, 400),
    wxTE_MULTILINE | wxTE_READONLY | wxTE_DONTWRAP);
#ifdef WIN32
  text->SetFont(font);
#endif
  text->SetDefaultStyle(wxTextAttr(wxNullColour, wxNullColour, font));
  text->AppendText(AsciiToWx(_MetaData));
  text->SetInsertionPoint(0);
  dlg->Fit();
  dlg->ShowModal();
  dlg->Destroy();
}

void MainFrame::OnExit(wxCommandEvent& /*event*/)
{
  Destroy();
}

void MainFrame::OnKeyInfo(wxCommandEvent& /*event*/)
{
  wxString text =
    wxT("Next Image\tReturn\n")
    wxT("Prev Image\tBackspace\n")
    wxT("Run/Stop\tCtrl-Space");

  wxMessageBox(text, wxT("Keys"),
    wxOK | wxICON_INFORMATION, this);
}

void MainFrame::OnAbout(wxCommandEvent& /*event*/)
{
  stringstream msg;
  string version = "$Revision: 2.80 $";
  version = version.substr(9, version.length()-10);

  msg << "The BiasViewWX Image Viewer\n\n"
    << "http://www.mip.informatik.uni-kiel.de\n\n"
    << "build on " << __DATE__ << " " << __TIME__ << "\n"
    << "from file " << __FILE__ << "\n"
    << "with cvs file version " << version << "\n\n"
    << "GUI using: " << wxVERSION_STRING << "\n"
    << "\n";

  wxMessageBox(AsciiToWx(msg.str()), wxT("About biasviewwx"),
    wxOK | wxICON_INFORMATION, this);
}


void MainFrame::OnNext(wxCommandEvent& /*event*/)
{
  _CurrentImage++;
  if (_CurrentImage >= (int)_FileNames.size())
  {
    _CurrentImage = 0;
  }
  LoadImageFromList(_CurrentImage);
}

void MainFrame::OnPrev(wxCommandEvent& /*event*/)
{
  _CurrentImage--;
  if (_CurrentImage < 0)
  {
    _CurrentImage = _FileNames.size() - 1;
  }
  LoadImageFromList(_CurrentImage);
}

void MainFrame::OnRunStop(wxCommandEvent& /*event*/)
{
  if (_Timer.IsRunning())
  {
    _Timer.Stop();
    _RunStopButton->SetLabel(wxT("Run"));
  }
  else
  {
    _Timer.Start(_DelaySpin->GetValue());
    _RunStopButton->SetLabel(wxT("Stop"));
  }
}

void MainFrame::OnDelay(wxSpinEvent& /*event*/)
{
  if (_Timer.IsRunning())
  {
    _Timer.Start(_DelaySpin->GetValue());
  }
}

void MainFrame::OnFit(wxCommandEvent& /*event*/)
{
  LoadImageFromList(_CurrentImage);
}

void MainFrame::OnTimer(wxTimerEvent& /*event*/)
{
  if (!_BlockAnimation)
  {
    // block animation until at least one idle event occurs,
    // so the viewer doesn't get stuck if delay is too low.
    _BlockAnimation = true;
    wxCommandEvent dummyEvent;

    if (_ReverseCheckBox->GetValue())
      OnPrev(dummyEvent);
    else
      OnNext(dummyEvent);
  }
}

void MainFrame::OnIdle(wxIdleEvent& /*event*/)
{
  _BlockAnimation = false;
}

void MainFrame::OnSize(wxSizeEvent& event)
{
  if (_FitCheckBox != NULL)
  {
    if (_FitCheckBox->IsChecked())
    {
      LoadImageFromList(_CurrentImage);
    }
  }
  event.Skip();
}

/*
===============================================================================
DragAndDropTarget
===============================================================================
*/

/** @brief drag and drop files onto application.
drag a file from MS explorer or kde file manager directly onto an application
(instaed of using a load from filename function.) 
@author Jan Woetzel 03/2005 */
class DragAndDropTarget : public wxFileDropTarget
{
public:
  DragAndDropTarget(MainFrame* frame);

  virtual bool OnDropFiles(wxCoord x, wxCoord y, const wxArrayString& filenames);
protected:
  MainFrame* _Frame;
};

DragAndDropTarget::DragAndDropTarget(MainFrame* frame)
{
  _Frame = frame;
}

bool DragAndDropTarget::OnDropFiles(wxCoord /*x*/, wxCoord /*y*/, const wxArrayString& filenames)
{
  BIASASSERT( _Frame != NULL);
  wxArrayString sortedFilenames(filenames);
  // sort alphabetically because file seq is not preserved "as expected"
  // (at least on Windows XP Explorer)
  sortedFilenames.Sort();
  _Frame->LoadImages(sortedFilenames);
  _Frame->FitSizetIfTooSmall();

  return true;
}

/*
===============================================================================
BiasViewApp
===============================================================================
*/

/** @class biasviewwx
    @ingroup g_tools
    @brief tool for displaying images using wxWidgets GUI.
    @author many - including woelk,evers,jw,... */
class BiasViewApp : public wxApp
{
public:
  virtual bool OnInit();
};

bool BiasViewApp::OnInit()
{
  // utility: create a Redirection console for cout, cerr etc.
  ConsoleRedirectIO();

  MainFrame* frame = 
    new MainFrame(wxT("BiasViewWX"), wxPoint(50, 50), FRAME_DEFAULT_SIZE);

  // append file list given on command line
  {
    wxArrayString cmdFiles;
    for (int i = 1; i < argc; i++){
      cmdFiles.Add(wxString(argv[i]));
    }
    frame->LoadImages(cmdFiles);
  }

  // adapt Frame size on first loaded image
  frame->FitSizetIfTooSmall();

  // create drag and drop target
  DragAndDropTarget* dndTarget = new DragAndDropTarget(frame);
  frame->SetDropTarget(dndTarget);

  // show frame
  frame->Show(TRUE);
  SetTopWindow(frame);

  return TRUE;
}

IMPLEMENT_APP(BiasViewApp)

