#include <Utils/TriangleMesh.hh>
#include <Utils/ThreeDOut.hh>
#include <Base/Image/ImageIO.hh>

#include <iostream>

using namespace std;
using namespace BIAS;

void usage(char* name) 
{
  cout<<"usage:\n";
  cout<<name<<" <depthmap> <pmatrix> <texture> [outputFile]\n";
}

int main(int argc, char* argv[]) 
{
  if(argc<3) {
    usage(argv[0]);
    return -1;
  }

  Image<float> depthMap;
  if(ImageIO::Load(argv[1], depthMap)!=0) {
    cout<<"depth map could not be loaded!\n";
    usage(argv[0]);
    return -1;
  }
  
  Image<unsigned char> texture;
  if(ImageIO::Load(argv[3], texture)!=0) {
    cout<<"texture could not be loaded!\n";
    usage(argv[0]);
    return -1;
  }

  PMatrix P;
  if(!P.Load(argv[2])) {
    cout<<"PMatrix could not be loaded!\n";
    usage(argv[0]);
    return -1;
  }

  cout<<"meshing..."; cout.flush();
  TriangleMesh mesh;
  
  mesh.GenerateDenseMesh(depthMap, P, texture);  
  cout<<"finished\n";

  ThreeDOut vrmlOut;
  vrmlOut.AddTriangleMesh(mesh, "mesh_from_"+string(argv[1]), 
                          argv[3], false);

  cout<<"Writing VRML to "; 
  if(argc>4) {
    cout<<argv[4]<<"... ";cout.flush();
    vrmlOut.VRMLOut(argv[4]);
  } 
  else {
    cout<<"DenseTriangleMesh.wrl... ";cout.flush();
    vrmlOut.VRMLOut("DenseTriangleMesh.wrl");
  }
  cout<<"finished\n";
  
  return 0;
}
