/* 
This file is part of the BIAS library (Basic ImageAlgorithmS).

Copyright (C) 2003, 2004    (see file CONTACTS for details)
  Multimediale Systeme der Informationsverarbeitung
  Institut fuer Informatik
  Christian-Albrechts-Universitaet Kiel


BIAS is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or
(at your option) any later version.

BIAS is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with BIAS; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


#ifndef _TRIANGLEMESH_HH_
#define _TRIANGLEMESH_HH_

#include <vector>

#include <Base/Image/Image.hh>
#include <Geometry/PMatrix.hh>
  
namespace BIAS {

  /** \class TriangleMesh 
   * \author bartczak
   * \brief create and represent a 3D triangle mesh
   */
  class BIASUtils_EXPORT TriangleMesh {
  public:
    ~TriangleMesh() {};
    

    /** Calculate a triangle mesh from dense depth map.
     * No triangles are discarded, hence no smoothing.
     */
    void GenerateDenseMesh(const BIAS::Image<float>& DenseDepthMap,
			   BIAS::PMatrix& P,
			   const BIAS::Image<unsigned char>& Texture);

    /** @brief Calculate a triangle mesh only using the given cutout

        recalculates depthmap, image and p and calls generic interface
        @author koeser  */
    void GenerateDenseMesh(const BIAS::Image<float>& DenseDepthMap,
                           BIAS::PMatrix& P,
                           const BIAS::Image<unsigned char>& Texture,
                           const ROI& cutout);

    void GetTexture(Image<unsigned char>& img) const {
      img = texture_;
    } 

    void GetVertices(std::vector<BIAS::Vector3<double> >& vertices) const {
      vertices = meshVertices_;
    } 

    void GetTexCoords(std::vector<BIAS::Vector2<float> >& texCoo) const {
      texCoo = textureCoords_;
    } 

    void GetTriangleIndices(std::vector<BIAS::Vector<int> >& triangleIndices) const {
      triangleIndices = triangleIndices_;
    } 

    void GetNormals(std::vector<BIAS::Vector3<double> >& normals) const 
    {
      normals = vertexNormals_;
    }

  protected:

    // exports for dll (cmenk)
    BIAS_EXPORT_STL_VECTOR( BIASUtils_EXPORT, BIAS::Vector3<double> )
    BIAS_EXPORT_STL_VECTOR( BIASUtils_EXPORT, BIAS::Vector2<float> )
    BIAS_EXPORT_STL_VECTOR( BIASUtils_EXPORT, BIAS::Vector<int> )

    BIAS::Image<unsigned char> texture_;
    std::vector<BIAS::Vector3<double> > meshVertices_;
    std::vector<BIAS::Vector3<double> > vertexNormals_;
    std::vector<BIAS::Vector2<float> > textureCoords_;
    std::vector<BIAS::Vector<int> > triangleIndices_;   
    
  };  //end class

}//end namespace


#endif
