#include "../MDcam.hh"
#include <MIPGui/MIPGui.hh>
#include <MIPParam/Param.hh>

int main(int argc, char *argv[])
{
  
  Param params;



  MDcam cams;
  int numcams;
  bool done = false;
  int downsample;
  char c = 0;
  Camera<unsigned char> *GrabImage;
  Camera<unsigned char> pic2;
  MIPGui *win;
  int res;
  
 
  params.AddParameter(MIPPARAM_INT,"downsample","Show downsampled by 2^n images",'d');
  
  params.AddParameter(MIPPARAM_STRING,"cameras","Configuration file (default: cameras.conf",'c');
  params.AddParameter(MIPPARAM_INT,"scanbus","Only scan the bus number N and write camerafile");
    
  params.SetParameter("downsample",0);    
  params.SetParameter("scanbus",-1);    


  res = params.ParseCommandLine(argc, argv);
  if (res <0) exit(1);
  


    // Init
  string camfile;
  params.GetParameter("cameras",(string)camfile);
  if (camfile == NULL)
    camfile = "cameras.conf";
  int scanbus;
  params.GetParameter("scanbus",scanbus);
  if (scanbus >=0) {
    ofstream ofs(camfile);
    if (!ofs) {
      perror(camfile);
      exit(1);
    }
    cams.ScanBus(scanbus,ofs);
    ofs.close();
    exit(0);
  }
    
  params.GetParameter("downsample",downsample);
  

  cams.SetDebugLevel(D_MDCAM_INIT );
  
  numcams = cams.InitCameras(camfile);
  if (numcams <0) exit(0);
  
  win = new MIPGui[numcams];
  GrabImage = new Camera<unsigned char>[numcams];
  for (int i=0; i<numcams; i++) {
    char title[128];
    sprintf(title,"Camera: %d",cams.GetID(i));
    win[i].SetTitle(title);
    cams.GetCamera(i)->InitImage(GrabImage[i]);
    
  }
  
  

  cams.PreGrab();
  
  
  // mainloop with show

  while (!done) {
//      gettimeofday(&now, NULL);
//      end = start;
//      start =  (now.tv_sec % 1000) * 1000 + (now.tv_usec / 1000);
//      fps = 1.0 / (float)(start-end) * 1000.0 ;
    
    c =0;
    
      cams.GrabSingle(GrabImage); 
      for (int i=0; i<numcams; i++) {
	char tmp;
	if (downsample >0)
	  pic2.GetDownsampledCopy(GrabImage[i],downsample);
	else
	  pic2.ToGrey(GrabImage[i]);
	win[i].ShowImage(pic2);
	tmp = win[i].CheckForKeyEvent();
	if (tmp >0) c = tmp;
      } 


      switch (c) {
      case 27: 
	done = true;
	break;
      }
      
  }


  cams.PostGrab();
  

  cams.ReleaseCameras();
  

  delete[] win;
  

  return 0;
}
