/* 
This file is part of the BIAS library (Basic ImageAlgorithmS).

Copyright (C) 2003, 2004    (see file CONTACTS for details)
  Multimediale Systeme der Informationsverarbeitung
  Institut fuer Informatik
  Christian-Albrechts-Universitaet Kiel


BIAS is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or
(at your option) any later version.

BIAS is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with BIAS; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifndef __MDCAM_HH__
#define __MDCAM_HH__

#include <bias_config.h>

#ifndef BIAS_HAVE_DCAM
#  error You need DCAM for BIAS Videosource_DCAM. Please recompile BIAS with USE_DCAM.
#endif // BIAS_HAVE_DCAM

#ifdef WIN32
#  error BIAS MDcam is unsupporte don WIN32, currently. Please fix the code/buildsystem.
#endif // WIN32

#include <string>

#include <VideoSource/VideoSource_Base.hh>
#include <VideoSource/VideoSource_DCAM.hh>
#include <Base/Debug/Debug.hh>
#include <Base/Debug/Error.hh>

#define D_MDCAM_INIT (1<<0)
#define D_MDCAM_GRAB (1<<2)



namespace BIAS{
  /**
     \class MDcam
     \group VideoG
     \author evers
     \brief MDcam simplifies acces to more then one DCam.
     It suports multiple IEEE1394-adapters, extern synchronization via 
     parallel port and networked capture cluster.
  */
  class MDcam:public BIAS::Debug 
  {
  public:
    /*! Activates the timestamp for PointGrey Scorpion SCOR-20SO
     */
    void ActivateScorpionTimestamp();

    MDcam();
    ~MDcam();

    void SetDebugLevelDcam(int d);
  
    int  ScanBus(int bus,std::ofstream &ofs);
  
    int InitCameras(const std::string filename);
    int ReleaseCameras();
  
    /// Call GrabSingle() with a prepared array of images to be filled
    /** The size of the array MUST be the numaber of cameras
     */
    int GrabSingle(BIAS::Camera<unsigned char> *Image);
    int PreGrab();
  
    int PostGrab();

    int SetFeature( int feature, unsigned int value);
    //    int GetFeature( int feature);
    int AutoMode( int feature, bool on);

    int OnePushAuto(int feature);

    int SetShutter(float exptime);
    float GetShutter()
    { return Camera_[0]->GetShutter(); }
    int SetGain(float g);
    float GetGain()
    { return Camera_[0]->GetGain(); }

    inline void SetFps(float fps){Fps_ = fps;};


    /** if on==true && active=true try to open parport at ParPort_, 
	if it fails, return -1
	else set cameras to external trigger mode 
	if on==false set cameras to internal trigger mode
	if active==true est cameras into external trigger mode but
	 ignore parport
    */  
    int ExternalTrigger(bool on, bool active);

    /// Read @param feature from @param master and distributed it to all
    int EqualizeFeature(int master, int feature);
  

    int SetCamera(CameraParameterSet camparam);
  
    int GetNumCams(){return NumCams_;};

    inline VideoSource_DCAM *GetCamera(int n) { return Camera_[n];};
    inline int GetID(int i) { 
      if (i <NumCams_ ) return Camera_[i]->GetID();
      else
	return -1;

    };
  



  protected:
    float Fps_;
    int DebuglevelDcam_;
  

    VideoSource_DCAM **Camera_;
    int NumCams_;
    bool ExternalTrigger_;
    bool ActiveTrigger_;
  
    // Address of parport for external trigger
    unsigned long ParPort_;
  

   
 

  
  };
} // namespace MIP

#endif
