#include "VideoSource_DV.hh"
#include <stdio.h>

using namespace MIP;
using namespace BIAS;
using namespace std;

int VideoSource_DV::InstancesTotal_ = 0;
int VideoSource_DV::Instances_[MAX_1394_CARDS];
raw1394handle_t VideoSource_DV::IEEE1394Handle_[MAX_1394_CARDS];

VideoSource_DV *videosourcefromchannel[MAX_1394_CHANNELS];



VideoSource_DV::VideoSource_DV(int Card): VideoSource()
{
  VideoSource_DV_Internal_Init_(Card);
}


VideoSource_DV::VideoSource_DV(): VideoSource()
{
  VideoSource_DV_Internal_Init_(0);
}



void VideoSource_DV::VideoSource_DV_Internal_Init_(int Card)
{
  Width_ = MIP_DV_WIDTH;
  Height_ = MIP_DV_HEIGHT;
  ColorChannels_ = 1;
  ColorMode_ = ImageBase::CM_DV;
  MaxTrials_ = 2;
  Channel_ = 63;

  //SetDebugLevel(D_DV);
  
  IEEE1394Card_ = Card;
  

  if (IEEE1394Card_ > MAX_1394_CARDS) {
    BIASERR("VideoSource_DV::VideoSource_DV(int Card): requested card #"<<IEEE1394Card_<< " exceeds maximum of "<<MAX_1394_CARDS);
    return ; 
  }

  if (InstancesTotal_ == 0) {
    BIASDOUT(D_DV,"Initializing Instances_[] and IEEE1394Handle_[]");
    for (int i=0; i<MAX_1394_CARDS; i++) {
      Instances_[i] = 0;
      IEEE1394Handle_[i] = NULL;
    }
    for (int i=0; i<MAX_1394_CHANNELS; i++)
      videosourcefromchannel[i] = NULL;
  } 

  InstancesTotal_++;
  IEEE1394Card_ = Card;
  
  if ( Instances_[IEEE1394Card_] == 0) {
    // I am the first instance on this Card, so open now
    BIASDOUT(D_DV,"IEEE1394Card_: "<<IEEE1394Card_);
    IEEE1394Handle_[IEEE1394Card_] = raw1394_new_handle();
    if (IEEE1394Handle_[IEEE1394Card_] == NULL) {
      perror("raw1394 - couldn't get handle");
      BIASERR("This error usually means that the ieee1394 drivers are not loaded or that /dev/raw1394  not exist.\n");
      return ;
    }

    if (raw1394_set_port(IEEE1394Handle_[IEEE1394Card_],IEEE1394Card_) < 0) 
                BIASERR("raw1394 - couldn't set port");
  }
 
 

  Instances_[IEEE1394Card_] ++;
  videosourcefromchannel[Channel_] = this;

}

VideoSource_DV::~VideoSource_DV()
{
  InstancesTotal_ --;
  Instances_[IEEE1394Card_] --;
  if (Instances_[IEEE1394Card_] == 0) {
    // I am the last instance on this card, so close the door 
    raw1394_destroy_handle(IEEE1394Handle_[IEEE1394Card_]);
    BIASDOUT(D_DV,"I am the last Instance on this bus, so shutting the lights.");
  } 
  videosourcefromchannel[Channel_] = NULL;

      
}



int VideoSource_DV::OpenDevice()
{
  return 0;
}


  
int VideoSource_DV::OpenDevice(const char *device)
{
  return 0;
}




int VideoSource_DV::CloseDevice()
{
  return 0;
}


  
int VideoSource_DV::UseChannel(int channel)
{
  Channel_ = channel;
  return 0;
}


int VideoSource_DV::GrabSingle(Camera <unsigned char> &image)
{ 
  int trials = 0;
  bool done = false;
  struct timeval time;

  gettimeofday(&time,NULL);
  image.SetTime(time.tv_sec,time.tv_usec);

  GrabBuffer_ = image.GetImageData();
  



  while (trials < MaxTrials_ && !done) {
    FrameFinished_ = false;
    FrameStarted_ = false;
    DroppedFrame_ = false;

   while (!FrameFinished_) 
     raw1394_loop_iterate(IEEE1394Handle_[IEEE1394Card_]);
    

   trials ++;
   done = !DroppedFrame_;
  }
  
  image.InvalidateUID();

  if (DroppedFrame_)
    return -1;
  else
  return 0;
}


int VideoSource_DV::PreGrab()
{
  iso_handler_t	g_oldhandler;
  BIASDOUT(D_DV,"PreGrab(), channel: "<< Channel_);
  g_oldhandler = raw1394_set_iso_handler(IEEE1394Handle_[IEEE1394Card_], 
					 Channel_, raw_dv_handler);
  
   if (raw1394_start_iso_rcv(IEEE1394Handle_[IEEE1394Card_], Channel_) < 0) {
   BIASERR("raw1394 - couldn't start iso receive");
    return -1;
  }

//    raw1394_set_bus_reset_handler(handle, reset_handler);  

  return 0;
  
}


int VideoSource_DV::PostGrab()
{
  BIASDOUT(D_DV,"PostGrab()");

   raw1394_stop_iso_rcv(IEEE1394Handle_[IEEE1394Card_], Channel_);
 return 0;
}




int MIP::raw_dv_handler(raw1394handle_t handle, int channel, size_t length,
                    quadlet_t *data)
{
  static int packetcount = 0;
//    cout << "raw_dv_tomem_handler()"<<endl;


  if (length > 16) {
    unsigned char *p = (unsigned char*) & data[3];
    int section_type = p[0] >> 5;          /* section type is in bits 5 - 7 */
    int dif_sequence = p[1] >> 4;    /* dif sequence number is in bits 4 - 7 */
    int dif_block = p[2];

    // detect begin of frame
    if (section_type == 0 && dif_sequence == 0) {
      if (!videosourcefromchannel[channel]->FrameFinished_)
	if (videosourcefromchannel[channel]->FrameStarted_)
	  videosourcefromchannel[channel]->DroppedFrame_ = true;
	else
	  videosourcefromchannel[channel]->FrameStarted_ = true;
    }

    
    

    if (videosourcefromchannel[channel]->FrameStarted_ && 
	!videosourcefromchannel[channel]->FrameFinished_) {
      packetcount++;
      switch (section_type) {
      case 0:           // 1 Header block
	memcpy(videosourcefromchannel[channel]->GrabBuffer_
	       + dif_sequence * 150 * 80, p, 480);
	break;
	
      case 1:           // 2 Subcode blocks
	memcpy(videosourcefromchannel[channel]->GrabBuffer_
	       + dif_sequence * 150 * 80 + (1 + dif_block) * 80,p, 480);
	break;
	
      case 2:           // 3 VAUX blocks
	memcpy(videosourcefromchannel[channel]->GrabBuffer_+
	       dif_sequence * 150 * 80 + (3 + dif_block) * 80, p, 480);
	break;
	
      case 3:           // 9 Audio blocks interleaved with video
	memcpy(videosourcefromchannel[channel]->GrabBuffer_
	       + dif_sequence * 150 * 80 + (6 + dif_block * 16) * 80,p, 480);
	break;
	
      case 4:           // 135 Video blocks interleaved with audio
	memcpy(videosourcefromchannel[channel]->GrabBuffer_+
	       dif_sequence * 150 * 80 + (7 + (dif_block / 15) + dif_block) 
	       * 80, p, 480);
	break;
	
      default:           // we cant handle any other data
	break;
      }
      //   cout << packetcount << " " ;
    }
    
    
    if (packetcount == 300 && !videosourcefromchannel[channel]->FrameFinished_) {
        videosourcefromchannel[channel]->FrameFinished_ = true;
	packetcount = 0;
    }

    
  }
  return 0;
}







