#include "VideoSource_DV2.hh"
#include <stdio.h>
#include <fcntl.h>
#include <sys/ioctl.h>
#include <unistd.h>
#include <sys/mman.h>

using namespace MIP;
using namespace BIAS;
using namespace std;

VideoSource_DV2::VideoSource_DV2(): VideoSource()
{
  Width_ = MIP_DV_WIDTH;
  Height_ = MIP_DV_HEIGHT;
  ColorChannels_ = 1;
  ColorMode_ = ImageBase::CM_DV;
  DeviceChannel_ = 0x63;

  SetDebugLevel(D_DV2);
  DV1394FD_ = 0;
  
  FrameSize_ = 0;
  Format_ = DV1394_PAL;
  Buffer_ = NULL;
  DefaultDevice_ = "/dev/dv1394";
  
  
}

VideoSource_DV2::~VideoSource_DV2()
{

      
}



int VideoSource_DV2::OpenDevice()
{
  return OpenDevice(DefaultDevice_);
}


  
int VideoSource_DV2::OpenDevice(const string &device)
{
  DV1394FD_ = open(device.c_str(),O_RDONLY);
  if (DV1394FD_ < 0) {
    BIASERR("Can not open file: "<<device);
    perror(device.c_str());
    return -1;
  }
  
  InitData_.channel = DeviceChannel_;
  // size of ringbuffer */
  InitData_.n_frames = 10;
  InitData_.format = Format_;
  InitData_.cip_n = 0;
  InitData_.cip_d = 0;
  InitData_.syt_offset = 0;

  
  switch (Format_) {
  case DV1394_PAL:
    FrameSize_ = DV1394_PAL_FRAME_SIZE;
    break;
  case DV1394_NTSC:
    FrameSize_ = DV1394_NTSC_FRAME_SIZE;
    break;
  }
  ioctl(DV1394FD_,DV1394_INIT, InitData_);
  BufferSize_ = FrameSize_ * InitData_.n_frames;
  
//     Buffer_ = (unsigned char *)mmap(0, BufferSize_, PROT_READ, MAP_SHARED, DV1394FD_, 0);
//     if (Buffer_ <0) {
//       perror("Error mmap() :");
//       return -1;
//     }
//     cout <<(void *)Buffer_<<endl;
   
  

  CompleteInitialized_ = true;
  
  return 0;
}




int VideoSource_DV2::CloseDevice()
{
  munmap(Buffer_, BufferSize_);
  
  close(DV1394FD_);
  CompleteInitialized_ = false;
  
  return 0;
}


  
int VideoSource_DV2::UseChannel(int channel)
{
  DeviceChannel_ = channel;
  return 0;
}


int VideoSource_DV2::GrabSingle(Camera <unsigned char> &image)
{ 
  struct timeval time;
  int got;
  int ReceiveNFrames = 1;
  int NewBuffers = 0;
  
  
//   cout <<"awaiting image"<<endl;

  ioctl(DV1394FD_, DV1394_RECEIVE_FRAMES,ReceiveNFrames);

  got = ioctl(DV1394FD_, DV1394_GET_STATUS, & Status_);
  NewBuffers = (Status_.first_clear_frame+ReceiveNFrames)% InitData_.n_frames;
  BIASDOUT_CLEAN(D_DV2,"First clear buffer: "<<Status_.first_clear_frame)
  BIASDOUT_CLEAN(D_DV2,"N buffers: "<<NewBuffers);
  
  if (NewBuffers >0) {
    gettimeofday(&time,NULL);
    image.SetTime(time.tv_sec,time.tv_usec);

    read(DV1394FD_, image.GetImageData(), FrameSize_);
    image.InvalidateUID();
  return 0;
  }
  else return -1;
  
}


int VideoSource_DV2::PreGrab()
{
  
  if (VideoSource::PreGrab() <0 )
    return -1;
  ioctl(DV1394FD_,DV1394_START_RECEIVE,NULL);
  
  return 0;
  
}


int VideoSource_DV2::PostGrab()
{
 
 
  if (VideoSource::PostGrab() <0 )
    return -1;

  ioctl(DV1394FD_,DV1394_SHUTDOWN, NULL);

 return 0;
}











