/* 
This file is part of the BIAS library (Basic ImageAlgorithmS).

Copyright (C) 2003, 2004    (see file CONTACTS for details)
  Multimediale Systeme der Informationsverarbeitung
  Institut fuer Informatik
  Christian-Albrechts-Universitaet Kiel


BIAS is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or
(at your option) any later version.

BIAS is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with BIAS; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#ifndef __MIPVIDEOSOURCE_SHM_HH__
#define __MIPVIDEOSOURCE_SHM_HH__



#include <iostream>
#include <sstream>
#include <VideoSource/VideoSource_Base.hh>

#ifdef WIN32
#  define WIN32_LEAN_AND_MEAN		// Exclude rarely-used stuff from Windows headers
#  include <windows.h>
#endif


namespace BIAS{



  /**
     \class VideoSource_Shm
     \ingroup VideoG
     \author evers
     \brief This class implements a video streaming client using shared memory to receive images from another application.
  */
  class BIASVideoSource_EXPORT VideoSource_Shm : public VideoSource {

  public: 
    struct ShmImageHeader {
      unsigned int Size_;
      double Time_;
    };

    /**
       @author Jan-Friso Evers
    **/
    VideoSource_Shm();
    ~VideoSource_Shm(){ delete pImgBuffer_; }

    int OpenDevice(const char *name);

    int OpenDevice(); 

    int PreGrab();
    int GrabSingle(BIAS::Camera <unsigned char> &image);
    int PostGrab();

    void SetSize(int w, int h, int bytesperpixel);
    int CloseDevice();

  protected:
    int Resize_(int newsize);
#ifdef WIN32
    HANDLE Context_; // _hFile
    HANDLE Mapping_;//! header describing the memory layout
    long int Granularity_;
#else
    int Context_;
#endif

    bool IsCreator_;
    std::string FileName_;
    void *Shm_;
    unsigned int DataLength_;
    unsigned int TotalLength_;
    std::stringstream *pImgBuffer_; // pointer needed to free mem
    ShmImageHeader *Header_;
    char *Data_;
    double LastTime_;

 protected:
    /*!\brief prohibit default function (move to 'public' if needed) */
  VideoSource_Shm(const VideoSource_Shm &source);
  /*!\brief prohibit default function (move to 'public' if needed) */
  VideoSource_Shm& operator =(const VideoSource_Shm &source);

 
  };


} // namespace MIP

#endif
